/*******************************************************************************
* File Name: can.c
* Version 1.50
*
* Description:
*  The CAN Component provide functionality of Control Area Network.
*  Two types of mailbox configurations available "Full" and "Basic".
*
* Note:
*  CAN configuration puts as constant to ROM and could change only directly
*  registers write by user.
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "can.h"

uint8 can_initVar = 0u;

/* Iniital values of CAN RX and TX registers */
const can_RX_CFG CYCODE can_RXConfigStruct[] =
{
    { 0u, 0x0u, 0x0u, 0x0u },
    { 1u, 0x0u, 0x0u, 0x0u },
    { 2u, 0x0u, 0x0u, 0x0u },
    { 3u, 0x0u, 0x0u, 0x0u },
    { 4u, 0x0u, 0x0u, 0x0u },
    { 5u, 0x0u, 0x0u, 0x0u },
    { 6u, 0x0u, 0x0u, 0x0u },
    { 7u, 0x0u, 0x0u, 0x0u },
    { 8u, 0x0u, 0x0u, 0x0u },
    { 9u, 0x0u, 0x0u, 0x0u },
    { 10u, 0x0u, 0x0u, 0x0u },
    { 11u, 0x0u, 0x0u, 0x0u },
    { 12u, 0x0u, 0x0u, 0x0u },
    { 13u, 0x0u, 0x0u, 0x0u },
    { 14u, 0x0u, 0x0u, 0x0u },
    { 15u, 0x0u, 0x0u, 0x0u }
};

const can_TX_CFG CYCODE can_TXConfigStruct[] =
{
    { 0u, 0x880008u, 0x0u },
    { 1u, 0x880008u, 0x0u },
    { 2u, 0x880008u, 0x0u },
    { 3u, 0x880008u, 0x0u },
    { 4u, 0x880008u, 0x0u },
    { 5u, 0x880008u, 0x0u },
    { 6u, 0x880008u, 0x0u },
    { 7u, 0x880008u, 0x0u }
};


/*******************************************************************************
* FUNCTION NAME:   can_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default CAN configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the configuration has been accepted or rejected.
*
* Side Effects:
*  All registers will be reset to their initial values. This will re-initialize 
*  the component with the following exceptions. It will not clear data from 
*  the mailboxes.
*  Enable power to the CAN Core.
*
*******************************************************************************/
uint8 can_Init(void) 
{
    uint8 i;
    uint8 result = can_FAIL;
    uint8 localResult = CYRET_SUCCESS;    
    
    /* Disable Interrupt. */
    CyIntDisable(can_ISR_NUMBER);

    /* Set the ISR to point to the can_ISR Interrupt. */
    CyIntSetVector(can_ISR_NUMBER, can_ISR);

    /* Set the priority. */
    CyIntSetPriority(can_ISR_NUMBER, can_ISR_PRIORITY);    

    /* Enable CAN block in Active mode */
    can_PM_ACT_CFG_REG |= can_ACT_PWR_EN;
    
    /* Enable CAN block in Alternate Active (Standby) mode */
    can_PM_STBY_CFG_REG |= can_STBY_PWR_EN;    

    if (can_SetPreScaler(can_BITRATE) == CYRET_SUCCESS)
    {
        if (can_SetArbiter(can_ARBITER) == CYRET_SUCCESS)
        {
            if (can_SetTsegSample(can_CFG_REG_TSEG1, can_CFG_REG_TSEG2,
                can_CFG_REG_SJW, can_SAMPLING_MODE) == CYRET_SUCCESS)
            {
                if (can_SetRestartType(can_RESET_TYPE) == CYRET_SUCCESS)
                {
                    if (can_SetEdgeMode(can_SYNC_EDGE) == CYRET_SUCCESS)
                    {
                        /* Initialize TX maiboxes */
                        for(i = 0u; i < can_NUMBER_OF_TX_MAILBOXES; i++)
                        {
                            if (can_TxBufConfig((can_TX_CFG *)
                                (&can_TXConfigStruct[i])) != CYRET_SUCCESS)
                            {
                                localResult = can_FAIL;
                                break;
                            }
                        }

                        if (localResult == CYRET_SUCCESS)
                        {
                            /* Initialize RX mailboxes */
                            for(i = 0u; i < can_NUMBER_OF_RX_MAILBOXES; i++)
                            {
                                if (can_RxBufConfig((can_RX_CFG *)
                                    (&can_RXConfigStruct[i])) != CYRET_SUCCESS)
                                {
                                    localResult = can_FAIL;
                                    break;
                                }
                            }

                            if (localResult == CYRET_SUCCESS)
                            {
                                /* Write IRQ Mask */
                                if (can_SetIrqMask(can_INIT_INTERRUPT_MASK) ==
                                    CYRET_SUCCESS)
                                {
                                    /* Active mode always */
                                    if (can_SetOpMode(can_ACTIVE_MODE) == CYRET_SUCCESS)
                                    {
                                        result = CYRET_SUCCESS;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }    

    return(result);
}


/*******************************************************************************
* Function Name: can_Enable
********************************************************************************
*
* Summary:   
*  This function enable CAN Component and ISR of CAN Component. 
*
* Parameters:  
*  None.
*
* Return: 
*  Indication whether register is written and verified..
*
*******************************************************************************/
uint8 can_Enable(void) 
{
    uint8 result = can_FAIL;
    uint8 enableInterrupts = 0u;
    
    enableInterrupts = CyEnterCriticalSection();
    
    /* Enable CAN block in Active mode */
    can_PM_ACT_CFG_REG |= can_ACT_PWR_EN;

    /* Enable CAN block in Alternate Active (Standby) mode */
    can_PM_STBY_CFG_REG |= can_STBY_PWR_EN;
    
    CyExitCriticalSection(enableInterrupts);
    
    /* Enable isr */
    CyIntEnable(can_ISR_NUMBER);
    
    /* Sets the CAN controller to run mode */
    can_CMD_REG.byte[0u] |= can_MODE_MASK;
    
    /* Verify that bit is set */
    if ((can_CMD_REG.byte[0u] & can_MODE_MASK) == can_MODE_MASK)
    {
        result = CYRET_SUCCESS;
    }
    
    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_Start
********************************************************************************
*
* Summary:
*  This function sets CAN Component into Run mode. Starts the Rate Counter if
*  polling mailboxes available.
*
* Parameters:
*  None. 
*
* Return:
*  Indication whether register is written and verified.  
*
* Global variables:
*  can_initVar - used to check initial configuration, modified on
*  first function call.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 can_Start(void)
{
    uint8 result = can_FAIL;
    
    if (can_initVar == 0u)
    {
        result = can_Init();
        can_initVar = 1u;        
    }
    
    if (result == CYRET_SUCCESS)
    {
        result = can_Enable();
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_Stop
********************************************************************************
*
* Summary:
*  This function sets CAN Component into Stop mode.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*
* Side Effects:
*  Disable power to the CAN Core.
*
*******************************************************************************/
uint8 can_Stop(void) 
{
    uint8 result = can_FAIL;
    uint8 enableInterrupts = 0u;        
    
    /* Sets the CAN controller to stop mode */
    can_CMD_REG.byte[0u] &= ~can_MODE_MASK;

    /* Verify that bit is cleared */
    if ((can_CMD_REG.byte[0u] & can_MODE_MASK) == 0u)
    {
        result = CYRET_SUCCESS;
    }
    
    /* Disable interrupt */
    CyIntDisable(can_ISR_NUMBER); 
    
    enableInterrupts = CyEnterCriticalSection(); 

    /* Disable CAN block in Active mode */
    can_PM_ACT_CFG_REG &= ~can_ACT_PWR_EN;

    /* Disable CAN block in Alternate Active (Standby) mode template */
    can_PM_STBY_CFG_REG &= ~can_STBY_PWR_EN;
    
    CyExitCriticalSection(enableInterrupts);

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_GlobalIntEnable
********************************************************************************
*
* Summary:
*  This function enables Global Interrupts from CAN Core.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_GlobalIntEnable(void) 
{
    uint8 result = can_FAIL;

    can_INT_EN_REG.byte[0u] |= can_GLOBAL_INT_MASK;

    /* Verify that bit is set */
    if ((can_INT_EN_REG.byte[0u] & can_GLOBAL_INT_MASK) == can_GLOBAL_INT_MASK)
    {
        result = CYRET_SUCCESS;
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_GlobalIntDisable
********************************************************************************
*
* Summary:
*  This function disables Global Interrupts from CAN Core.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_GlobalIntDisable(void) 
{
    uint8 result = can_FAIL;

    can_INT_EN_REG.byte[0u] &= ~can_GLOBAL_INT_MASK;

    /* Verify that bit is cleared */
    if ((can_INT_EN_REG.byte[0u] & can_GLOBAL_INT_MASK) == 0u)
    {
        result = CYRET_SUCCESS;
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_SetPreScaler
********************************************************************************
*
* Summary:
*  This function sets PreScaler for generation the time quantum which defines
*  the time quanta. Value between 0x0 and 0x7FFF are valid.
*
* Parameters:
*  bitrate: PreScaler value.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_SetPreScaler(uint16 bitrate) 
{
    uint8 result = can_OUT_OF_RANGE;

    if (bitrate <= can_BITRATE_MASK)
    {
        result = can_FAIL;

        /* Set prescaler */
        CY_SET_REG16((reg16 *) &can_CFG_REG.byte[2u], bitrate);

        /* Verify that prescaler is set */
        if (CY_GET_REG16((reg16 *) &can_CFG_REG.byte[2u]) == bitrate)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_SetArbiter
********************************************************************************
*
* Summary:
*  This function sets arbitration type for transmit mailboxes. Types of
*  arbiters are Round Robin and Fixed priority. Value 0 and 1 are valid.
*
* Parameters:
*  arbiter: Arbitaration type for transmit mailboxes.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_SetArbiter(uint8 arbiter) 
{
    uint8 result = can_FAIL;

    if (arbiter == can_ROUND_ROBIN)
    {
        can_CFG_REG.byte[1u] &= ~can_ARBITRATION_MASK;    /* Round Robin */

        /* Verify that bit is cleared */
        if ((can_CFG_REG.byte[1u] & can_ARBITRATION_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }        
    }
    else
    {
        can_CFG_REG.byte[1u] |= can_ARBITRATION_MASK;    /* Fixed priority */

        /* Verify that bit is set */
        if ((can_CFG_REG.byte[1u] & can_ARBITRATION_MASK) ==
            can_ARBITRATION_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME: can_SetTsegSample
********************************************************************************
*
* Summary:
*  This function configures: Time segment 1, Time segment 2, Sampling Mode
*  and Synchronization Jump Width.
*
* Parameters:
*  cfgTseg1: Time segment 1, value between 0x2 and 0xF are valid;
*  cfgTseg2: Time segment 2, value between 0x1 and 0x7 are valid;
*  sjw: Synchronization Jump Width, value between 0x0 and 0x3 are valid;
*  sm: Sampling Mode, one or three sampling points are used.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_SetTsegSample(uint8 cfgTseg1, uint8 cfgTseg2, uint8 sjw, uint8 sm)
                                     
{
    uint8 result = can_OUT_OF_RANGE;
    uint8 cfgTemp;

    if ((cfgTseg1 >= can_CFG_REG_TSEG1_LOWER_LIMIT) && (cfgTseg1 <=
         can_CFG_REG_TSEG1_UPPER_LIMIT))
    {
        if (((cfgTseg2 >= can_CFG_REG_TSEG2_LOWER_LIMIT) &&
             (cfgTseg2 <= can_CFG_REG_TSEG2_UPPER_LIMIT)) || ((sm == can_ONE_SAMPLE_POINT) &&
             (cfgTseg2 == can_CFG_REG_TSEG2_EXCEPTION)))
        {
            if ((sjw <= can_CFG_REG_SJW_LOWER_LIMIT) && (sjw <= cfgTseg1) && (sjw <= cfgTseg2))
            {
                result = can_FAIL;

                cfgTemp = can_CFG_REG.byte[1];
                cfgTemp &= ~can_CFG_REG_TSEG1_MASK;
                cfgTemp |= cfgTseg1;

                /* write register byte 1 */
                can_CFG_REG.byte[1u] = cfgTemp;

                /* Verify 2nd byte of can_CFG_REG register */
                if (can_CFG_REG.byte[1u] == cfgTemp)
                {
                    /* read and clean bits */
                    cfgTemp = can_CFG_REG.byte[0u];
                    cfgTemp &= ~(can_CFG_REG_TSEG2_MASK | can_CFG_REG_SJW_MASK |
                                  can_SAMPLE_MODE_MASK);

                    cfgTemp = 0u;
                    /* Set appropriate bits */
                    if (sm != can_ONE_SAMPLE_POINT)
                    {
                        cfgTemp |= can_SAMPLE_MODE_MASK;
                    }
                    cfgTemp |= ((cfgTseg2 << can_CFG_REG_TSEG2_SHIFT) |
                                 (sjw << can_CFG_REG_SJW_SHIFT));

                    /* write register byte 0 */
                    can_CFG_REG.byte[0u] = cfgTemp;

                    /* Verify 1st byte of can_CFG_REG register */
                    if (can_CFG_REG.byte[0u] == cfgTemp)
                    {
                        result = CYRET_SUCCESS;
                    }
                }
            }
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_SetRestartType
********************************************************************************
*
* Summary:
*  This function sets Reset type. Types of Reset are Automatic and Manual.
*  Manual Reset is recommended setting. Value 0 and 1 are valid.
*
* Parameters:
*  reset: Reset Type.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_SetRestartType(uint8 reset) 
{
    uint8 result = can_FAIL;

    if (reset == can_RESTART_BY_HAND)
    {
        can_CFG_REG.byte[0u] &= ~can_RESET_MASK;   /* Manual reset */

        /* Verify that bit is cleared */
        if ((can_CFG_REG.byte[0u] & can_RESET_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }
    }
    else
    {
        can_CFG_REG.byte[0u] |= can_RESET_MASK;    /* Automatic restart */

        /* Verify that bit is set */
        if ((can_CFG_REG.byte[0u] & can_RESET_MASK) == can_RESET_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_SetEdgeMode
********************************************************************************
*
* Summary:
*  This function sets Edge Mode. Modes are 'R' to 'D'(Reccessive to Dominant)
*  and Both edges are used. Value 0 and 1 are valid.
*
* Parameters:
*  edge: Edge Mode.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_SetEdgeMode(uint8 edge) 
{
    uint8 result = can_FAIL;

    if (edge == can_EDGE_R_TO_D)
    {
        /* Recessive to Dominant is used for synchronization */
        can_CFG_REG.byte[0u] &= ~can_EDGE_MODE_MASK;

        /* Verify that bit is cleared */
        if ((can_CFG_REG.byte[0u] & can_EDGE_MODE_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }                      
    }
    else
    {
        /* Both edges are to be used */
        can_CFG_REG.byte[0u] |= can_EDGE_MODE_MASK;

        /* Verify that bit is set */
        if ((can_CFG_REG.byte[0u] & can_EDGE_MODE_MASK) == can_EDGE_MODE_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_SetOpMode
********************************************************************************
*
* Summary:
*  This function sets Operation Mode. Operations Modes are Active of Listen
*  Only. Value 0 and 1 are valid.
*
* Parameters:
*  opMode: Operation Mode value.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_SetOpMode(uint8 opMode) 
{
    uint8 result = can_FAIL;

    if (opMode == can_ACTIVE_MODE)
    {
        /* CAN in Active mode */
        can_CMD_REG.byte[0u] &= ~can_OPMODE_MASK;

        /* Verify that bit is cleared */
        if ((can_CMD_REG.byte[0u] & can_OPMODE_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }                
    }
    else
    {
        /* CAN listen only */
        can_CMD_REG.byte[0u] |= can_OPMODE_MASK;

        /* Verify that bit is set */
        if ((can_CMD_REG.byte[0u] & can_OPMODE_MASK) == can_OPMODE_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_RXRegisterInit
********************************************************************************
*
* Summary:
*  This function writes only receive CAN registers.
*
* Parameters:
*  regAddr: Pointer to CAN receive register;
*  config:  Value that will be written in register.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_RXRegisterInit(reg32 *regAddr, uint32 config)
                                      
{
    uint8 result = can_OUT_OF_RANGE;    
    
    if ( (((uint32) regAddr & 0x0000FFFFu) >= ((uint32)can_RX_FIRST_REGISTER_PTR & 0x0000FFFFu)) &&
         ((((uint32) regAddr & 0x0000FFFFu)) <= ((uint32)can_RX_LAST_REGISTER_PTR & 0x0000FFFFu)) )
    {
        result = can_FAIL;
        
        if ((((uint32) regAddr & 0x0000FFFFu) % can_RX_CMD_REG_WIDTH) == 0u)
        {
            config |= can_RX_WPN_SET;

            /* Disable Interrupt. */
            CyIntDisable(can_ISR_NUMBER);
        
            /* Write defined RX CMD registers */
            CY_SET_REG32(regAddr, config);
            
            /* Enable isr */
            CyIntEnable(can_ISR_NUMBER);


            /* Verify register */
            if ( (CY_GET_REG32(regAddr) & can_RX_READ_BACK_MASK) ==
                (config & can_RX_READ_BACK_MASK) )
            {
                result = CYRET_SUCCESS;
            }
        }
        /* All registers except RX CMD*/
        else
        {            
            /* Disable Interrupt. */
            CyIntDisable(can_ISR_NUMBER);
            
            /* Write defined CAN receive register */
            CY_SET_REG32(regAddr, config);
            
            /* Enable isr */
            CyIntEnable(can_ISR_NUMBER);

            /* Verify register */
            if (CY_GET_REG32(regAddr) == config)
            {
                result = CYRET_SUCCESS;
            }
        }
    }
    
    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   can_SetIrqMask
********************************************************************************
*
* Summary:
*  This function sets to enable/disable particular interrupt sources. Interrupt
*  Mask directly write to CAN Interrupt Enable register.
*
* Parameters:
*  mask: Interrupt enable/disable request. 1 bit per interrupt source.
*
* Return:
*  Indication whether register is written and verified.
*
*******************************************************************************/
uint8 can_SetIrqMask(uint16 mask) 
{
    uint8 result = can_FAIL;

    /* Write byte 0 and 1 of can_INT_EN_REG register */
    CY_SET_REG16((reg16 *) &can_INT_EN_REG, mask);

    /* Verify can_INT_EN_REG register */
    if (CY_GET_REG16((reg16 *) &can_INT_EN_REG) == mask)
    {
        result = CYRET_SUCCESS;
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_GetTXErrorflag
********************************************************************************
*
* Summary:
*  This function returns the bit that indicates if the number of TX errors
*  exceeds 0x60.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the number of TX errors exceeds 0x60.
*
*******************************************************************************/
uint8 can_GetTXErrorFlag(void) 
{
    /* Get the state of the transmit error flag */
    return(((can_ERR_SR_REG.byte[2u] & can_TX_ERROR_FLAG_MASK) ==
             can_TX_ERROR_FLAG_MASK) ? 1u : 0u);
}


/*******************************************************************************
* FUNCTION NAME:   can_GetRXErrorFlag
********************************************************************************
*
* Summary:
*  This function returns the bit that indicates if the number of RX errors
*  exceeds 0x60.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the number of TX errors exceeds 0x60.
*
*******************************************************************************/
uint8 can_GetRXErrorFlag(void) 
{
    /* Get the state of the receive error flag */
    return(((can_ERR_SR_REG.byte[2u] & can_RX_ERROR_FLAG_MASK) ==
             can_RX_ERROR_FLAG_MASK) ? 1u : 0u);
}


/*******************************************************************************
* FUNCTION NAME:   can_TXErrorCount
********************************************************************************
*
* Summary:
*  This function returns the number of Transmit Errors.
*
* Parameters:
*  None.
*
* Return:
*  Number of Transmit Errors.
*
*******************************************************************************/
uint8 can_GetTXErrorCount(void) 
{
    /* Get the state of the transmit error count */
    return(can_ERR_SR_REG.byte[0u]);    /* bits 7-0 */
}


/*******************************************************************************
* FUNCTION NAME:   can_RXErrorCount
********************************************************************************
*
* Summary:
*  This function returns the number of Receive Errors.
*
* Parameters:
*  None.
*
* Return:
*  Number of Receive Errors.
*
*******************************************************************************/
uint8 can_GetRXErrorCount(void) 
{
    /* Get the state of the receive error count */
    return(can_ERR_SR_REG.byte[1u]);    /* bits 15-8 */
}


/*******************************************************************************
* FUNCTION NAME:   can_GetErrorState
********************************************************************************
*
* Summary:
*  This function returns error status of CAN Component.
*
* Parameters:
*  None.
*
* Return:
*  Error status.
*
*******************************************************************************/
uint8 can_GetErrorState(void) 
{
    /* Get the error state of the receiver */
    return(can_ERR_SR_REG.byte[2u] & can_ERROR_STATE_MASK);
}


/*******************************************************************************
* FUNCTION NAME:   can_RxBufConfig
********************************************************************************
*
* Summary:
*  This function configures all receive registers for particular mailbox.
*
* Parameters:
*  rxConfig: Pointer to structure that contain all required values to configure
*  all receive registers for particular mailbox.
*
* Return:
*  Indication if particular configuration has been accepted or rejected.
*
*******************************************************************************/
uint8 can_RxBufConfig(can_RX_CFG *rxConfig) 
{
    uint8 result = can_FAIL;

    /* Write RX CMD Register */
    CY_SET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxcmd, (rxConfig->rxcmd |
                 can_RX_WPN_SET));
    if ((CY_GET_REG32((reg32 *) & can_RX[rxConfig->rxmailbox].rxcmd) &
         can_RX_READ_BACK_MASK) == (rxConfig->rxcmd & can_RX_WPN_CLEAR))
    {
        /* Write RX AMR Register */
        CY_SET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxamr, rxConfig->rxamr);
        if (CY_GET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxamr) == rxConfig->rxamr)
        {
            /* Write RX ACR Register */
            CY_SET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxacr, rxConfig->rxacr);
            if (CY_GET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxacr) == rxConfig->rxacr)
            {
                /* Write RX AMRD Register */
                CY_SET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxamrd, 0xFFFFFFFFu);
                if (CY_GET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxamrd) == 0xFFFFFFFFu)
                {
                    /* Write RX ACRD Register */
                    CY_SET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxacrd, 0x00000000u);
                    if (CY_GET_REG32((reg32 *) &can_RX[rxConfig->rxmailbox].rxacrd) == 0x00000000u)
                    {
                            result = CYRET_SUCCESS;
                    }
                }
            }
        }
    }

    return(result);
}


/*******************************************************************************
* FUNCTION NAME:   can_TxBufConfig
********************************************************************************
*
* Summary:
*  This function configures all transmit registers for particular mailbox.
*  Mailbox number contains can_TX_CFG structure.
*
* Parameters:
*  txConfig: Pointer to structure that contain all required values to configure
*  all transmit registers for particular mailbox.
*
* Return:
*  Indication if particular configuration has been accepted or rejected.
*
*******************************************************************************/
uint8 can_TxBufConfig(can_TX_CFG *txConfig) 
{
    uint8 result = can_FAIL;

    /* Write TX CMD Register */
    CY_SET_REG32((reg32 *) &can_TX[txConfig->txmailbox].txcmd, (txConfig->txcmd |
                 can_TX_WPN_SET));
    if ((CY_GET_REG32((reg32 *) & can_TX[txConfig->txmailbox].txcmd) &
        can_TX_READ_BACK_MASK) == (txConfig->txcmd & can_TX_WPN_CLEAR))
    {
        /* Write TX ID Register */
        CY_SET_REG32( (reg32 *) &can_TX[txConfig->txmailbox].txid, txConfig->txid);
        if (CY_GET_REG32( (reg32 *) &can_TX[txConfig->txmailbox].txid) == txConfig->txid)
        {
            result = CYRET_SUCCESS;
        }
    }

    return(result);
}


/* [] END OF FILE */
