/*******************************************************************************
* File Name: can_PM.c
* Version 1.50
*
* Description:
*  This file contains the setup, control and status commands to support 
*  component operations in low power mode.  
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "can.h"

static can_BACKUP_STRUCT can_backup = { 0u,
                                                                  0u,
                                                                  can_INIT_INTERRUPT_MASK,
                                                                  can_MODE_MASK };


/*******************************************************************************
* Function Name: can_SaveConfig
********************************************************************************
*
* Summary:
*  Save CAN configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
* 
* Global Variables:
*  can_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void can_SaveConfig(void)
{
    can_backup.intSr = (CY_GET_REG32(can_INT_SR_PTR));
    can_backup.intEn = (CY_GET_REG32(can_INT_EN_PTR));
    can_backup.cmd = (CY_GET_REG32(can_CMD_PTR));
    can_backup.cfg = (CY_GET_REG32(can_CFG_PTR));
}


/*******************************************************************************
* Function Name: can_RestoreConfig
********************************************************************************
*
* Summary:
*  Restore CAN configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  can_backup - used when non-retention registers are restored.
*
* Side Effects:
*  If this API is called without first calling SaveConfig then in the following
*  registers will be default values from Customizer: can_INT_SR, 
*  can_INT_EN, can_CMD, can_CFG.
*
*******************************************************************************/
void can_RestoreConfig(void) 
{
    CY_SET_REG32(can_INT_SR_PTR, can_backup.intSr);
    CY_SET_REG32(can_INT_EN_PTR, can_backup.intEn);
    CY_SET_REG32(can_CMD_PTR, can_backup.cmd);
    CY_SET_REG32(can_CFG_PTR, can_backup.cfg);
}


/*******************************************************************************
* Function Name: can_Sleep
********************************************************************************
* 
* Summary:
*  Prepare CAN Component goes to sleep.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  can_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void can_Sleep(void)
{   
    if((can_CMD_REG.byte[0u] & can_MODE_MASK) == can_MODE_MASK)       
    {
        can_backup.enableState = 1u;
    }
    else /* The CAN block is disabled */
    {
        can_backup.enableState = 0u;
    }
    
    can_SaveConfig();
    can_Stop();    
}


/*******************************************************************************
* Function Name: can_Wakeup
********************************************************************************
* 
* Summary:
*  Prepare CAN Component to wake up.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  can_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void can_Wakeup(void) 
{           
    /* Enable power to CAN */
    can_PM_ACT_CFG_REG |= can_ACT_PWR_EN;
    
    can_RestoreConfig();
    
    if(can_backup.enableState != 0u)
    {
        /* Enable component's operation */
        can_Enable();
    } /* Do nothing if component's block was disabled before */
}


/* [] END OF FILE */
