/*******************************************************************************
* File Name: debug_uart.c
* Version 2.0
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/


#include "debug_uart.h"
#include "CyLib.h"


/***************************************
* Global data allocation
***************************************/

#if( debug_uart_TX_ENABLED && (debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH))
    volatile uint8 debug_uart_txBuffer[debug_uart_TXBUFFERSIZE];
    volatile uint8 debug_uart_txBufferRead = 0u;
    uint8 debug_uart_txBufferWrite = 0u;
#endif /* End debug_uart_TX_ENABLED */
#if( ( debug_uart_RX_ENABLED || debug_uart_HD_ENABLED ) && \
     (debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH) )
    volatile uint8 debug_uart_rxBuffer[debug_uart_RXBUFFERSIZE];
    volatile uint8 debug_uart_rxBufferRead = 0u;
    volatile uint8 debug_uart_rxBufferWrite = 0u;
    volatile uint8 debug_uart_rxBufferLoopDetect = 0u;
    volatile uint8 debug_uart_rxBufferOverflow = 0u;
#endif /* End debug_uart_RX_ENABLED */


/***************************************
* Local data allocation
***************************************/

uint8 debug_uart_initVar = 0u;


/*******************************************************************************
* Function Name: debug_uart_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the UART component.
*  Enable the clock input to enable operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The debug_uart_intiVar variable is used to indicate initial 
*  configuration of this component. The variable is initialized to zero (0u) 
*  and set to one (1u) the first time UART_Start() is called. This allows for 
*  component initialization without re-initialization in all subsequent calls 
*  to the debug_uart_Start() routine. 
*
* Reentrant:
*  No.
*
*******************************************************************************/
void debug_uart_Start(void)
{
    /* If not Initialized then initialize all required hardware and software */
    if(debug_uart_initVar == 0u)
    {
        debug_uart_Init();
        debug_uart_initVar = 1u;
    }
    debug_uart_Enable();
}


/*******************************************************************************
* Function Name: debug_uart_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  debug_uart_Start().
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void debug_uart_Init(void) 
{
    #if(debug_uart_RX_ENABLED || debug_uart_HD_ENABLED)

        #if(debug_uart_RX_INTERRUPT_ENABLED && (debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH))
            /* Set the RX Interrupt. */
            CyIntSetVector(debug_uart_RX_VECT_NUM,   debug_uart_RXISR);
            CyIntSetPriority(debug_uart_RX_VECT_NUM, debug_uart_RX_PRIOR_NUM);
        #endif /* End debug_uart_RX_INTERRUPT_ENABLED */

        #if (debug_uart_RXHW_ADDRESS_ENABLED)
            debug_uart_SetRxAddressMode(debug_uart_RXAddressMode);
            debug_uart_SetRxAddress1(debug_uart_RXHWADDRESS1);
            debug_uart_SetRxAddress2(debug_uart_RXHWADDRESS2);
        #endif /* End debug_uart_RXHW_ADDRESS_ENABLED */

        /* Configure the Initial RX interrupt mask */
        debug_uart_RXSTATUS_MASK_REG  = debug_uart_INIT_RX_INTERRUPTS_MASK;
    #endif /* End debug_uart_RX_ENABLED || debug_uart_HD_ENABLED*/

    #if(debug_uart_TX_ENABLED)
        #if(debug_uart_TX_INTERRUPT_ENABLED && (debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH))
            /* Set the TX Interrupt. */
            CyIntSetVector(debug_uart_TX_VECT_NUM,   debug_uart_TXISR);
            CyIntSetPriority(debug_uart_TX_VECT_NUM, debug_uart_TX_PRIOR_NUM);
        #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if(debug_uart_TXCLKGEN_DP)
            debug_uart_TXBITCLKGEN_CTR_REG = debug_uart_BIT_CENTER;
            debug_uart_TXBITCLKTX_COMPLETE_REG = (debug_uart_NUMBER_OF_DATA_BITS + \
                                                    debug_uart_NUMBER_OF_START_BIT) * \
                                                    debug_uart_OVER_SAMPLE_COUNT;
        #else
            debug_uart_TXBITCTR_COUNTER_REG = (debug_uart_NUMBER_OF_DATA_BITS + \
                                                    debug_uart_NUMBER_OF_START_BIT) * \
                                                    debug_uart_OVER_SAMPLE_8;
        #endif /* End debug_uart_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if(debug_uart_TX_INTERRUPT_ENABLED && (debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH))
            debug_uart_TXSTATUS_MASK_REG = debug_uart_TX_STS_FIFO_EMPTY;
        #else
            debug_uart_TXSTATUS_MASK_REG = debug_uart_INIT_TX_INTERRUPTS_MASK;
        #endif /*End debug_uart_TX_INTERRUPT_ENABLED*/
        
    #endif /* End debug_uart_TX_ENABLED */

    #if(debug_uart_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        debug_uart_WriteControlRegister( \
            (debug_uart_ReadControlRegister() & ~debug_uart_CTRL_PARITY_TYPE_MASK) | \
            (debug_uart_PARITY_TYPE << debug_uart_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End debug_uart_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: debug_uart_Enable
********************************************************************************
*
* Summary:
*  Enables the UART block operation
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void debug_uart_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();
    
    #if(debug_uart_RX_ENABLED || debug_uart_HD_ENABLED)
        /*RX Counter (Count7) Enable */
        debug_uart_RXBITCTR_CONTROL_REG |= debug_uart_CNTR_ENABLE;
        /* Enable the RX Interrupt. */
        debug_uart_RXSTATUS_ACTL_REG  |= debug_uart_INT_ENABLE;
        #if(debug_uart_RX_INTERRUPT_ENABLED && (debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH))
            CyIntEnable(debug_uart_RX_VECT_NUM);
        #endif /* End debug_uart_RX_INTERRUPT_ENABLED */
    #endif /* End debug_uart_RX_ENABLED || debug_uart_HD_ENABLED*/

    #if(debug_uart_TX_ENABLED)
        /*TX Counter (DP/Count7) Enable */
        #if(!debug_uart_TXCLKGEN_DP)
            debug_uart_TXBITCTR_CONTROL_REG |= debug_uart_CNTR_ENABLE;
        #endif /* End debug_uart_TXCLKGEN_DP */
        /* Enable the TX Interrupt. */
        debug_uart_TXSTATUS_ACTL_REG |= debug_uart_INT_ENABLE;
        #if(debug_uart_TX_INTERRUPT_ENABLED && (debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH))
            CyIntEnable(debug_uart_TX_VECT_NUM);
        #endif /* End debug_uart_TX_INTERRUPT_ENABLED*/
     #endif /* End debug_uart_TX_ENABLED */

    #if(debug_uart_INTERNAL_CLOCK_USED)
        /* Set the bit to enable the clock. */
        debug_uart_INTCLOCK_CLKEN_REG |= debug_uart_INTCLOCK_CLKEN_MASK;
    #endif /* End debug_uart_INTERNAL_CLOCK_USED */
    
    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: debug_uart_Stop
********************************************************************************
*
* Summary:
*  Disable the UART component
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void debug_uart_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /*Write Bit Counter Disable */
    #if(debug_uart_RX_ENABLED || debug_uart_HD_ENABLED)
        debug_uart_RXBITCTR_CONTROL_REG &= ~debug_uart_CNTR_ENABLE;
    #endif /* End debug_uart_RX_ENABLED */

    #if(debug_uart_TX_ENABLED)
        #if(!debug_uart_TXCLKGEN_DP)
            debug_uart_TXBITCTR_CONTROL_REG &= ~debug_uart_CNTR_ENABLE;
        #endif /* End debug_uart_TXCLKGEN_DP */
    #endif /* debug_uart_TX_ENABLED */

    #if(debug_uart_INTERNAL_CLOCK_USED)
        /* Clear the bit to enable the clock. */
        debug_uart_INTCLOCK_CLKEN_REG &= ~debug_uart_INTCLOCK_CLKEN_MASK;
    #endif /* End debug_uart_INTERNAL_CLOCK_USED */
    
    /*Disable internal interrupt component*/
    #if(debug_uart_RX_ENABLED || debug_uart_HD_ENABLED)
        debug_uart_RXSTATUS_ACTL_REG  &= ~debug_uart_INT_ENABLE;
        #if(debug_uart_RX_INTERRUPT_ENABLED && (debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH))
            debug_uart_DisableRxInt();
        #endif /* End debug_uart_RX_INTERRUPT_ENABLED */
    #endif /* End debug_uart_RX_ENABLED */
    
    #if(debug_uart_TX_ENABLED)
        debug_uart_TXSTATUS_ACTL_REG &= ~debug_uart_INT_ENABLE;
        #if(debug_uart_TX_INTERRUPT_ENABLED && (debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH))
            debug_uart_DisableTxInt();
        #endif /* End debug_uart_TX_INTERRUPT_ENABLED */
    #endif /* End debug_uart_TX_ENABLED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: debug_uart_ReadControlRegister
********************************************************************************
*
* Summary:
*  Read the current state of the control register
*
* Parameters:
*  None.
*
* Return:
*  Current state of the control register.
*
*******************************************************************************/
uint8 debug_uart_ReadControlRegister(void) 
{
    #if( debug_uart_CONTROL_REG_REMOVED )
        return(0u);
    #else
        return(debug_uart_CONTROL_REG);
    #endif /* End debug_uart_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: debug_uart_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  debug_uart_WriteControlRegister(uint8 control) 
{
    #if( debug_uart_CONTROL_REG_REMOVED )
        control = control;      /* Reassigning to release compiler warning */ 
    #else
       debug_uart_CONTROL_REG = control;
    #endif /* End debug_uart_CONTROL_REG_REMOVED */
}


#if(debug_uart_RX_ENABLED || debug_uart_HD_ENABLED)

    #if(debug_uart_RX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: debug_uart_EnableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void debug_uart_EnableRxInt(void) 
        {
            CyIntEnable(debug_uart_RX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: debug_uart_DisableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void debug_uart_DisableRxInt(void) 
        {
            CyIntDisable(debug_uart_RX_VECT_NUM);
        }

    #endif /* debug_uart_RX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: debug_uart_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  IntSrc:  An or'd combination of the desired status bit masks (defined in
    *           the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void debug_uart_SetRxInterruptMode(uint8 intSrc) 
    {
        debug_uart_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: debug_uart_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns data in RX Data register without checking status register to 
    *  determine if data is valid
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  debug_uart_rxBuffer - RAM buffer pointer for save received data.
    *  debug_uart_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     checked to identify new data. 
    *  debug_uart_rxBufferRead - cyclic index for read from rxBuffer, 
    *     incremented after each byte has been read from buffer.
    *  debug_uart_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR. 
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 debug_uart_ReadRxData(void)
    {
        uint8 rxData;

        #if(debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_DisableRxInt();
            #endif /* End debug_uart_RX_INTERRUPT_ENABLED */

            if( (debug_uart_rxBufferRead != debug_uart_rxBufferWrite) || \
                (debug_uart_rxBufferLoopDetect > 0u) )
            {
                if(debug_uart_rxBufferLoopDetect > 0u ) 
                {
                    debug_uart_rxBufferLoopDetect = 0u;
                }

                if(debug_uart_rxBufferRead >= debug_uart_RXBUFFERSIZE)
                {
                    debug_uart_rxBufferRead = 0u;
                }

                rxData = debug_uart_rxBuffer[debug_uart_rxBufferRead];

                debug_uart_rxBufferRead++;
            }
            else
            {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
                rxData = debug_uart_RXDATA_REG;
            }

            /* Enable Rx interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_EnableRxInt();
            #endif /* End debug_uart_RX_INTERRUPT_ENABLED */

        #else /* debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH */

            /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
            rxData = debug_uart_RXDATA_REG;

        #endif /* debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: debug_uart_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the current state of the status register
    *  And detect software buffer overflow.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Global Variables:
    *  debug_uart_rxBufferOverflow - used to indicate overload condition. 
    *   It set to one in RX interrupt when there isn?t free space in 
    *   debug_uart_rxBufferRead to write new data. This condition returned 
    *   and cleared to zero by this API as an 
    *   debug_uart_RX_STS_SOFT_BUFF_OVER bit along with RX Status register 
    *   bits.
    *
    *******************************************************************************/
    uint8 debug_uart_ReadRxStatus(void) 
    {
        uint8 status;

        status = debug_uart_RXSTATUS_REG;
        status &= debug_uart_RX_HW_MASK;

        #if(debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH)
            if( debug_uart_rxBufferOverflow )
            {
                status |= debug_uart_RX_STS_SOFT_BUFF_OVER;
                debug_uart_rxBufferOverflow = 0u;
            }
        #endif /* debug_uart_RXBUFFERSIZE */

        return(status);
    }


    /*******************************************************************************
    * Function Name: debug_uart_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, if data is not available or an error 
    *  condition exists, zero is returned; otherwise, character is read and 
    *  returned.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  debug_uart_rxBuffer - RAM buffer pointer for save received data.
    *  debug_uart_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     checked to identify new data. 
    *  debug_uart_rxBufferRead - cyclic index for read from rxBuffer, 
    *     incremented after each byte has been read from buffer.
    *  debug_uart_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR. 
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 debug_uart_GetChar(void)
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

        #if(debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_DisableRxInt();
            #endif /* debug_uart_RX_INTERRUPT_ENABLED */

            if( (debug_uart_rxBufferRead != debug_uart_rxBufferWrite) || \
                (debug_uart_rxBufferLoopDetect > 0u) )
            {
                if(debug_uart_rxBufferLoopDetect > 0u ) 
                {
                    debug_uart_rxBufferLoopDetect = 0u;
                }

                if(debug_uart_rxBufferRead >= debug_uart_RXBUFFERSIZE)
                {
                    debug_uart_rxBufferRead = 0u;
                }

                rxData = debug_uart_rxBuffer[debug_uart_rxBufferRead];

                debug_uart_rxBufferRead++;
            }
            else
            {   rxStatus =debug_uart_RXSTATUS_REG;
                if(rxStatus & debug_uart_RX_STS_FIFO_NOTEMPTY)
                {   /* Read received data from FIFO*/
                    rxData = debug_uart_RXDATA_REG;
                    /*Check status on error*/
                    if(rxStatus & (debug_uart_RX_STS_BREAK | debug_uart_RX_STS_PAR_ERROR | \
                                   debug_uart_RX_STS_STOP_ERROR | debug_uart_RX_STS_OVERRUN))
                    {
                        rxData = 0u;
                    }    
                }
            }

            /* Enable Rx interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_EnableRxInt();
            #endif /* debug_uart_RX_INTERRUPT_ENABLED */

        #else /* debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH */

            rxStatus =debug_uart_RXSTATUS_REG;
            if(rxStatus & debug_uart_RX_STS_FIFO_NOTEMPTY)
            {   /* Read received data from FIFO*/
                rxData = debug_uart_RXDATA_REG;
                /*Check status on error*/
                if(rxStatus & (debug_uart_RX_STS_BREAK | debug_uart_RX_STS_PAR_ERROR | \
                               debug_uart_RX_STS_STOP_ERROR | debug_uart_RX_STS_OVERRUN))
                {
                    rxData = 0u;
                }
            }
        #endif /* debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: debug_uart_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Grab the next available byte of data from the recieve FIFO
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains Status Register and LSB contains UART RX data
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 debug_uart_GetByte(void)
    {
        return ( ((uint16)debug_uart_ReadRxStatus() << 8u) | debug_uart_ReadRxData() );
    }


    /*******************************************************************************
    * Function Name: debug_uart_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of bytes left in the RX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Integer count of the number of bytes left 
    *  in the RX buffer
    *
    * Global Variables:
    *  debug_uart_rxBufferWrite - used to calculate left bytes. 
    *  debug_uart_rxBufferRead - used to calculate left bytes.
    *  debug_uart_rxBufferLoopDetect - checked to decide left bytes amount. 
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 debug_uart_GetRxBufferSize(void)
    {
        uint8 size;

        #if(debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_DisableRxInt();
            #endif /* debug_uart_RX_INTERRUPT_ENABLED */

            if(debug_uart_rxBufferRead == debug_uart_rxBufferWrite)
            {
                if(debug_uart_rxBufferLoopDetect > 0u)
                {
                    size = debug_uart_RXBUFFERSIZE;
                }
                else
                {
                    size = 0u;
                }
            }
            else if(debug_uart_rxBufferRead < debug_uart_rxBufferWrite)
            {
                size = (debug_uart_rxBufferWrite - debug_uart_rxBufferRead);
            }
            else
            {
                size = (debug_uart_RXBUFFERSIZE - debug_uart_rxBufferRead) + debug_uart_rxBufferWrite;
            }

            /* Enable Rx interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_EnableRxInt();
            #endif /* End debug_uart_RX_INTERRUPT_ENABLED */

        #else /* debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH */

            /* We can only know if there is data in the fifo. */
            size = (debug_uart_RXSTATUS_REG & debug_uart_RX_STS_FIFO_NOTEMPTY) ? 1u : 0u;

        #endif /* End debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: debug_uart_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clear the RX RAM buffer by setting the read and write pointers both to zero.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_rxBufferWrite - cleared to zero. 
    *  debug_uart_rxBufferRead - cleared to zero.
    *  debug_uart_rxBufferLoopDetect - cleared to zero. 
    *  debug_uart_rxBufferOverflow - cleared to zero. 
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to 
    *  read and writing will resume at address 0 overwriting any data that may 
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *******************************************************************************/
    void debug_uart_ClearRxBuffer(void)
    {
        #if(debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH)
            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_DisableRxInt();
            #endif /* End debug_uart_RX_INTERRUPT_ENABLED */

            debug_uart_rxBufferRead = 0u;
            debug_uart_rxBufferWrite = 0u;
            debug_uart_rxBufferLoopDetect = 0u;
            debug_uart_rxBufferOverflow = 0u;

            /* Enable Rx interrupt. */
            #if(debug_uart_RX_INTERRUPT_ENABLED)
                debug_uart_EnableRxInt();
            #endif /* End debug_uart_RX_INTERRUPT_ENABLED */
        #endif /* End debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: debug_uart_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the receive addressing mode
    *
    * Parameters:
    *  addressMode: Use one of the Enumerated Types listed below
    *              #define debug_uart__B_UART__AM_SW_BYTE_BYTE 1
#define debug_uart__B_UART__AM_SW_DETECT_TO_BUFFER 2
#define debug_uart__B_UART__AM_HW_BYTE_BY_BYTE 3
#define debug_uart__B_UART__AM_HW_DETECT_TO_BUFFER 4
#define debug_uart__B_UART__AM_NONE 0

    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void debug_uart_SetRxAddressMode(uint8 addressMode)  
    {
        #if(debug_uart_RXHW_ADDRESS_ENABLED)
            #if(debug_uart_CONTROL_REG_REMOVED)
                addressMode = addressMode;
            #else /* debug_uart_CONTROL_REG_REMOVED */
                uint8 tmpCtrl = 0u;
                tmpCtrl = debug_uart_CONTROL_REG & ~debug_uart_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= ((addressMode << debug_uart_CTRL_RXADDR_MODE0_SHIFT) & \
                           debug_uart_CTRL_RXADDR_MODE_MASK);
                debug_uart_CONTROL_REG = tmpCtrl;
            #endif /* End debug_uart_CONTROL_REG_REMOVED */
        #else /* debug_uart_RXHW_ADDRESS_ENABLED */
            addressMode = addressMode;
        #endif /* End debug_uart_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: debug_uart_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Set the first hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void debug_uart_SetRxAddress1(uint8 address) 

    {
        debug_uart_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: debug_uart_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Set the second hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void debug_uart_SetRxAddress2(uint8 address) 
    {
        debug_uart_RXADDRESS2_REG = address;
    }

#endif  /* debug_uart_RX_ENABLED || debug_uart_HD_ENABLED*/


#if( (debug_uart_TX_ENABLED) || (debug_uart_HD_ENABLED) )

    #if(debug_uart_TX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: debug_uart_EnableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void debug_uart_EnableTxInt(void) 
        {
            CyIntEnable(debug_uart_TX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: debug_uart_DisableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void debug_uart_DisableTxInt(void) 
        {
            CyIntDisable(debug_uart_TX_VECT_NUM);
        }

    #endif /* debug_uart_TX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: debug_uart_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  intSrc: An or'd combination of the desired status bit masks (defined in
    *          the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void debug_uart_SetTxInterruptMode(uint8 intSrc) 
    {
        debug_uart_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: debug_uart_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Write a byte of data to the Transmit FIFO or TX buffer to be sent when the 
    *  bus is available. WriteTxData sends a byte without checking for buffer room 
    *  or status. It is up to the user to separately check status.    
    *
    * Parameters:
    *  TXDataByte: byte of data to place in the transmit FIFO
    *
    * Return:
    * void
    *
    * Global Variables:
    *  debug_uart_txBuffer - RAM buffer pointer for save data for transmission
    *  debug_uart_txBufferWrite - cyclic index for write to txBuffer, 
    *    incremented after each byte saved to buffer.
    *  debug_uart_txBufferRead - cyclic index for read from txBuffer, 
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  debug_uart_initVar - checked to identify that the component has been  
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void debug_uart_WriteTxData(uint8 txDataByte)
    {
        /* If not Initialized then skip this function*/
        if(debug_uart_initVar != 0u)
        {
            #if(debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH)

                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(debug_uart_TX_INTERRUPT_ENABLED)
                    debug_uart_DisableTxInt();
                #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

                if( (debug_uart_txBufferRead == debug_uart_txBufferWrite) && \
                   !(debug_uart_TXSTATUS_REG & debug_uart_TX_STS_FIFO_FULL) )
                {
                    /* Add directly to the FIFO. */
                    debug_uart_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(debug_uart_txBufferWrite >= debug_uart_TXBUFFERSIZE)
                    {
                        debug_uart_txBufferWrite = 0;
                    }

                    debug_uart_txBuffer[debug_uart_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    debug_uart_txBufferWrite++;

                }

                /* Enable Tx interrupt. */
                #if(debug_uart_TX_INTERRUPT_ENABLED)
                    debug_uart_EnableTxInt();
                #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

            #else /* debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH */

                /* Add directly to the FIFO. */
                debug_uart_TXDATA_REG = txDataByte;

            #endif /* End debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH */
        }
    }


    /*******************************************************************************
    * Function Name: debug_uart_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the status register for the component
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the status register which is clear on read. It is up to 
    *  the user to handle all bits in this return value accordingly, even if the bit 
    *  was not enabled as an interrupt source the event happened and must be handled
    *  accordingly.    
    *
    *******************************************************************************/
    uint8 debug_uart_ReadTxStatus(void) 
    {
        return(debug_uart_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: debug_uart_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Wait to send byte until TX register or buffer has room.
    *
    * Parameters:
    *  txDataByte: The 8-bit data value to send across the UART.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_txBuffer - RAM buffer pointer for save data for transmission
    *  debug_uart_txBufferWrite - cyclic index for write to txBuffer, 
    *     checked to identify free space in txBuffer and incremented after each byte 
    *     saved to buffer.
    *  debug_uart_txBufferRead - cyclic index for read from txBuffer, 
    *     checked to identify free space in txBuffer.
    *  debug_uart_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void debug_uart_PutChar(uint8 txDataByte)
    {
            #if(debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH)

                /* Block if buffer is full, so we dont overwrite. */
                while( debug_uart_txBufferWrite == (debug_uart_txBufferRead - 1u) || \
                    (uint8)(debug_uart_txBufferWrite - debug_uart_txBufferRead) ==
                    (uint8)(debug_uart_TXBUFFERSIZE - 1u) )
                {
                    /* Software buffer is full. */
                }
                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(debug_uart_TX_INTERRUPT_ENABLED)
                    debug_uart_DisableTxInt();
                #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

                if( (debug_uart_txBufferRead == debug_uart_txBufferWrite) && \
                   !(debug_uart_TXSTATUS_REG & debug_uart_TX_STS_FIFO_FULL) )
                {
                    /* Add directly to the FIFO. */
                    debug_uart_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(debug_uart_txBufferWrite >= debug_uart_TXBUFFERSIZE)
                    {
                        debug_uart_txBufferWrite = 0;
                    }

                    debug_uart_txBuffer[debug_uart_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    debug_uart_txBufferWrite++;

                }

                /* Enable Rx interrupt. */
                #if(debug_uart_TX_INTERRUPT_ENABLED)
                    debug_uart_EnableTxInt();
                #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

            #else /* debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH */

                /* Block if there isnt room. */
                while(debug_uart_TXSTATUS_REG & debug_uart_TX_STS_FIFO_FULL);

                /* Add directly to the FIFO. */
                debug_uart_TXDATA_REG = txDataByte;

            #endif /* End debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: debug_uart_PutString
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: uint8 pointer to character string of Data to Send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  This function will block if there is not enough memory to place the whole 
    *  string, it will block until the entire string has been written to the 
    *  transmit buffer.
    *
    *******************************************************************************/
    void debug_uart_PutString(uint8* string)
    {
        /* If not Initialized then skip this function*/
        if(debug_uart_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent*/
            while(*string != 0u)
            {
                debug_uart_PutChar(*string++);
            }
        }
    }


    /*******************************************************************************
    * Function Name: debug_uart_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of Bytes to be transmitted.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void debug_uart_PutArray(uint8* string, uint8 byteCount)
    {
        /* If not Initialized then skip this function*/
        if(debug_uart_initVar != 0u)
        {
            while(byteCount > 0u)
            {
                debug_uart_PutChar(*string++);
                byteCount--;
            }
        }
    }


    /*******************************************************************************
    * Function Name: debug_uart_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Write a character and then carriage return and line feed.
    *
    * Parameters:
    *  txDataByte: uint8 Character to send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void debug_uart_PutCRLF(uint8 txDataByte)
    {
        /* If not Initialized then skip this function*/
        if(debug_uart_initVar != 0u)
        {
            debug_uart_PutChar(txDataByte);
            debug_uart_PutChar(0x0Du);
            debug_uart_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: debug_uart_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of space left in the TX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Integer count of the number of bytes left in the TX buffer
    *
    * Global Variables:
    *  debug_uart_txBufferWrite - used to calculate left space. 
    *  debug_uart_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 debug_uart_GetTxBufferSize(void)
    {
        uint8 size;

        #if(debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(debug_uart_TX_INTERRUPT_ENABLED)
                debug_uart_DisableTxInt();
            #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

            if(debug_uart_txBufferRead == debug_uart_txBufferWrite)
            {
                size = 0u;
            }
            else if(debug_uart_txBufferRead < debug_uart_txBufferWrite)
            {
                size = (debug_uart_txBufferWrite - debug_uart_txBufferRead);
            }
            else
            {
                size = (debug_uart_TXBUFFERSIZE - debug_uart_txBufferRead) + debug_uart_txBufferWrite;
            }

            /* Enable Tx interrupt. */
            #if(debug_uart_TX_INTERRUPT_ENABLED)
                debug_uart_EnableTxInt();
            #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

        #else /* debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH */

            size = debug_uart_TXSTATUS_REG;

            /* Is the fifo is full. */
            if(size & debug_uart_TX_STS_FIFO_FULL)
            {
                size = debug_uart_FIFO_LENGTH;
            }
            else if(size & debug_uart_TX_STS_FIFO_EMPTY)
            {
                size = 0u;
            }
            else
            {
                /* We only know there is data in the fifo. */
                size = 1u;
            }

        #endif /* End debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: debug_uart_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clear the TX RAM buffer by setting the read and write pointers both to zero.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_txBufferWrite - cleared to zero. 
    *  debug_uart_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to 
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *
    *******************************************************************************/
    void debug_uart_ClearTxBuffer(void)
    {
        #if(debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(debug_uart_TX_INTERRUPT_ENABLED)
                debug_uart_DisableTxInt();
            #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

            debug_uart_txBufferRead = 0u;
            debug_uart_txBufferWrite = 0u;

            /* Enable Tx interrupt. */
            #if(debug_uart_TX_INTERRUPT_ENABLED)
                debug_uart_EnableTxInt();
            #endif /* End debug_uart_TX_INTERRUPT_ENABLED */

        #endif /* End debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: debug_uart_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Write a Break command to the UART
    *
    * Parameters:
    *  uint8 retMode:  Wait mode,
    *   0 - Initialize registers for Break, sends the Break signal and return 
    *       imediately.
    *   1 - Wait until Break sending is complete, reinitialize registers to normal
    *       transmission mode then return.
    *   2 - Reinitialize registers to normal transmission mode then return.
    *   3 - both steps: 0 and 1
    *       init registers for Break, send Break signal
    *       wait until Break sending is complete, reinit registers to normal
    *       transmission mode then return.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_initVar - checked to identify that the component has been  
    *     initialized.
    *  tx_period - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  Trere are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Funcition will block CPU untill transmition 
    *     complete.
    *  2) User may want to use bloking time if UART configured to the low speed 
    *     operation
    *     Emample for this case:
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to init and use the interrupt for complete 
    *     break operation.
    *     Example for this case:
    *     Init TX interrupt whith "TX - On TX Complete" parameter
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     When interrupt appear with UART_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *   Uses static variable to keep registers configuration.
    *
    *******************************************************************************/
    void debug_uart_SendBreak(uint8 retMode)
    {

        /* If not Initialized then skip this function*/
        if(debug_uart_initVar != 0u)
        {
            /*Set the Counter to 13-bits and transmit a 00 byte*/
            /*When that is done then reset the counter value back*/
            uint8 tmpStat;

            #if(debug_uart_HD_ENABLED) /* Half Duplex mode*/

                if( (retMode == debug_uart_SEND_BREAK) ||
                    (retMode == debug_uart_SEND_WAIT_REINIT ) )
                {
                    /* CTRL_HD_SEND_BREAK - sends break bits in HD mode*/
                    debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() | \
                                                          debug_uart_CTRL_HD_SEND_BREAK);
                    /* Send zeros*/
                    debug_uart_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = debug_uart_TXSTATUS_REG;
                    }while(tmpStat & debug_uart_TX_STS_FIFO_EMPTY);
                }

                if( (retMode == debug_uart_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == debug_uart_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = debug_uart_TXSTATUS_REG;
                    }while(~tmpStat & debug_uart_TX_STS_COMPLETE);
                }

                if( (retMode == debug_uart_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == debug_uart_REINIT) || \
                    (retMode == debug_uart_SEND_WAIT_REINIT) )
                {
                    debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() & \
                                                         ~debug_uart_CTRL_HD_SEND_BREAK);
                }

            #else /* debug_uart_HD_ENABLED Full Duplex mode */

                static uint8 tx_period; 
                
                if( (retMode == debug_uart_SEND_BREAK) || \
                    (retMode == debug_uart_SEND_WAIT_REINIT) )
                {
                    /* CTRL_HD_SEND_BREAK - skip to send parity bit @ Break signal in Full Duplex mode*/
                    if( (debug_uart_PARITY_TYPE != debug_uart__B_UART__NONE_REVB) || \
                         debug_uart_PARITY_TYPE_SW )
                    {
                        debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() | \
                                                              debug_uart_CTRL_HD_SEND_BREAK);
                    }                                                          

                    #if(debug_uart_TXCLKGEN_DP)
                        tx_period = debug_uart_TXBITCLKTX_COMPLETE_REG;
                        debug_uart_TXBITCLKTX_COMPLETE_REG = debug_uart_TXBITCTR_BREAKBITS;
                    #else
                        tx_period = debug_uart_TXBITCTR_PERIOD_REG;
                        debug_uart_TXBITCTR_PERIOD_REG = debug_uart_TXBITCTR_BREAKBITS8X;
                    #endif /* End debug_uart_TXCLKGEN_DP */

                    /* Send zeros*/
                    debug_uart_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = debug_uart_TXSTATUS_REG;
                    }while(tmpStat & debug_uart_TX_STS_FIFO_EMPTY);
                }

                if( (retMode == debug_uart_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == debug_uart_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = debug_uart_TXSTATUS_REG;
                    }while(~tmpStat & debug_uart_TX_STS_COMPLETE);
                }

                if( (retMode == debug_uart_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == debug_uart_REINIT) || \
                    (retMode == debug_uart_SEND_WAIT_REINIT) )
                {

                    #if(debug_uart_TXCLKGEN_DP)
                        debug_uart_TXBITCLKTX_COMPLETE_REG = tx_period;
                    #else
                        debug_uart_TXBITCTR_PERIOD_REG = tx_period;
                    #endif /* End debug_uart_TXCLKGEN_DP */

                    if( (debug_uart_PARITY_TYPE != debug_uart__B_UART__NONE_REVB) || 
                         debug_uart_PARITY_TYPE_SW )
                    {
                        debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() & \
                                                             ~debug_uart_CTRL_HD_SEND_BREAK);
                    }                                     
                }
            #endif    /* End debug_uart_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: debug_uart_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the transmit addressing mode
    *
    * Parameters:
    *  addressMode: 0 -> Space
    *               1 -> Mark
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void debug_uart_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable*/
        if(addressMode != 0)
        {
            debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() | debug_uart_CTRL_MARK);
        }
        else
        {
            debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() & ~debug_uart_CTRL_MARK);
        }
    }

#endif  /* Enddebug_uart_TX_ENABLED */

#if(debug_uart_HD_ENABLED)


    /*******************************************************************************
    * Function Name: debug_uart_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Rx configuration if required and loads the
    *  Tx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Tx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART. 
    *
    *******************************************************************************/
    void debug_uart_LoadTxConfig(void) 
    {
        debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() | debug_uart_CTRL_HD_SEND);
        debug_uart_RXBITCTR_PERIOD_REG = debug_uart_HD_TXBITCTR_INIT;
    }


    /*******************************************************************************
    * Function Name: debug_uart_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Tx configuration if required and loads the
    *  Rx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Rx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART
    *
    *******************************************************************************/
    void debug_uart_LoadRxConfig(void) 
    {
        debug_uart_WriteControlRegister(debug_uart_ReadControlRegister() & ~debug_uart_CTRL_HD_SEND);
        debug_uart_RXBITCTR_PERIOD_REG = debug_uart_HD_RXBITCTR_INIT;
    }

#endif  /* debug_uart_HD_ENABLED */


/* [] END OF FILE */
