/*******************************************************************************
* File Name: debug_uart_INT.c
* Version 2.0
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "debug_uart.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (debug_uart_RX_ENABLED || debug_uart_HD_ENABLED) && \
     (debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH))

    extern volatile uint8 debug_uart_rxBuffer[];
    extern volatile uint8 debug_uart_rxBufferRead;
    extern volatile uint8 debug_uart_rxBufferWrite;
    extern volatile uint8 debug_uart_rxBufferLoopDetect;
    extern volatile uint8 debug_uart_rxBufferOverflow;


    /*******************************************************************************
    * Function Name: debug_uart_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_rxBuffer - RAM buffer pointer for save received data.
    *  debug_uart_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     increments after each byte saved to buffer.
    *  debug_uart_rxBufferRead - cyclic index for read from rxBuffer, 
    *     checked to detect overflow condition.
    *  debug_uart_rxBufferOverflow - software overflow flag. Set to one
    *     when debug_uart_rxBufferWrite index overtakes 
    *     debug_uart_rxBufferRead index.
    *  debug_uart_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when debug_uart_rxBufferWrite is equal to 
    *    debug_uart_rxBufferRead
    *
    *******************************************************************************/
    CY_ISR(debug_uart_RXISR)
    {
        uint8 readData;

        /* User code required at start of ISR */
        /* `#START debug_uart_RXISR_START` */

        /* `#END` */

        readData = debug_uart_RXSTATUS_REG;

        if(readData & (debug_uart_RX_STS_BREAK | debug_uart_RX_STS_PAR_ERROR | \
                       debug_uart_RX_STS_STOP_ERROR | debug_uart_RX_STS_OVERRUN))
        {
            /* ERROR handling. */
            /* `#START debug_uart_RXISR_ERROR` */

            /* `#END` */
        }

        while(readData & debug_uart_RX_STS_FIFO_NOTEMPTY)
        {
            /* Check pointer. */
            if(debug_uart_rxBufferWrite >= debug_uart_RXBUFFERSIZE)
            {
                debug_uart_rxBufferWrite = 0u;
                if(debug_uart_rxBufferWrite == debug_uart_rxBufferRead)
                {   /* Set Software Buffer status Overflow */
                    debug_uart_rxBufferOverflow = 1u;
                }
            }

            debug_uart_rxBuffer[debug_uart_rxBufferWrite] = debug_uart_RXDATA_REG;

            if(debug_uart_rxBufferLoopDetect)
            {   /* Set Software Buffer status Overflow */
                debug_uart_rxBufferOverflow = 1u;
            }
            /* Set next pointer. */
            debug_uart_rxBufferWrite++;

            if(debug_uart_rxBufferWrite == debug_uart_rxBufferRead)
            {
                debug_uart_rxBufferLoopDetect = 1u;
            }

            /* Check again if there is data. */
            readData = debug_uart_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START debug_uart_RXISR_END` */

        /* `#END` */

        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(debug_uart_PSOC3_ES2 && (debug_uart_RXInternalInterrupt__ES2_PATCH))
            debug_uart_ISR_PATCH();
        #endif /* End debug_uart_PSOC3_ES2*/
    }

#endif /* End debug_uart_RX_ENABLED && (debug_uart_RXBUFFERSIZE > debug_uart_FIFO_LENGTH) */


#if(debug_uart_TX_ENABLED && (debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH))

    extern volatile uint8 debug_uart_txBuffer[];
    extern volatile uint8 debug_uart_txBufferRead;
    extern uint8 debug_uart_txBufferWrite;


    /*******************************************************************************
    * Function Name: debug_uart_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  debug_uart_txBuffer - RAM buffer pointer for transmit data from.
    *  debug_uart_txBufferRead - cyclic index for read and transmit data 
    *     from txBuffer, increments after each transmited byte.
    *  debug_uart_rxBufferWrite - cyclic index for write to txBuffer, 
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(debug_uart_TXISR)
    {

        /* User code required at start of ISR */
        /* `#START debug_uart_TXISR_START` */

        /* `#END` */

        while((debug_uart_txBufferRead != debug_uart_txBufferWrite) && \
             !(debug_uart_TXSTATUS_REG & debug_uart_TX_STS_FIFO_FULL))
        {
            /* Check pointer. */
            if(debug_uart_txBufferRead >= debug_uart_TXBUFFERSIZE)
            {
                debug_uart_txBufferRead = 0u;
            }

            debug_uart_TXDATA_REG = debug_uart_txBuffer[debug_uart_txBufferRead];

            /* Set next pointer. */
            debug_uart_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START debug_uart_TXISR_END` */

        /* `#END` */
        
        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(debug_uart_PSOC3_ES2 && (debug_uart_TXInternalInterrupt__ES2_PATCH))
            debug_uart_ISR_PATCH();
        #endif /* End debug_uart_PSOC3_ES2*/
    }

#endif /* End debug_uart_TX_ENABLED && (debug_uart_TXBUFFERSIZE > debug_uart_FIFO_LENGTH) */


/* [] END OF FILE */
