/*******************************************************************************
* File Name: divider_mux.c
* Version 1.50
*
*  Description:
*    This file contains all functions required for the analog multiplexer
*    AMux User Module.
*
*   Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "divider_mux.h"

uint8 divider_mux_initVar = 0u;
uint8 divider_mux_lastChannel = divider_mux_NULL_CHANNEL;


/*******************************************************************************
* Function Name: divider_mux_Start
********************************************************************************
* Summary:
*  Disconnect all channels.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_Start(void)
{
    divider_mux_DisconnectAll();
    divider_mux_initVar = 1u;
}


/*******************************************************************************
* Function Name: divider_mux_Init
********************************************************************************
* Summary:
*  Disconnect all channels.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_Init(void)
{
    divider_mux_DisconnectAll();
}


/*******************************************************************************
* Function Name: divider_mux_Stop
********************************************************************************
* Summary:
*  Disconnect all channels.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_Stop(void)
{
    divider_mux_DisconnectAll();
}


/*******************************************************************************
* Function Name: divider_mux_Select
********************************************************************************
* Summary:
*  This functions first disconnects all channels then connects the given
*  channel.
*
* Parameters:
*  channel:  The channel to connect to the common terminal.
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_Select(uint8 channel) 
{
    divider_mux_DisconnectAll();        /* Disconnect all previous connections */
    divider_mux_Connect(channel);       /* Make the given selection */
    divider_mux_lastChannel = channel;  /* Update last channel */
}


/*******************************************************************************
* Function Name: divider_mux_FastSelect
********************************************************************************
* Summary:
*  This function first disconnects the last connection made with FastSelect or
*  Select, then connects the given channel. The FastSelect function is similar
*  to the Select function, except it is faster since it only disconnects the
*  last channel selected rather than all channels.
*
* Parameters:
*  channel:  The channel to connect to the common terminal.
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_FastSelect(uint8 channel) 
{
    /* Disconnect the last valid channel */
    if( divider_mux_lastChannel != divider_mux_NULL_CHANNEL)   /* Update last channel */
    {
        divider_mux_Disconnect(divider_mux_lastChannel);
    }

    /* Make the new channel connection */
    divider_mux_Connect(channel);
    divider_mux_lastChannel = channel;   /* Update last channel */
}


#if(divider_mux_MUXTYPE == divider_mux_MUX_DIFF)
/*******************************************************************************
* Function Name: divider_mux_Connect
********************************************************************************
* Summary:
*  This function connects the given channel without affecting other connections.
*
* Parameters:
*  channel:  The channel to connect to the common terminal.
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_Connect(uint8 channel) 
{
    divider_mux_CYAMUXSIDE_A_Set(channel);
    divider_mux_CYAMUXSIDE_B_Set(channel);
}


/*******************************************************************************
* Function Name: divider_mux_Disconnect
********************************************************************************
* Summary:
*  This function disconnects the given channel from the common or output
*  terminal without affecting other connections.
*
* Parameters:
*  channel:  The channel to disconnect from the common terminal.
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_Disconnect(uint8 channel) 
{
    divider_mux_CYAMUXSIDE_A_Unset(channel);
    divider_mux_CYAMUXSIDE_B_Unset(channel);
}
#endif


/*******************************************************************************
* Function Name: divider_mux_DisconnectAll
********************************************************************************
* Summary:
*  This function disconnects all channels.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void divider_mux_DisconnectAll(void) 
{
    uint8 chan;

#if(divider_mux_MUXTYPE == divider_mux_MUX_SINGLE)
    for(chan = 0; chan < divider_mux_CHANNELS ; chan++)
    {
        divider_mux_Unset(chan);
    }
#else
    for(chan = 0; chan < divider_mux_CHANNELS ; chan++)
    {
        divider_mux_CYAMUXSIDE_A_Unset(chan);
        divider_mux_CYAMUXSIDE_B_Unset(chan);
    }
#endif
}


/* [] END OF FILE */
