/*******************************************************************************
* File Name: i2c.c  
* Version 2.10
*
* Description:
*  This file provides the source code of APIs for the I2C component.
*  Actual protocol and operation code resides in the interrupt service routine 
*  file.
*
* Note: 
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "i2c.h"  


/**********************************
*      System variables
**********************************/

uint8 i2c_initVar = 0u;
extern volatile uint8 i2c_State;    /* Current state of I2C state machine */

/* Master variables */
#if (i2c_MODE & i2c_MODE_MASTER)
   extern volatile uint8 i2c_mstrStatus;          /* Master Status byte */
   extern volatile uint8 i2c_mstrControl;         /* Master Control byte */
   
   /* Transmit buffer variables */
   extern volatile uint8 * i2c_mstrRdBufPtr;      /* Pointer to Master Read buffer */
   extern volatile uint8   i2c_mstrRdBufSize;     /* Master Read buffer size */
   extern volatile uint8   i2c_mstrRdBufIndex;    /* Master Read buffer Index */
    
   /* Receive buffer variables */
   extern volatile uint8 * i2c_mstrWrBufPtr;      /* Pointer to Master Write buffer */
   extern volatile uint8   i2c_mstrWrBufSize;     /* Master Write buffer size */
   extern volatile uint8   i2c_mstrWrBufIndex;    /* Master Write buffer Index */
   
#endif  /* End (i2c_MODE & i2c_MODE_MASTER) */

/* Slave variables */
#if (i2c_MODE & i2c_MODE_SLAVE)
   extern volatile uint8 i2c_slStatus;            /* Slave Status  */
   
   #if (i2c_ADDR_DECODE == i2c_SW_DECODE)
      extern volatile uint8 i2c_slAddress;        /* Software address variable */
   #endif   /* End (i2c_ADDR_DECODE == i2c_SW_DECODE) */
   
   /* Transmit buffer variables */
   extern volatile uint8 * i2c_slRdBufPtr;        /* Pointer to Transmit buffer */
   extern volatile uint8   i2c_slRdBufSize;       /* Slave Transmit buffer size */
   extern volatile uint8   i2c_slRdBufIndex;      /* Slave Transmit buffer Index */

   /* Receive buffer variables */
   extern volatile uint8 * i2c_slWrBufPtr;        /* Pointer to Receive buffer */
   extern volatile uint8   i2c_slWrBufSize;       /* Slave Receive buffer size */
   extern volatile uint8   i2c_slWrBufIndex;      /* Slave Receive buffer Index */

#endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */


/*******************************************************************************
* Function Name: i2c_Init
********************************************************************************
*
* Summary:
*  Initializes I2C registers with initial values provided from customizer.
*
* Parameters:  
*  None
*
* Return: 
*  None
*
* Global variables:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
void i2c_Init(void)
{
    #if (i2c_IMPLEMENTATION == i2c_FF)
        
        /* Enable Master or Slave */
        i2c_CFG_REG = (i2c_ENABLE_SLAVE | i2c_ENABLE_MASTER);
        
        /* 50 kHz - 32 samples/bit */
        i2c_CFG_REG |= i2c_DEFAULT_CLK_RATE;
         
        /* Set devide factor */
        #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
            i2c_CLKDIV_REG = i2c_DEFAULT_DIVIDE_FACTOR;
        #else
            i2c_CLKDIV1_REG = LO8(i2c_DEFAULT_DIVIDE_FACTOR);
            i2c_CLKDIV2_REG = HI8(i2c_DEFAULT_DIVIDE_FACTOR);
        #endif /* End (I2C_PSOC3_ES2 || I2C_PSOC5_ES1) */
        
        /* if I2C block will be used as wake up source */
        #if (i2c_ENABLE_WAKEUP)
            /* I2C block as wake-up source */
            i2c_XCFG_REG = i2c_XCFG_I2C_ON;
        
            /* Process sio_select and pselect */
            #if(i2c_I2C_PAIR_SELECTED == i2c_I2C_PAIR0)
                /* Set I2C0 SIO pair P12[0,1] */
                i2c_CFG_REG |= i2c_CFG_SIO_SELECT;
            #else
                /* Do nothing for I2C1 SIO pair P12[4,5] */
            #endif /* End (i2c_ENABLE_WAKEUP) */
            
            i2c_CFG_REG |= i2c_CFG_PSELECT;
        
        #endif  /* End ((i2c_ENABLE_WAKEUP)*/
        
        /* Clear Status register */
        i2c_CSR_REG = 0x00u;
        
    #else
        /* Enable Byte Complete for interrupt in the mask register */
        i2c_INT_MASK_REG |= i2c_BYTE_COMPLETE_IE_MASK;
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */
    
    /* Set address detection type */
    #if (i2c_IMPLEMENTATION == i2c_FF)
        #if (i2c_ADDR_DECODE == i2c_HDWR_DECODE)
            /* Turn on hardware address detection and enable the clock */
            i2c_XCFG_REG  |= (i2c_XCFG_HDWR_ADDR_EN | i2c_XCFG_CLK_EN);
            
        #else
            /* Enable the clock */
            i2c_XCFG_REG  |= i2c_XCFG_CLK_EN;
            
        #endif  /* End (i2c_ADDR_DECODE == i2c_HDWR_DECODE) */
        
    #else
        #if (i2c_MODE & i2c_MODE_SLAVE)
            #if (i2c_ADDR_DECODE == i2c_HDWR_DECODE)
                /* Turn off any address match */
                i2c_CFG_REG &= ~(i2c_CTRL_ANY_ADDRESS_MASK);
                
            #else
                /* Turn on any address match */
                i2c_CFG_REG |= i2c_CTRL_ANY_ADDRESS_MASK;
                
            #endif  /* End (i2c_ADDR_DECODE == i2c_HDWR_DECODE) */
        
        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
    
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */
    
    /* Disable Interrupt */
    CyIntDisable(i2c_ISR_NUMBER);
    
    /* Set the ISR to point to the RTC_SUT_isr Interrupt */
    CyIntSetVector(i2c_ISR_NUMBER, i2c_ISR);
    
    /* Set the priority */
    CyIntSetPriority(i2c_ISR_NUMBER, i2c_ISR_PRIORITY);
    
    /* Put state machine in idle state */
    i2c_State = i2c_SM_IDLE; 
    
    #if (i2c_MODE & i2c_MODE_SLAVE)
        /* Set default status */
        i2c_SlaveClearReadBuf();
        i2c_SlaveClearWriteBuf();
        i2c_SlaveClearReadStatus();
        i2c_SlaveClearWriteStatus();
        
        /* Set default address */
        i2c_SlaveSetAddress(i2c_DEFAULT_ADDR);
    
    #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
    
    #if (i2c_MODE & i2c_MODE_MASTER)
        /* Set default status */
        i2c_MasterClearReadBuf();
        i2c_MasterClearWriteBuf();
        i2c_MasterClearStatus();
        
    #endif  /* End (i2c_MODE & i2c_MODE_MASTER) */
}


/*******************************************************************************
* Function Name: i2c_Enable
********************************************************************************
*
* Summary:
*  Enables I2C operations.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  None
*
*******************************************************************************/
void i2c_Enable(void) 
{          
    #if ((i2c_IMPLEMENTATION != i2c_UDB) || \
         (i2c_MODE & i2c_MODE_SLAVE))
        uint8 enableInterrupts;
    #endif  /* End ((i2c_IMPLEMENTATION != i2c_UDB) || \
                    (i2c_MODE & i2c_MODE_SLAVE) ) */
                   
    #if(i2c_IMPLEMENTATION == i2c_FF)
        enableInterrupts = CyEnterCriticalSection();
        /* Enable power to I2C Module */
        i2c_ACT_PWRMGR_REG  |= i2c_ACT_PWR_EN;
        i2c_STBY_PWRMGR_REG |= i2c_STBY_PWR_EN;
        CyExitCriticalSection(enableInterrupts);
        
    #else
        /* Enable the I2C */
        i2c_CFG_REG |= (i2c_ENABLE_MASTER | i2c_ENABLE_SLAVE);
        
        /* Enable bit counter */
        #if (i2c_MODE & i2c_MODE_SLAVE)
            enableInterrupts = CyEnterCriticalSection();
            i2c_COUNTER_AUX_CTL_REG |= i2c_COUNTER_ENABLE_MASK;
            CyExitCriticalSection(enableInterrupts);
        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */   
}


/*******************************************************************************
* Function Name: i2c_Start
********************************************************************************
*
* Summary:
*  Starts the I2C hardware. Enables Active mode power template bits or clock 
*  gating as appropriate. It is required to be executed before I2C bus operation.
*  The I2C interrupt remains disabled after this function call.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  This component automatically enables it's interrupt.  If I2C is enabled
*  without the interrupt enabled, it could lock up the I2C bus.
*
* Global variables:
*  i2c_initVar - used to check initial configuration, modified 
*  on first function call.
*
* Reentrant:
*  No
*
*******************************************************************************/
void i2c_Start(void)
{
    /* Initialize I2C registers, reset I2C buffer index and clears status */
    if (i2c_initVar == 0u)
    {
        i2c_Init();
        i2c_initVar = 1u;
    }
      
    i2c_Enable();
}


/*******************************************************************************
* Function Name: i2c_Stop
********************************************************************************
*
* Summary:
*  Disables I2C hardware and disables I2C interrupt. Disables Active mode power 
*  template bits or clock gating as appropriate.
*
* Parameters:
*  None 
*
* Return: 
*  None 
*
*******************************************************************************/
void i2c_Stop(void)  
{   
    #if ( (i2c_IMPLEMENTATION != i2c_UDB) || \
          (i2c_MODE & i2c_MODE_SLAVE) )
        uint8 enableInterrupts;
    #endif  /* End ( (i2c_IMPLEMENTATION != i2c_FF) || \
                   (i2c_MODE & i2c_MODE_SLAVE) ) */
    
    /* Disable Interrupt */
    i2c_DisableInt();
    
    #if (i2c_IMPLEMENTATION == i2c_FF)
        enableInterrupts = CyEnterCriticalSection();
        /* Disable power to I2C block */
        i2c_ACT_PWRMGR_REG  &= ~i2c_ACT_PWR_EN;
        i2c_STBY_PWRMGR_REG &= ~i2c_STBY_PWR_EN;
        CyExitCriticalSection(enableInterrupts);
        
    #else
        /* Reset the I2C before starting */
        i2c_CFG_REG &= ~(i2c_ENABLE_MASTER | i2c_ENABLE_SLAVE);
        
        /* Disable bit counter */
        #if (i2c_MODE & i2c_MODE_SLAVE)
            enableInterrupts = CyEnterCriticalSection();
            i2c_COUNTER_AUX_CTL_REG &= ~i2c_COUNTER_ENABLE_MASK;
            CyExitCriticalSection(enableInterrupts);
        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */
}


/*******************************************************************************
* Function Name: i2c_EnableInt
********************************************************************************
*
* Summary:
*  Enables I2C interrupt. Interrupts are required for most operations.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void i2c_EnableInt(void)   
{
    #if (i2c_IMPLEMENTATION == i2c_UDB)
        i2c_INT_ENABLE_REG |= i2c_INT_ENABLE_MASK;
    #endif  /* End (i2c_IMPLEMENTATION == i2c_UDB) */
    
    CyIntEnable(i2c_ISR_NUMBER);
}


/*******************************************************************************
* Function Name: i2c_DisableInt
********************************************************************************
*
* Summary:
*  Disables I2C interrupts. Normally this function is not required since the 
*  Stop function disables the interrupt. If the I2C interrupt is disabled while 
*  the I2C master is still running, it may cause the I2C bus to lock up.
*
* Parameters:
*  None
*
* Return: 
*  None
*
* Side Effects:
*  If the I2C interrupt is disabled and the master is addressing the current 
*  slave, the bus will be locked until the interrupt is re-enabled.
*
*******************************************************************************/
void i2c_DisableInt(void)   
{
    #if (i2c_IMPLEMENTATION == i2c_UDB)
        i2c_INT_ENABLE_REG |= i2c_INT_ENABLE_MASK;
    #endif  /* End (i2c_IMPLEMENTATION == i2c_UDB) */
    
    CyIntDisable(i2c_ISR_NUMBER);
}


#if (i2c_MODE & i2c_MODE_MASTER)

    /*******************************************************************************
    * Function Name: i2c_MasterStatus
    ********************************************************************************
    *
    * Summary:
    *  Returns the master's communication status.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Current status of I2C master.
    *
    * Global variables:
    *  i2c_mstrStatus - used to store current status of I2C Master.
    *
    *******************************************************************************/
    uint8 i2c_MasterStatus(void) 
    {
        uint8 status = i2c_mstrStatus; 
        
        if(i2c_State != i2c_SM_SL_WR_IDLE)
        {
            /* Add transaction in progress activity to master status */
            status |= i2c_MSTAT_XFER_INP;
        }
        else
        {
            /* Current master status is valid */
        }
        
        return (status);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterClearStatus
    ********************************************************************************
    *
    * Summary:
    *  Clears all status flags and returns the master status.
    *
    * Parameters:
    *  None
    *
    * Return: 
    *  Current status of I2C master.
    *
    * Global variables:
    *  i2c_mstrStatus - used to store current status of I2C Master.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_MasterClearStatus(void)
    {
        /* Current master status */
        uint8 status = i2c_mstrStatus;
        
        /* Clear master status */
        i2c_mstrStatus  = i2c_MSTAT_CLEAR; 
        
        return (status);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterWriteBuf
    ********************************************************************************
    *
    * Summary:
    *  Automatically writes an entire buffer of data to a slave device. Once the 
    *  data transfer is initiated by this function, further data transfer is handled
    *  by the included ISR in byte by byte mode.
    *
    * Parameters:
    *  slaveAddr: 7-bit slave address.
    *  xferData:  Pointer to buffer of data to be sent.
    *  cnt:       Size of buffer to send.
    *  mode:      Transfer mode defines: start or restart condition generation at 
    *             begin of the transfer and complete the transfer or halt before 
    *             generating a stop.
    *
    * Return:
    *  Status error - zero means no errors.
    *
    * Side Effects:
    *  The included ISR will start transfer after start or restart condition will 
    *  be generated.
    *
    * Global variables:
    *  i2c_mstrStatus  - used to store current status of I2C Master.
    *  i2c_State       - used to store current state of state machine.
    *  i2c_mstrControl - used to control master end of transaction with
    *  or without the Stop generation.
    *  i2c_mstrWrBufPtr - used to store pointer to master write buffer.
    *  i2c_mstrWrBufIndex - used to current index within master write 
    *  buffer.
    *  i2c_mstrWrBufSize - used to store master write buffer size.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_MasterWriteBuf(uint8 slaveAddr, uint8 * xferData, uint8 cnt, uint8 mode)
    {
        uint8 errStatus = i2c_MSTR_SLAVE_BUSY; 
        
        /* Check if I2C in proper state to generate Start/ReStart condition */
        if((i2c_State == i2c_SM_IDLE) || 
           (i2c_State == i2c_SM_MSTR_HALT))
        {
            /* If IDLE, check if bus is free */
            if(i2c_State == i2c_SM_IDLE)
            {
                /* If Bus is free proceed, no exist */
                if(i2c_CHECK_BUS_FREE(i2c_MCSR_REG))
                {
                    errStatus = i2c_MSTR_NO_ERROR;
                }
                else
                {
                    errStatus = i2c_MSTR_BUS_BUSY;
                }
            }
            else   /* Bus halted waiting for restart */
            {
                CyIntClearPending(i2c_ISR_NUMBER);
                i2c_mstrStatus &= ~i2c_MSTAT_XFER_HALT;
                errStatus = i2c_MSTR_NO_ERROR;
            }
            
            /* If no errors, generate start */
            if(errStatus == i2c_MSTR_NO_ERROR)
            {
                /* Determine whether or not to automatically generate a stop condition */
                if((mode & i2c_MODE_NO_STOP) != 0u)
                {
                    /* Do not generate a Stop at the end of transfer */
                    i2c_mstrControl |= i2c_MSTR_NO_STOP;
                }
                else  /* Generate a Stop */
                {
                    i2c_mstrControl &= ~i2c_MSTR_NO_STOP;
                }
                
                i2c_State = i2c_SM_MSTR_WR_ADDR;
                slaveAddr = (slaveAddr << 1u);              /* Set Address */
                i2c_DATA_REG = slaveAddr;      /* Write address to data reg */
                
                i2c_mstrWrBufIndex = 0u;       /* Start buffer at zero */
                i2c_mstrWrBufSize  = cnt;      /* Set buffer size */
                i2c_mstrWrBufPtr   = (volatile uint8 *) xferData; /* Set buffer pointer */
                
                /* Generate a Start or ReStart depending on flag passed */
                if((mode & i2c_MODE_REPEAT_START) != 0u)
                {
                    i2c_GENERATE_RESTART;  /* Generate a ReStart */
                }
                else
                {
                    i2c_GENERATE_START;    /* Generate a Start */
                }
                
                /* Enable interrupts to process transfer */
                i2c_EnableInt();
                
                /* Clear write complete flag */
                i2c_mstrStatus &= ~i2c_MSTAT_WR_CMPLT; 
            }
        }
   
        return (errStatus);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterReadBuf
    ********************************************************************************
    *
    * Summary:
    *  Automatically writes an entire buffer of data to a slave device. Once the 
    *  data transfer is initiated by this function, further data transfer is handled
    *  by the included ISR in byte by byte mode.  
    *
    * Parameters:
    *  slaveAddr: 7-bit slave address.
    *  xferData:  Pointer to buffer where to put data from slave.
    *  cnt:       Size of buffer to read.
    *  mode:      Transfer mode defines: start or restart condition generation at 
    *             begin of the transfer and complete the transfer or halt before 
    *             generating a stop.
    *
    * Return:
    *  Status error - zero means no errors.
    *
    * Side Effects:
    *  The included ISR will start transfer after start or restart condition will 
    *  be generated.
    *
    * Global variables:
    *  i2c_mstrStatus  - used to store current status of I2C Master.
    *  i2c_State       - used to store current state of state machine.
    *  i2c_mstrControl - used to control master end of transaction with
    *  or without the Stop generation.
    *  i2c_mstrRdBufPtr - used to store pointer to master write buffer.
    *  i2c_mstrRdBufIndex - used to current index within master write 
    *  buffer.
    *  i2c_mstrRdBufSize - used to store master write buffer size.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_MasterReadBuf(uint8 slaveAddr, uint8 * xferData, uint8 cnt, uint8 mode)
    {
        uint8 errStatus = i2c_MSTR_SLAVE_BUSY;
    
        /* Check if I2C in proper state to generate Start/ReStart condition */
        if((i2c_State == i2c_SM_IDLE) ||
           (i2c_State == i2c_SM_MSTR_HALT))
        {
            /* If IDLE, check if bus is free */
            if(i2c_State == i2c_SM_IDLE)
            {
                 /* If Bus is free proceed, no exist */
                if(i2c_CHECK_BUS_FREE(i2c_MCSR_REG))
                {
                    errStatus = i2c_MSTR_NO_ERROR;
                }
                else
                {
                    errStatus = i2c_MSTR_BUS_BUSY;
                }
            }
            else   /* Bus halted waiting for restart */
            {
                CyIntClearPending(i2c_ISR_NUMBER);
                i2c_mstrStatus &= ~i2c_MSTAT_XFER_HALT;
                errStatus = i2c_MSTR_NO_ERROR;
            }
            
            /* If no error, generate Start/ReStart condition */
            if(errStatus == i2c_MSTR_NO_ERROR)
            {
                /* Determine whether or not to automatically generate a stop condition */
                if((mode & i2c_MODE_NO_STOP) != 0u)
                {
                    /* Do not generate a Stop at the end of transfer */
                    i2c_mstrControl |= i2c_MSTR_NO_STOP;
                }
                else  /* Generate a Stop */
                {
                    i2c_mstrControl &= ~i2c_MSTR_NO_STOP;
                }
                
                i2c_State = i2c_SM_MSTR_RD_ADDR;
                slaveAddr = (slaveAddr << 1);               /* Set Address */
                slaveAddr |= i2c_READ_FLAG;    /* Set the Read flag */
                i2c_DATA_REG = slaveAddr;      /* Write address to data reg */
                 
                i2c_mstrRdBufIndex  = 0u;      /* Start buffer at zero */
                i2c_mstrRdBufSize   = cnt;     /* Set buffer size */
                i2c_mstrRdBufPtr    = (volatile uint8 *) xferData; /* Set buffer pointer */
                
                /* Generate a Start or ReStart depending on flag passed */
                if((mode & i2c_MODE_REPEAT_START) != 0u)
                {
                    i2c_GENERATE_RESTART;  /* Generate a ReStart */
                }
                else
                {
                    i2c_GENERATE_START;    /* Generate a Start */
                }
                
                /* Enable interrupts to process transfer */
                i2c_EnableInt();
                
                /* Clear read complete flag */
                i2c_mstrStatus &= ~i2c_MSTAT_RD_CMPLT; 
            }
        }
        
        return (errStatus);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterSendStart
    ********************************************************************************
    *
    * Summary:
    *  Generates Start condition and sends slave address with read/write bit.
    *
    * Parameters:  
    *  slaveAddress:  7-bit slave address. 
    *  R_nW:          Zero, send write command, non-zero send read command.
    *
    * Return: 
    *  Status error - zero means no errors.
    *
    * Side Effects:
    *  This function is entered without a 'byte complete' bit set in the I2C_CSR 
    *  register. It does not exit until it will be set.
    *
    * Global variables:
    *  i2c_State - used to store current state of state machine.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_MasterSendStart(uint8 slaveAddress, uint8 R_nW)
    {
        uint8 errStatus = i2c_MSTR_SLAVE_BUSY;
        
        /* If IDLE, check if bus is free */
        if(i2c_State == i2c_SM_IDLE)
        {
            /* If bus is free, generate Start condition */
            if(i2c_CHECK_BUS_FREE(i2c_MCSR_REG))
            {
                /* Disable ISR for Manual functions */
                i2c_DisableInt();
                
                slaveAddress = (slaveAddress << 1u);        /* Set Address */
                if(R_nW != 0u)                              /* Set the Read/Write flag */                                 
                {
                    slaveAddress |= i2c_READ_FLAG;   
                    i2c_State = i2c_SM_MSTR_RD_ADDR;
                }
                else
                {
                    i2c_State = i2c_SM_MSTR_WR_ADDR;
                }
                i2c_DATA_REG = slaveAddress;   /* Write address to data reg */
                
                /* Generates a START */
                i2c_GENERATE_START;
                
                /* Wait for the address to be transfered */
                while(i2c_WAIT_BYTE_COMPLETE(i2c_CSR_REG));
                
                #if(i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
                    /* Check for loss of arbitration */
                    if(i2c_CHECK_LOST_ARB(i2c_CSR_REG))
                    {
                        /* MultiMaster-Slave */
                        #if(i2c_MODE & i2c_MODE_SLAVE)
                            /* Check if Start has been generated */
                            if((i2c_MCSR_REG & i2c_MCSR_START_GEN) != 0u)
                            {
                                /* Clear if was not generated */
                                i2c_MCSR_REG &= ~i2c_MCSR_START_GEN;
                            }
                        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
                        
                        /* Arbitration has been lost, reset state machine to IDLE */
                        i2c_State = i2c_SM_IDLE;
                        errStatus = i2c_MSTR_ERR_ARB_LOST; /* Master lost arbitrage */
                    }
                    else if(i2c_CHECK_ADDR_NAK(i2c_CSR_REG))
                    {
                        /* Address has been NACKed, reset state machine to IDLE */
                        i2c_State = i2c_SM_IDLE;
                        errStatus = i2c_MSTR_ERR_LB_NAK;    /* No device ACKed the Master */
                    }
                    else
                    {
                        errStatus = i2c_MSTR_NO_ERROR;     /* Send Start witout errors */
                    }
                    
                #else
                    /* Check ACK address if Master mode */
                    if(i2c_CHECK_ADDR_NAK(i2c_CSR_REG))
                    {
                        /* Address has been NACKed, reset state machine to IDLE */
                        i2c_State = i2c_SM_IDLE;
                        errStatus = i2c_MSTR_ERR_LB_NAK;    /* No device ACKed the Master */
                    }
                    else    
                    {
                        errStatus = i2c_MSTR_NO_ERROR;     /* Send START witout errors */
                    }
                    
                #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
            }
            else
            {
                errStatus = i2c_MSTR_BUS_BUSY;     /* Bus is busy */
            }
        }
        
        return (errStatus);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterSendRestart
    ********************************************************************************
    *
    * Summary:
    *  Generates ReStart condition and sends slave address with read/write bit.
    *
    * Parameters:  
    *  slaveAddress:  7-bit slave address.
    *  R_nW:          Zero, send write command, non-zero send read command.
    *
    * Return: 
    *  Status error - zero means no errors.
    *
    * Side Effects:
    *  This function is entered without a 'byte complete' bit set in the I2C_CSR 
    *  register. It does not exit until it will be set.
    *
    * Global variables:
    *  i2c_State - used to store current state of state machine.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_MasterSendRestart(uint8 slaveAddress, uint8 R_nW)
    {
        uint8 errStatus = i2c_MSTR_SLAVE_BUSY;
        
        /* Check if START condition was generated */
        if(i2c_CHECK_MASTER_MODE(i2c_MCSR_REG))
        {
            slaveAddress = (slaveAddress << 1u);        /* Set Address */
            if(R_nW != 0u)                              /* Set the Read/Write flag */
            {
                slaveAddress |= i2c_READ_FLAG;
                i2c_State = i2c_SM_MSTR_RD_ADDR;
            }
            else
            {
                i2c_State = i2c_SM_MSTR_WR_ADDR;
            }
            i2c_DATA_REG = slaveAddress;    /* Write address to data reg */
            
            /* Generates RESTART */
            i2c_GENERATE_RESTART;
            #if (i2c_IMPLEMENTATION == i2c_UDB)
                while(i2c_CHECK_BYTE_COMPLETE(i2c_CSR_REG));
            #endif /* End (i2c_IMPLEMENTATION == i2c_UDB) */
            
            /* Wait for the address to be transfered  */
            while(i2c_WAIT_BYTE_COMPLETE(i2c_CSR_REG));
            
            if(i2c_CHECK_ADDR_NAK(i2c_CSR_REG))
            {
                errStatus = i2c_MSTR_ERR_LB_NAK;   /* No device ACKed the Master */
            }
            else
            {
                errStatus = i2c_MSTR_NO_ERROR;     /* Send RESTART witout errors */
            }
        }
        
        return (errStatus);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterSendStop
    ********************************************************************************
    *
    * Summary:
    *  Generates I2C Stop condition on bus. Function do nothing if Start or Restart 
    *  condition was failed before call this function.
    *
    * Parameters:  
    *  None
    *
    * Return: 
    *  Status error - zero means no errors.
    *
    * Side Effects:
    *  The Stop generation is required to complete transaction.
    *  This function does not wait while Stop condition will be generated.
    *
    * Global variables:
    *  i2c_State - used to store current state of state machine.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_MasterSendStop(void)
    {
        uint8 errStatus = i2c_MSTR_SLAVE_BUSY;
        
        /* Check if START condition was generated */
        if(i2c_CHECK_MASTER_MODE(i2c_MCSR_REG))
        {
            i2c_GENERATE_STOP;                     /* Generate STOP */
            i2c_State  = i2c_SM_IDLE; /* Reset state to IDLE */
            errStatus = i2c_MSTR_NO_ERROR;         /* Start send STOP witout errors */
                
            /* Wait for STOP generation or BYTE COMPLETE (lost arbitrage) */
            #if(i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
                
                #if (i2c_IMPLEMENTATION == i2c_UDB)
                    while(i2c_CHECK_BYTE_COMPLETE(i2c_CSR_REG));
                #endif /* End (i2c_IMPLEMENTATION == i2c_UDB) */
                
                while((i2c_CSR_REG & (i2c_CSR_BYTE_COMPLETE | 
                                                   i2c_CSR_STOP_STATUS)) == 0u);
                
                /* Check LOST ARBITRAGE */
                if(i2c_CHECK_LOST_ARB(i2c_CSR_REG))
                {
                    errStatus = i2c_MSTR_ERR_ARB_LOST; /* Generate NACK and STOP with LOST ARBITRAGE */
                }
                /* STOP condition generated */
                else
                {
                    errStatus = i2c_MSTR_NO_ERROR;     /* Generate STOP wihtout */
                }
                
            #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
        }
        
        return (errStatus);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterWriteByte
    ********************************************************************************
    *
    * Summary:
    *  Sends one byte to a slave. A valid Start or ReStart condition must be 
    *  generated before this call this function. Function do nothing if Start or 
    *  Restart condition was failed before call this function.
    *
    * Parameters:
    *  data:  The data byte to send to the slave.
    *
    * Return:
    *  Status error - zero means no errors.
    *
    * Side Effects:
    *  This function is entered without a 'byte complete' bit set in the I2C_CSR 
    *  register. It does not exit until it will be set.
    *
    * Global variables:
    *  i2c_State - used to store current state of state machine.
    *
    *******************************************************************************/
    uint8 i2c_MasterWriteByte(uint8 theByte)
    {
        uint8 errStatus = i2c_MSTR_SLAVE_BUSY;
        
        /* Check if START condition was generated */
        if(i2c_CHECK_MASTER_MODE(i2c_MCSR_REG))
        {
            i2c_DATA_REG = theByte;        /* Write DATA register */
            i2c_TRANSMIT_DATA;             /* Set transmit mode */
            i2c_State = i2c_SM_MSTR_WR_DATA;  /* Set state WR_DATA */
            #if(i2c_IMPLEMENTATION == i2c_UDB)
                while(i2c_CHECK_BYTE_COMPLETE(i2c_CSR_REG));
            #endif
            
            /* Make sure the last byte has been transfered first */
            while(i2c_WAIT_BYTE_COMPLETE(i2c_CSR_REG));
            
        #if (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
                /* Check for LOST ARBITRATION */
                if(i2c_CHECK_LOST_ARB(i2c_CSR_REG))
                {
                    /* Arbitration has been lost, reset state machine to IDLE */
                    i2c_State = i2c_SM_IDLE;          /* Reset state to IDLE */
                    errStatus = i2c_MSTR_ERR_ARB_LOST;             /* The Master LOST ARBITRAGE */
                }
                /* Check LRB bit */
                else if(i2c_CHECK_DATA_ACK(i2c_CSR_REG))
                {
                    i2c_State = i2c_SM_MSTR_HALT;     /* Set state to HALT */
                    errStatus = i2c_MSTR_NO_ERROR;                 /* The LRB was ACKed */
                }
                else
                {
                    i2c_State = i2c_SM_MSTR_HALT;     /* Set state to HALT */
                    errStatus = i2c_MSTR_ERR_LB_NAK;               /* The LRB was NACKed */
                }
                
            #else
                
                i2c_State = i2c_SM_MSTR_HALT;     /* Set state HALT */
                
                /* Check LRB bit */
                if(i2c_CHECK_DATA_ACK(i2c_CSR_REG))
                {
                    errStatus = i2c_MSTR_NO_ERROR;             /* The LRB was ACKed */
                }
                else
                {
                    errStatus = i2c_MSTR_ERR_LB_NAK;           /* The LRB was NACKed */
                }
                
            #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
        }
        
        return (errStatus);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterReadByte
    ********************************************************************************
    *
    * Summary:
    *  Reads one byte from a slave and ACK or NACK the transfer. A valid Start or 
    *  ReStart condition must be generated before this call this function. Function
    *  do nothing if Start or Restart condition was failed before call this 
    *  function.
    *
    * Parameters:
    *  acknNack:  Zero, response with NACK, if non-zero response with ACK.
    *
    * Return:
    *  Byte read from slave.
    *
    * Side Effects:
    *  This function is entered without a 'byte complete' bit set in the I2C_CSR 
    *  register. It does not exit until it will be set.
    *
    * Global variables:
    *  i2c_State - used to store current state of state machine.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_MasterReadByte(uint8 acknNak)
    {
        uint8 theByte = 0u;
        
        /* Check if START condition was generated */
        if(i2c_CHECK_MASTER_MODE(i2c_MCSR_REG))
        {
            /* When address phase need release the bus and receive the byte, then decide ACK or NACK */
            if(i2c_State == i2c_SM_MSTR_RD_ADDR)
            {
                i2c_READY_TO_READ;
                i2c_State = i2c_SM_MSTR_DATA;
                #if (i2c_IMPLEMENTATION == i2c_UDB)
                    while(i2c_CHECK_BYTE_COMPLETE(i2c_CSR_REG));
                #endif /* End (i2c_IMPLEMENTATION == i2c_UDB) */
            }
            
            while(i2c_WAIT_BYTE_COMPLETE(i2c_CSR_REG));
            
            theByte = i2c_DATA_REG;
            
            /* Now if the ACK flag was set, ACK the data which will release the bus and start the next byte in
               otherwise do NOTHING to the CSR reg.
               This will allow the calling routine to generate a repeat start or a stop depending on it's preference. */
            if(acknNak != 0u)   /* Do ACK */
            {
                i2c_ACK_AND_RECEIVE;
                #if (i2c_IMPLEMENTATION == i2c_UDB)
                    while(i2c_CHECK_BYTE_COMPLETE(i2c_CSR_REG));
                #endif /* End (i2c_IMPLEMENTATION == i2c_UDB) */
            }
            else                /* Do NACK */
            {
                /* Do nothing to be able work with ReStart */
                i2c_State = i2c_SM_MSTR_HALT;
            }
        }
        
        return (theByte);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterGetReadBufSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the amount of bytes that has been transferred with an 
    *  I2C_MasterReadBuf command.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Byte count of transfer. If the transfer is not yet complete, it will return 
    *  the byte count transferred so far.
    *
    * Global variables:
    *  i2c_mstrRdBufIndex - used to current index within master read 
    *  buffer.
    *
    *******************************************************************************/
    uint16 i2c_MasterGetReadBufSize(void) 
    {
        return (i2c_mstrRdBufIndex);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterGetWriteBufSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the amount of bytes that has been transferred with an 
    *  I2C_MasterWriteBuf command.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Byte count of transfer. If the transfer is not yet complete, it will return 
    *  the byte count transferred so far.
    *
    * Global variables:
    *  i2c_mstrWrBufIndex - used to current index within master write 
    *  buffer.
    *
    *******************************************************************************/
    uint16 i2c_MasterGetWriteBufSize(void) 
    {
        return (i2c_mstrWrBufIndex);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterClearReadBuf
    ********************************************************************************
    *
    * Summary:
    *  Resets the read buffer pointer back to the first byte in the buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Global variables:
    *  i2c_mstrRdBufIndex - used to current index within master read 
    *   buffer.
    *  i2c_mstrStatus - used to store current status of I2C Master.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    void i2c_MasterClearReadBuf(void)
    {
        i2c_mstrRdBufIndex = 0u;
        i2c_mstrStatus &= ~i2c_MSTAT_RD_CMPLT;
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_MasterClearWriteBuf
    ********************************************************************************
    *
    * Summary:
    *  Resets the write buffer pointer back to the first byte in the buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Global variables:
    *  i2c_mstrRdBufIndex - used to current index within master read 
    *   buffer.
    *  i2c_mstrStatus - used to store current status of I2C Master.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    void i2c_MasterClearWriteBuf(void)
    {
        i2c_mstrWrBufIndex = 0u;
        i2c_mstrStatus &= ~i2c_MSTAT_WR_CMPLT;
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_Workaround
    ********************************************************************************
    *
    * Summary:
    *  Do nothing. This fake fuction use as workaround for CDT 78083.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    void i2c_Workaround(void)
    {
    
    }
    
#endif  /* End (i2c_MODE & i2c_MODE_MASTER) */


#if (i2c_MODE & i2c_MODE_SLAVE)

    /*******************************************************************************
    * Function Name: i2c_SlaveStatus
    ********************************************************************************
    *
    * Summary:
    *  Returns I2C slave's communication status.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Current status of I2C slave.
    *
    * Global variables:
    *  i2c_slStatus  - used to store current status of I2C slave.
    *
    *******************************************************************************/
    uint8 i2c_SlaveStatus(void)   
    {
        return (i2c_slStatus);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveClearReadStatus
    ********************************************************************************
    *
    * Summary:
    *  Clears the read status flags and returns they values. No other status flags 
    *  are affected.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Current read status of I2C slave.
    *
    * Global variables:
    *  i2c_slStatus  - used to store current status of I2C slave.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_SlaveClearReadStatus(void) 
    {
        uint8 status;
        
        status = i2c_slStatus & i2c_SSTAT_RD_MASK;
        
        /* Mask of transfer complete flag and Error status */
        i2c_slStatus &= ~i2c_SSTAT_RD_CLEAR;
        
        return (status);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveClearWriteStatus
    ********************************************************************************
    *
    * Summary:
    *  Clears the write status flags and returns they values. No other status flags
    *  are affected.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Current write status of I2C slave.
    *
    * Global variables:
    *  i2c_slStatus  - used to store current status of I2C slave.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    uint8 i2c_SlaveClearWriteStatus(void)
    {
        uint8 status;
        
        status = i2c_slStatus & i2c_SSTAT_WR_MASK;
        
        /* Mask of transfer complete flag and Error status */
        i2c_slStatus &= ~i2c_SSTAT_WR_CLEAR;
        
        return (status);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveSetAddress
    ********************************************************************************
    *
    * Summary:
    *  Sets the I2C slave address.
    *
    * Parameters:
    *  address: I2C slave address for the primary device. This value may be any 
    *  address between 0 and 127.
    *
    * Return:
    *  None
    *
    * Global variables:
    *  i2c_Address  - used to store I2C slave address for the primary 
    *  device when software address detect feature is used.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    #if (i2c_ADDR_DECODE == i2c_HDWR_DECODE)
        void i2c_SlaveSetAddress(uint8 address) 
        {
            i2c_ADDR_REG = address & i2c_SLAVE_ADDR_MASK; /* Set I2C Address register */
        }
        
    #else
        void i2c_SlaveSetAddress(uint8 address)
        {
            i2c_slAddress = address & i2c_SLAVE_ADDR_MASK;  /* Set Address variable */ 
        }
        
    #endif  /* End (i2c_ADDR_DECODE == i2c_HDWR_DECODE) */    
     
    
    /*******************************************************************************
    * Function Name: i2c_SlaveInitReadBuf
    ********************************************************************************
    *
    * Summary:
    *  Sets the buffer pointer and size of the read buffer. This function also 
    *  resets the transfer count returned with the I2C_SlaveGetReadBufSize function.
    *
    * Parameters:
    *  readBuf:  Pointer to the data buffer to be read by the master.
    *  bufSize:  Size of the read buffer exposed to the I2C master.
    *
    * Return:
    *  None
    *
    * Global variables:
    *  i2c_slRdBufPtr - used to store pointer to slave read buffer.
    *  i2c_slRdBufSize - used to store salve read buffer size.
    *  i2c_slRdBufIndex - used to store current index within slave
    *  read buffer.
    *
    * Side Effects:
    *  If this function is called during a bus transaction, data from the previous 
    *  buffer location and the beginning of current buffer may be transmitted.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    void i2c_SlaveInitReadBuf(uint8 * readBuf, uint8 bufSize)
    {
        i2c_slRdBufPtr   = (volatile uint8 *) readBuf;    /* Set buffer pointer */
        i2c_slRdBufSize  = bufSize;    /* Set buffer size */
        i2c_slRdBufIndex = 0u;         /* Clears buffer index */
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveInitWriteBuf
    ********************************************************************************
    *
    * Summary:
    *  Sets the buffer pointer and size of the read buffer. This function also 
    *  resets the transfer count returned with the I2C_SlaveGetReadBufSize function.
    *
    * Parameters:
    *  writeBuf:  Pointer to the data buffer to be read by the master.
    *  bufSize:  Size of the buffer exposed to the I2C master.
    *
    * Return:
    *  None
    *
    * Global variables:
    *  i2c_slWrBufPtr - used to store pointer to slave write buffer.
    *  i2c_slWrBufSize - used to store salve write buffer size.
    *  i2c_slWrBufIndex - used to store current index within slave
    *  write buffer.
    *
    * Side Effects:
    *  If this function is called during a bus transaction, data from the previous 
    *  buffer location and the beginning of current buffer may be transmitted.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    void i2c_SlaveInitWriteBuf(uint8 * writeBuf, uint8 bufSize)
    {
        i2c_slWrBufPtr   = (volatile uint8 *) writeBuf;  /* Set buffer pointer */
        i2c_slWrBufSize  = bufSize;   /* Set buffer size */
        i2c_slWrBufIndex = 0u;        /* Clears buffer index */
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveGetReadBufSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of bytes read by the I2C master since an 
    *  I2C_SlaveInitReadBuf or I2C_SlaveClearReadBuf function was executed. 
    *  The maximum return value will be the size of the read buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Bytes read by master.
    *
    * Global variables:
    *  i2c_slRdBufIndex - used to store current index within slave
    *  read buffer.
    *
    *******************************************************************************/
    uint8 i2c_SlaveGetReadBufSize(void) 
    {
        return (i2c_slRdBufIndex);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveGetWriteBufSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of bytes written by the I2C master since an 
    *  I2C_SlaveInitWriteBuf or I2C_SlaveClearWriteBuf function was executed.
    *  The maximum return value will be the size of the write buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Bytes written by master.
    *
    * Global variables:
    *  i2c_slWrBufIndex - used to store current index within slave
    *  write buffer.
    *
    *******************************************************************************/
    uint8 i2c_SlaveGetWriteBufSize(void) 
    {
        return (i2c_slWrBufIndex);
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveClearReadBuf
    ********************************************************************************
    *
    * Summary:
    *  Resets the read pointer to the first byte in the read buffer. The next byte 
    *  read by the master will be the first byte in the read buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Global variables:
    *  i2c_slRdBufIndex - used to store current index within slave
    *  read buffer.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    void i2c_SlaveClearReadBuf(void)
    {
        i2c_slRdBufIndex = 0u;
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveClearRxBuf
    ********************************************************************************
    *
    * Summary:
    *  Resets the write pointer to the first byte in the write buffer. The next byte
    *  written by the master will be the first byte in the write buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Global variables:
    *  i2c_slWrBufIndex - used to store current index within slave
    *  write buffer.
    *
    * Reentrant:
    *  No
    *
    *******************************************************************************/
    void i2c_SlaveClearWriteBuf(void)
    {
        i2c_slWrBufIndex = 0u;
    }
    
    /*******************************************************************************
    * Function Name: i2c_SlavePutReadByte
    ********************************************************************************
    *
    * Summary:
    *  For Master Read, sends 1 byte out Slave transmit buffer.
    *  Wait to send byte until buffer has room.  Used to preload
    *  the transmit buffer.
    *
    *  In byte by byte mode if the last byte was ACKed, stall the master
    *  (on the first bit of the next byte) if needed until the next byte
    *  is PutChared.  If the last byte was NAKed it does not stall the bus
    *  because the master will generate a stop or restart condition.
    *
    * Parameters:
    *  (uint8) transmitDataByte: Byte containing the data to transmit.
    *
    * Return:
    *  void
    *
    *******************************************************************************/
    void i2c_SlavePutReadByte(uint8 transmitDataByte) 
    {
        i2c_DATA_REG = transmitDataByte;
        i2c_ACK_AND_TRANSMIT;
    }
    
    
    /*******************************************************************************
    * Function Name: i2c_SlaveGetWriteByte
    ********************************************************************************
    *
    * Summary:
    *  For a Master Write, ACKs or NAKs the previous byte and reads out the last
    *  byte tranmitted.  The first byte read of a packet is the Address byte in 
    *  which case there is no previous data so no ACK or NAK is generated.  The
    *  bus is stalled until the next GetByte, therefore a GetByte must be executed
    *  after the last byte in order to send the final ACK or NAK before the Master
    *  can send a Stop or restart condition.
    *
    * Parameters:
    *  ackNak:  1 = ACK, 0 = NAK for the previous byte received.
    *
    * Return: 
    *  Last byte transmitted or last byte in buffer from Master.
    *
    *******************************************************************************/
    uint8 i2c_SlaveGetWriteByte(uint8 ackNak) 
    {
        uint8 dataByte;
    
        dataByte = i2c_DATA_REG;
        
        if(ackNak == i2c_ACK_DATA)
        {
            i2c_ACK_AND_RECEIVE;
        }
        else
        {
            i2c_NAK_AND_RECEIVE;
        }
        
        return (dataByte);
    }
#endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */


/* [] END OF FILE */
