/*******************************************************************************
* File Name: i2c.h  
* Version 2.10
*
* Description:
*  This file provides constants and parameter values for the I2C component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#if !defined(CY_I2C_i2c_H)
#define CY_I2C_i2c_H

#include "cytypes.h"
#include "cyfitter.h"
#include "CyLib.h"


/***************************************
*   Conditional Compilation Parameters
****************************************/

#define i2c_IMPLEMENTATION             (1u)
#define i2c_MODE                       (2u)
#define i2c_ADDR_DECODE                (1u)
#define i2c_ENABLE_WAKEUP              (0u)
#define i2c_I2C_PAIR_SELECTED          (0u)

/* I2C implementation types */
#define i2c_UDB                        (0x00u)
#define i2c_FF                         (0x01u)

/* I2C modes */
#define i2c_MODE_SLAVE                 (0x01u) /* I2C Slave Mode */
#define i2c_MODE_MASTER                (0x02u) /* I2C Master Mode */
#define i2c_MODE_MULTI_MASTER          (0x06u) /* I2C Multi-Master Mode */
#define i2c_MODE_MULTI_MASTER_SLAVE    (0x07u) /* I2C Multi-Master Slave Mode */
#define i2c_MODE_MULTI_MASTER_ENABLE   (0x04u) /* I2C Multi-Master Mode enable */

/* Address detection */
#define i2c_SW_DECODE                  (0x00u) /* Software address decode type */
#define i2c_HDWR_DECODE                (0x01u) /* Hardware address decode type */

#define i2c_I2C_PAIR0                  (0x01u) /* SIO pair 0 - P12[0] & P12[1] */
#define i2c_I2C_PAIR1                  (0x02u) /* SIO pair 1 - P12[4] & P12[5] */

/* PSoC3 ES2 or early */
#define i2c_PSOC3_ES2  ( (CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_3A) && \
                                      (CYDEV_CHIP_REVISION_USED <= CYDEV_CHIP_REVISION_3A_ES2) )
/* PSoC5 ES1 or early */
#define i2c_PSOC5_ES1  ( (CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_5A) && \
                                      (CYDEV_CHIP_REVISION_USED <= CYDEV_CHIP_REVISION_5A_ES1) )
/* PSoC3 ES3 or later */
#define i2c_PSOC3_ES3  ( (CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_3A) && \
                                      (CYDEV_CHIP_REVISION_USED >= CYDEV_CHIP_REVISION_3A_ES3) )
/* PSoC5 ES2 or later */
#define i2c_PSOC5_ES2  ( (CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_5A) && \
                                      (CYDEV_CHIP_REVISION_USED > CYDEV_CHIP_REVISION_5A_ES1) )


/***************************************
*       Type defines
***************************************/

/* Structure to save registers before go to sleep */
typedef struct _i2c_BACKUP_STRUCT
{
    uint8 enableState;
        
    #if((i2c_IMPLEMENTATION == i2c_FF) && (i2c_ENABLE_WAKEUP == 0u))
        uint8 xcfg;
        uint8 cfg;

        #if (i2c_MODE & i2c_MODE_SLAVE)
            uint8 addr;
        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
        
        #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
            uint8   clk_div;        /* only for TO3 */
        #else
            uint8   clk_div1;       /* only for TO4 */
            uint8   clk_div2;       /* only for TO4 */
        #endif  /* End  (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
        
    #else
        #if(i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
            uint8 int_mask;         /* Status interrupt mask register */
            
            #if(i2c_MODE & i2c_MODE_SLAVE)
                uint8 addr;         /* D0 register */
                                    /* Auxiliary Control register, clears on Stop() */
                                    /* Auxiliary Status register, disables after _Start() */
                                    /* Period Register = i2c_PERIOD_VALUE */
            #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
        #else
            /* ES3 Saves: 
                - Status Int mask: int_mask;
                - D0 register: addr;
                - Auxiliary Control: aux_ctl;
                - Period Register;
            */
        #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)*/
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF)*/
    
} i2c_BACKUP_STRUCT;


/***************************************
*        Function Prototypes 
***************************************/

void i2c_Init(void);
void i2c_Enable(void) ;

void i2c_Start(void);
void i2c_Stop(void) ;
void i2c_EnableInt(void)  ;
void i2c_DisableInt(void) ;

void i2c_SaveConfig(void);
void i2c_Sleep(void);
void i2c_RestoreConfig(void) ;
void i2c_Wakeup(void) ;

/* I2C Master functions prototypes */
#if(i2c_MODE & i2c_MODE_MASTER)
    /* Read and Clear status functions */
    uint8   i2c_MasterStatus(void) ;
    uint8   i2c_MasterClearStatus(void);
    
    /* Interrupt based operation functions */
    uint8   i2c_MasterWriteBuf(uint8 slaveAddr, uint8 * wrData, uint8 cnt, uint8 mode);
    uint8   i2c_MasterReadBuf(uint8 slaveAddr, uint8 * rdData, uint8 cnt, uint8 mode);
    uint16  i2c_MasterGetReadBufSize(void) ;
    uint16  i2c_MasterGetWriteBufSize(void)  ;
    void    i2c_MasterClearReadBuf(void);
    void    i2c_MasterClearWriteBuf(void);
    
    /* Manual operation functions */
    uint8   i2c_MasterSendStart(uint8 slaveAddress, uint8 R_nW);
    uint8   i2c_MasterSendRestart(uint8 slaveAddress, uint8 R_nW);
    uint8   i2c_MasterSendStop(void);
    uint8   i2c_MasterWriteByte(uint8 theByte);
    uint8   i2c_MasterReadByte(uint8 acknNak);
    
    /* This fake function use as workaround for CDT 78083 */
    void i2c_Workaround(void);
    
#endif  /* End (i2c_MODE & i2c_MODE_MASTER) */

/* I2C Slave functions prototypes */
#if(i2c_MODE & i2c_MODE_SLAVE)
    /* Read and Clear status functions */
    uint8 i2c_SlaveStatus(void) ;
    uint8 i2c_SlaveClearReadStatus(void);
    uint8 i2c_SlaveClearWriteStatus(void);
    #if (i2c_ADDR_DECODE == i2c_HDWR_DECODE)
        void i2c_SlaveSetAddress(uint8 address) ;
    #else
        void i2c_SlaveSetAddress(uint8 address);
    #endif  /* End (i2c_ADDR_DECODE == i2c_HDWR_DECODE) */ 
    
    /* Interrupt based operation functions */
    void i2c_SlaveInitReadBuf(uint8 * rdBuf, uint8 bufSize);
    void i2c_SlaveInitWriteBuf(uint8 * wrBuf, uint8 bufSize);
    uint8 i2c_SlaveGetReadBufSize(void) ;
    uint8 i2c_SlaveGetWriteBufSize(void) ;
    void i2c_SlaveClearReadBuf(void);
    void i2c_SlaveClearWriteBuf(void);
    
    void i2c_SlavePutReadByte(uint8 transmitDataByte) ;
    uint8 i2c_SlaveGetWriteByte(uint8 ackNak) ;

    /* Communication bootloader I2C Slave APIs */
    #if defined(CYDEV_BOOTLOADER_IO_COMP) && ((CYDEV_BOOTLOADER_IO_COMP == CyBtldr_i2c) || \
                                              (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_Custom_Interface))   
        /* Physical layer functions */
        void i2c_CyBtldrCommStart(void);
        void i2c_CyBtldrCommStop(void);
        void i2c_CyBtldrCommReset(void);
        cystatus i2c_CyBtldrCommWrite(uint8 * Data, uint16 Size, uint16 * Count, uint8 TimeOut);
        cystatus i2c_CyBtldrCommRead(uint8 * Data, uint16 Size, uint16 * Count, uint8 TimeOut);
        
        /* Size of Read/Write buffers for I2C bootloader  */
        #define i2c_BTLDR_SIZEOF_READ_BUFFER   (0x80u)
        #define i2c_BTLDR_SIZEOF_WRITE_BUFFER  (0x80u)
        
        #if (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_i2c)
            #define CyBtldrCommStart    i2c_CyBtldrCommStart
            #define CyBtldrCommStop     i2c_CyBtldrCommStop
            #define CyBtldrCommReset    i2c_CyBtldrCommReset
            #define CyBtldrCommWrite    i2c_CyBtldrCommWrite
            #define CyBtldrCommRead     i2c_CyBtldrCommRead
        #endif  /* End (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_i2c)*/
        
    #endif  /* End defined(CYDEV_BOOTLOADER_IO_COMP) && ((CYDEV_BOOTLOADER_IO_COMP == CyBtldr_i2c) || \
                                                         (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_Custom_Interface))*/

#endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */

/* Interrupt handler */
CY_ISR_PROTO(i2c_ISR);

#if(i2c_PSOC3_ES2 && (i2c_I2C_IRQ__ES2_PATCH))
    #include <intrins.h>
    #define i2c_ISR_PATCH() _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();
#endif  /* End (i2c_PSOC3_ES2 && (i2c_I2C_IRQ__ES2_PATCH)) */


/***************************************
*   Initial Parameter Constants 
***************************************/

#define i2c_DEFAULT_ADDR               (4u) 
#define i2c_BUS_SPEED                  (100u)

#define i2c_BUS_SPEED_50KHZ            (50u)
#define i2c_BUS_SPEED_100KHZ           (100u)

/*
    CLK_DIV = BUS_CLK(kHz) / (BusSpeed * OversampleRate);
    For Slave picks up the grater unsigned integer.
    For Master/MultiMaster/MultiMaster-Slave picks up the smallest unsigned integer(round).
    The OversampleRate equal 16 for BusSpeed >= 100, for others 32 (truncate).
    The real BusSpeed could be differ from desired due division with round/truncate.
*/
#if(i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
    /* Define CLK_DIV */
    #define i2c_DEFAULT_DIVIDE_FACTOR      (5u)
    
    /* Define proper clock rate according to Bus Speed */
    #if (i2c_BUS_SPEED <= i2c_BUS_SPEED_50KHZ)
        #define i2c_DEFAULT_CLK_RATE       i2c_CFG_CLK_RATE_050
    #elif (i2c_BUS_SPEED <= i2c_BUS_SPEED_100KHZ)
        #define i2c_DEFAULT_CLK_RATE       i2c_CFG_CLK_RATE_100
    #else
        #define i2c_DEFAULT_CLK_RATE       i2c_CFG_CLK_RATE_400
    #endif  /* End (i2c_BUS_SPEED <= i2c_BUS_SPEED_50KHZ) */
    
#else
    /* Define CLK_DIV1 and CLK_DIV2 */
    #define i2c_DEFAULT_DIVIDE_FACTOR      ((uint16) 30u )
    
    /* Define proper clock rate accordinf to Bus Speed */
    #if (i2c_BUS_SPEED <= i2c_BUS_SPEED_50KHZ)
        #define i2c_DEFAULT_CLK_RATE       i2c_CFG_CLK_RATE_LESS_EQUAL_50
    #else
        #define i2c_DEFAULT_CLK_RATE       i2c_CFG_CLK_RATE_GRATER_50
    #endif  /* End (i2c_BUS_SPEED <= i2c_BUS_SPEED_50KHZ) */
    
#endif /* End (I2C_PSOC3_ES2 || I2C_PSOC5_ES1) */


/***************************************
* I2C state machine constants 
***************************************/

/* Default slave address states */
#define  i2c_DEV_MASK                  (0xF0u)    /* Wait for sub-address */
#define  i2c_SM_IDLE                   (0x10u)    /* Idle I2C state */
#define  i2c_DEV_MASTER_XFER           (0x40u)    /* Wait for sub-address */

/* Default slave address states */
#define  i2c_SM_SL_WR_IDLE             (0x10u)    /* Slave Idle, waiting for start */
#define  i2c_SM_SL_WR_DATA             (0x11u)    /* Slave waiting for master to write data */
#define  i2c_SM_SL_RD_DATA             (0x12u)    /* Slave waiting for master to read data */
#define  i2c_SM_SL_STOP                (0x14u)    /* Slave waiting for stop */

/* Master mode states */
#define  i2c_SM_MASTER                 (0x40u)    /* Master or Multi-Master mode is set */
#define  i2c_SM_MASTER_IDLE            (0x40u)    /* Hardware in Master mode and sitting idle */

#define  i2c_SM_MSTR_ADDR              (0x43u)    /* Master has sent Start/Address */
#define  i2c_SM_MSTR_WR_ADDR           (0x42u)    /* Master has sent a Start/Address/WR */
#define  i2c_SM_MSTR_RD_ADDR           (0x43u)    /* Master has sent a Start/Address/RD */

#define  i2c_SM_MSTR_DATA              (0x44u)    /* Master is writing data to external slave */
#define  i2c_SM_MSTR_WR_DATA           (0x44u)    /* Master is writing data to external slave */
#define  i2c_SM_MSTR_RD_DATA           (0x45u)    /* Master is receiving data from external slave */

#define  i2c_SM_MSTR_WAIT_STOP         (0x48u)    /* Master Send Stop */

#define  i2c_SM_MSTR_HALT              (0x60u)    /* Master Halt state */


/***************************************
*            API Constants        
***************************************/

/* Master/Slave control constants */
#define i2c_READ_XFER_MODE             (0x01u)    /* Read */
#define i2c_WRITE_XFER_MODE            (0x00u)    /* Write */
#define i2c_ACK_DATA                   (0x01u)    /* Send ACK */
#define i2c_NAK_DATA                   (0x00u)    /* Send NAK */
    
#if(i2c_MODE & i2c_MODE_MASTER)

    /* "Mode" constants for MasterWriteBuf() or MasterReadBuf() function */
    #define i2c_MODE_COMPLETE_XFER     (0x00u)    /* Full transfer with Start and Stop */
    #define i2c_MODE_REPEAT_START      (0x01u)    /* Begin with a ReStart instead of a Start */
    #define i2c_MODE_NO_STOP           (0x02u)    /* Complete the transfer without a Stop */

    /* Master status */
    #define i2c_MSTAT_CLEAR            (0x00u)    /* Clear (init) status value */
    
    #define i2c_MSTAT_RD_CMPLT         (0x01u)    /* Read complete */
    #define i2c_MSTAT_WR_CMPLT         (0x02u)    /* Write complete */
    #define i2c_MSTAT_XFER_INP         (0x04u)    /* Master transfer in progress */
    #define i2c_MSTAT_XFER_HALT        (0x08u)    /* Transfer is halted */
    
    #define i2c_MSTAT_ERR_MASK         (0xF0u)    /* Mask for all errors */
    #define i2c_MSTAT_ERR_SHORT_XFER   (0x10u)    /* Master NAKed before end of packet */
    #define i2c_MSTAT_ERR_ADDR_NAK     (0x20u)    /* Slave did not ACK */
    #define i2c_MSTAT_ERR_ARB_LOST     (0x40u)    /* Master lost arbitration during communication */
    #define i2c_MSTAT_ERR_XFER         (0x80u)    /* Error during transfer */
    #define i2c_MSTAT_ERR_BUF_OVFL     (0x80u)    /* Buffer overflow/underflow */
    
    /* Master API returns */
    #define i2c_MSTR_NO_ERROR          (0x00u)    /* Function complete without error */
    #define i2c_MSTR_BUS_BUSY          (0x01u)    /* Bus is busy, process not started */
    #define i2c_MSTR_SLAVE_BUSY        (0x02u)    /* Master not Master on the bus or 
                                                                  Slave operation in progress */
    #define i2c_MSTR_ERR_LB_NAK        (0x03u)    /* Last Byte Naked */
    #define i2c_MSTR_ERR_ARB_LOST      (0x04u)    /* Master lost arbitration during communication */
    
    /* mstrControl bit definitions */
    #define  i2c_MSTR_GEN_STOP         (0x01u)    /* Generate a stop after a data transfer */
    #define  i2c_MSTR_NO_STOP          (0x01u)    /* Do not generate a stop after a data transfer */
    
    #define i2c_READ_FLAG              (0x01u)     /* Read flag of the Address */
    
#endif  /* End (i2c_MODE & i2c_MODE_MASTER) */

#if(i2c_MODE & i2c_MODE_SLAVE)

    /* Slave Status Constants */
    #define i2c_SSTAT_RD_CMPT          (0x01u)    /* Read transfer complete */
    #define i2c_SSTAT_RD_BUSY          (0x02u)    /* Read transfer in progress */
    #define i2c_SSTAT_RD_ERR           (0x08u)    /* Read Error buffer */
    #define i2c_SSTAT_RD_NO_ERR        (0x00u)    /* Read no Error */
    #define i2c_SSTAT_RD_ERR_OVFL      (0x04u)    /* Read overflow Error */
    #define i2c_SSTAT_RD_MASK          (0x0Fu)    /* Read Status Mask */
    
    #define i2c_SSTAT_WR_CMPT          (0x10u)    /* Write transfer complete */
    #define i2c_SSTAT_WR_BUSY          (0x20u)    /* Write transfer in progress */
    #define i2c_SSTAT_WR_ERR           (0xC0u)    /* Write Error buffer */
    #define i2c_SSTAT_WR_NO_ERR        (0x00u)    /* Write no Error */
    #define i2c_SSTAT_WR_ERR_OVFL      (0x40u)    /* Write overflow Error */
    #define i2c_SSTAT_WR_MASK          (0xF0u)    /* Write Status Mask  */
    
    #define i2c_SSTAT_RD_CLEAR         (0x0Du)    /* Read Status clear */
    #define i2c_SSTAT_WR_CLEAR         (0xD0u)    /* Write Status Clear */
        
#endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */


/***************************************
*              Registers
***************************************/

#if(i2c_IMPLEMENTATION == i2c_FF)

    /* Fixed Function registers */
    #define i2c_XCFG_REG               (*(reg8 *) i2c_I2C_FF__XCFG )
    #define i2c_XCFG_PTR               ( (reg8 *) i2c_I2C_FF__XCFG )
    
    #define i2c_ADDR_REG               (*(reg8 *) i2c_I2C_FF__ADR )
    #define i2c_ADDR_PTR               ( (reg8 *) i2c_I2C_FF__ADR )
    
    #define i2c_CFG_REG                (*(reg8 *) i2c_I2C_FF__CFG )
    #define i2c_CFG_PTR                ( (reg8 *) i2c_I2C_FF__CFG )
    
    #define i2c_CSR_REG                (*(reg8 *) i2c_I2C_FF__CSR )
    #define i2c_CSR_PTR                ( (reg8 *) i2c_I2C_FF__CSR )
    
    #define i2c_DATA_REG               (*(reg8 *) i2c_I2C_FF__D )
    #define i2c_DATA_PTR               ( (reg8 *) i2c_I2C_FF__D )
    
    #define i2c_MCSR_REG               (*(reg8 *) i2c_I2C_FF__MCSR )
    #define i2c_MCSR_PTR               ( (reg8 *) i2c_I2C_FF__MCSR )
    
    #define i2c_ACT_PWRMGR_REG         (*(reg8 *) i2c_I2C_FF__PM_ACT_CFG )
    #define i2c_ACT_PWRMGR_PTR         ( (reg8 *) i2c_I2C_FF__PM_ACT_CFG )
    #define i2c_ACT_PWR_EN                        i2c_I2C_FF__PM_ACT_MSK
    
    #define i2c_STBY_PWRMGR_REG        (*(reg8 *) i2c_I2C_FF__PM_STBY_CFG )
    #define i2c_STBY_PWRMGR_PTR        ( (reg8 *) i2c_I2C_FF__PM_STBY_CFG ) 
    #define i2c_STBY_PWR_EN                       i2c_I2C_FF__PM_STBY_MSK
    
    #define i2c_PWRSYS_CR1_REG         (*(reg8 *) CYREG_PWRSYS_CR1 )
    #define i2c_PWRSYS_CR1_PTR         ( (reg8 *) CYREG_PWRSYS_CR1 )
    
    /* Clock divider register depends on silicon */
    #if(i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
        #define i2c_CLKDIV_REG         (*(reg8 *) i2c_I2C_FF__CLK_DIV )
        #define i2c_CLKDIV_PTR         ( (reg8 *) i2c_I2C_FF__CLK_DIV )
        
    #else
        #define i2c_CLKDIV1_REG        (*(reg8 *) i2c_I2C_FF__CLK_DIV1 )
        #define i2c_CLKDIV1_PTR        ( (reg8 *) i2c_I2C_FF__CLK_DIV1 )
        #define i2c_CLKDIV2_REG        (*(reg8 *) i2c_I2C_FF__CLK_DIV2 )
        #define i2c_CLKDIV2_PTR        ( (reg8 *) i2c_I2C_FF__CLK_DIV2 )
        
    #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)*/
    
#else

    /* UDB implementation registers */
    #define i2c_CFG_REG    (*(reg8 *) \
                                           i2c_bI2C_UDB_AsyncCtl_CtrlReg__CONTROL_REG )
    #define i2c_CFG_PTR    ( (reg8 *) \
                                           i2c_bI2C_UDB_AsyncCtl_CtrlReg__CONTROL_REG )
    
    #define i2c_CSR_REG                (*(reg8 *) i2c_bI2C_UDB_StsReg__STATUS_REG )
    #define i2c_CSR_PTR                ( (reg8 *) i2c_bI2C_UDB_StsReg__STATUS_REG )
    
    #define i2c_INT_MASK_REG           (*(reg8 *) i2c_bI2C_UDB_StsReg__MASK_REG )
    #define i2c_INT_MASK_PTR           ( (reg8 *) i2c_bI2C_UDB_StsReg__MASK_REG )
    
    #define i2c_INT_ENABLE_REG         (*(reg8 *) i2c_bI2C_UDB_StsReg__STATUS_AUX_CTL_REG )
    #define i2c_INT_ENABLE_PTR         ( (reg8 *) i2c_bI2C_UDB_StsReg__STATUS_AUX_CTL_REG )
    
    #define i2c_DATA_REG               (*(reg8 *) i2c_bI2C_UDB_Shifter_u0__A0_REG )
    #define i2c_DATA_PTR               ( (reg8 *) i2c_bI2C_UDB_Shifter_u0__A0_REG )
    
    #define i2c_GO_REG                 (*(reg8 *) i2c_bI2C_UDB_Shifter_u0__F1_REG )
    #define i2c_GO_PTR                 ( (reg8 *) i2c_bI2C_UDB_Shifter_u0__F1_REG )
    
    #define i2c_MCLK_PRD_REG           (*(reg8 *) i2c_bI2C_UDB_Master_ClkGen_u0__D0_REG )
    #define i2c_MCLK_PRD_PTR           ( (reg8 *) i2c_bI2C_UDB_Master_ClkGen_u0__D0_REG )
    
    #define i2c_MCLK_CMP_REG           (*(reg8 *) i2c_bI2C_UDB_Master_ClkGen_u0__D1_REG )
    #define i2c_MCLK_CMP_PTR           ( (reg8 *) i2c_bI2C_UDB_Master_ClkGen_u0__D1_REG )
    
    #if(i2c_MODE & i2c_MODE_SLAVE)          
        
        /* UDB implementation registers --- Slave only */
        #define i2c_ADDR_REG       (*(reg8 *) i2c_bI2C_UDB_Shifter_u0__D0_REG )
        #define i2c_ADDR_PTR       ( (reg8 *) i2c_bI2C_UDB_Shifter_u0__D0_REG )
                                                                                              
        #define i2c_PERIOD_REG     (*(reg8 *) i2c_bI2C_UDB_Slave_BitCounter__PERIOD_REG )
        #define i2c_PERIOD_PTR     ( (reg8 *) i2c_bI2C_UDB_Slave_BitCounter__PERIOD_REG )
        
        #define i2c_COUNTER_REG    (*(reg8 *) i2c_bI2C_UDB_Slave_BitCounter__COUNT_REG )
        #define i2c_COUNTER_PTR    ( (reg8 *) i2c_bI2C_UDB_Slave_BitCounter__COUNT_REG )
        
        #define i2c_COUNTER_AUX_CTL_REG  (*(reg8 *) \
                                                       i2c_bI2C_UDB_Slave_BitCounter__CONTROL_AUX_CTL_REG )
        #define i2c_COUNTER_AUX_CTL_PTR  ( (reg8 *) \
                                                       i2c_bI2C_UDB_Slave_BitCounter__CONTROL_AUX_CTL_REG )
        
    #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
    
#endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */


/***************************************
*        Registers Constants       
***************************************/ 

/* Number and priority of the I2C interrupt */
#define i2c_ISR_NUMBER                 i2c_I2C_IRQ__INTC_NUMBER
#define i2c_ISR_PRIORITY               i2c_I2C_IRQ__INTC_PRIOR_NUM

/* I2C Slave Data Register */
#define i2c_SLAVE_ADDR_MASK            (0x7Fu)
#define i2c_DATA_MASK                  (0xFFu)
#define i2c_READ_FLAG                  (0x01u)
#if(i2c_IMPLEMENTATION == i2c_FF)
    
    /* XCFG I2C Extended Configuration Register */
    #define i2c_XCFG_CLK_EN            (0x80u)    /* Enable gated clock to block */
    #define i2c_XCFG_I2C_ON            (0x40u)    /* Enable I2C as wake up source*/
    #define i2c_XCFG_RDY_TO_SLEEP      (0x20u)    /* I2C ready go to sleep */
    #define i2c_XCFG_FORCE_NACK        (0x10u)    /* Force NACK all incomming transactions */
    #define i2c_XCFG_NO_BC_INT         (0x08u)    /* No interrupt on byte complete */
    #define i2c_XCFG_BUF_MODE          (0x02u)    /* Enable buffer mode */
    #define i2c_XCFG_HDWR_ADDR_EN      (0x01u)    /* Enable Hardware address match */
    
    /* CFG I2C Configuration Register */
    #define i2c_CFG_SIO_SELECT         (0x80u)    /* Pin Select for SCL/SDA lines */
    #define i2c_CFG_PSELECT            (0x40u)    /* Pin Select */
    #define i2c_CFG_BUS_ERR_IE         (0x20u)    /* Bus Error Interrupt Enable */
    #define i2c_CFG_STOP_IE            (0x10u)    /* Enable Interrupt on STOP condition */
    #define i2c_CFG_STOP_ERR_IE        (0x10u)    /* Enable Interrupt on STOP condition */
    #define i2c_CFG_CLK_RATE_MSK       (0x0Cu)    /* Clock rate select  **CHECK**  */
    #define i2c_CFG_CLK_RATE_100       (0x00u)    /* Clock rate select 100K */
    #define i2c_CFG_CLK_RATE_400       (0x04u)    /* Clock rate select 400K */
    #define i2c_CFG_CLK_RATE_050       (0x08u)    /* Clock rate select 50K  */
    #define i2c_CFG_CLK_RATE_RSVD      (0x0Cu)    /* Clock rate select Invalid */
    #define i2c_CFG_EN_MSTR            (0x02u)    /* Enable Master operation */
    #define i2c_CFG_EN_SLAVE           (0x01u)    /* Enable Slave operation */
    
    #define i2c_CFG_CLK_RATE_LESS_EQUAL_50 (0x04u) /* Clock rate select <= 50kHz */
    #define i2c_CFG_CLK_RATE_GRATER_50     (0x00u) /* Clock rate select > 50kHz */
   
    /* CSR I2C Control and Status Register */
    #define i2c_CSR_BUS_ERROR          (0x80u)    /* Active high when bus error has occured */
    #define i2c_CSR_LOST_ARB           (0x40u)    /* Set to 1 if lost arbitration in host mode */
    #define i2c_CSR_STOP_STATUS        (0x20u)    /* Set if Stop has been detected */
    #define i2c_CSR_ACK                (0x10u)    /* ACK response */
    #define i2c_CSR_NAK                (0x00u)    /* NAK response */
    #define i2c_CSR_ADDRESS            (0x08u)    /* Set in firmware 0 = status bit, 1 Address is slave */
    #define i2c_CSR_TRANSMIT           (0x04u)    /* Set in firmware 1 = transmit, 0 = receive */
    #define i2c_CSR_LRB                (0x02u)    /* Last received bit */
    #define i2c_CSR_LRB_ACK            (0x00u)    /* Last received bit was an ACK */
    #define i2c_CSR_LRB_NAK            (0x02u)    /* Last received bit was an NAK */
    #define i2c_CSR_BYTE_COMPLETE      (0x01u)    /* Informs that last byte has been sent */
    #define i2c_CSR_STOP_GEN           (0x00u)    /* Generate a stop condition */
    #define i2c_CSR_RDY_TO_RD          (0x00u)    /* Set to recieve mode */
      
    /* MCSR I2C Master Control and Status Register */
    #define i2c_MCSR_STOP_GEN          (0x10u)    /* Firmware sets this bit to initiate a Stop condition */
    #define i2c_MCSR_BUS_BUSY          (0x08u)    /* Status bit, Set at Start and cleared at Stop condition */
    #define i2c_MCSR_MSTR_MODE         (0x04u)    /* Status bit, Set at Start and cleared at Stop condition */
    #define i2c_MCSR_RESTART_GEN       (0x02u)    /* Firmware sets this bit to initiate a ReStart condition */
    #define i2c_MCSR_START_GEN         (0x01u)    /* Firmware sets this bit to initiate a Start condition */
    
    /* CLK_DIV I2C Clock Divide Factor Register */
    #define i2c_CLK_DIV_MSK            (0x07u)    /* Status bit, Set at Start and cleared at Stop condition */
    #define i2c_CLK_DIV_1              (0x00u)    /* Divide input clock by  1 */
    #define i2c_CLK_DIV_2              (0x01u)    /* Divide input clock by  2 */
    #define i2c_CLK_DIV_4              (0x02u)    /* Divide input clock by  4 */
    #define i2c_CLK_DIV_8              (0x03u)    /* Divide input clock by  8 */
    #define i2c_CLK_DIV_16             (0x04u)    /* Divide input clock by 16 */
    #define i2c_CLK_DIV_32             (0x05u)    /* Divide input clock by 32 */
    #define i2c_CLK_DIV_64             (0x06u)    /* Divide input clock by 64 */
    
    /* PWRSYS_CR1 to handle Sleep */
    #define i2c_PWRSYS_CR1_I2C_REG_BACKUP      (0x04u)    /* Enables, power to I2C regs while sleep */
    
    /* UDB compatible defines */
    #define i2c_DISABLE_INT_ON_STOP        { i2c_CFG_REG &= ~i2c_CFG_STOP_IE; }
    #define i2c_ENABLE_INT_ON_STOP         { i2c_CFG_REG |= i2c_CFG_STOP_IE; }
    
    #define i2c_TRANSMIT_DATA              { i2c_CSR_REG = i2c_CSR_TRANSMIT; }
    #define i2c_ACK_AND_TRANSMIT           { i2c_CSR_REG = (i2c_CSR_ACK | \
                                                                                      i2c_CSR_TRANSMIT); \
                                                        }
                                                        
    #define i2c_NAK_AND_TRANSMIT           { i2c_CSR_REG = i2c_CSR_NAK; }
    
    /* Special case: udb needs to ack, ff needs to nak. */
    #define i2c_ACKNAK_AND_TRANSMIT        { i2c_CSR_REG  = (i2c_CSR_NAK | \
                                                                                       i2c_CSR_TRANSMIT); \
                                                        }
    
    #define i2c_ACK_AND_RECEIVE            { i2c_CSR_REG = i2c_CSR_ACK; }
    #define i2c_NAK_AND_RECEIVE            { i2c_CSR_REG = i2c_CSR_NAK; }
    #define i2c_READY_TO_READ              { i2c_CSR_REG = i2c_CSR_RDY_TO_RD; }
        
    #define i2c_GENERATE_START             { i2c_MCSR_REG = i2c_MCSR_START_GEN; }
    
    #if(i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
        #define i2c_GENERATE_RESTART   { i2c_MCSR_REG = i2c_MCSR_RESTART_GEN; \
                                                      i2c_NAK_AND_RECEIVE; \
                                                    }
                                                    
        #define i2c_GENERATE_STOP      { i2c_CSR_REG = i2c_CSR_STOP_GEN; }
    
    #else   /* PSoC3 ES3 handlees zero lenght packets */
        #define i2c_GENERATE_RESTART   { i2c_MCSR_REG = (i2c_MCSR_RESTART_GEN | \
                                                                                   i2c_MCSR_STOP_GEN); \
                                                      i2c_TRANSMIT_DATA; \
                                                    }
                                                    
        #define i2c_GENERATE_STOP      { i2c_MCSR_REG = i2c_MCSR_STOP_GEN; \
                                                      i2c_TRANSMIT_DATA; }
    #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
    
    /* Conditions definitions */
    #define i2c_CHECK_ADDR_ACK(csr)    ( ((csr) & \
                                                      (i2c_CSR_LRB | i2c_CSR_ADDRESS)) == \
                                                      (i2c_CSR_LRB_ACK | i2c_CSR_ADDRESS) \
                                                    )
                                                    
    #define i2c_CHECK_ADDR_NAK(csr)    ( ((csr) & \
                                                      (i2c_CSR_LRB | i2c_CSR_ADDRESS)) == \
                                                      (i2c_CSR_LRB_NAK | i2c_CSR_ADDRESS) \
                                                    )
    
    #define i2c_CHECK_DATA_ACK(csr)    ( ((csr) & i2c_CSR_LRB) == \
                                                       i2c_CSR_LRB_ACK \
                                                    )

    #define i2c_CHECK_MASTER_MODE(mcsr)    ( ((mcsr) & i2c_MCSR_MSTR_MODE) != 0u )
    #define i2c_CHECK_BUS_FREE(mcsr)       ( ((mcsr) & i2c_MCSR_BUS_BUSY) == 0u )
    #define i2c_CHECK_LOST_ARB(csr)        ( ((csr) & i2c_CSR_LOST_ARB) != 0u )  
    #define i2c_CHECK_BYTE_COMPLETE(csr)   ( ((csr) & i2c_CSR_BYTE_COMPLETE) != 0u )
    #define i2c_WAIT_BYTE_COMPLETE(csr)    ( ((csr) & i2c_CSR_BYTE_COMPLETE) == 0u )
    #define i2c_CHECK_NO_STOP(mstrCntl)    ( ((mstrCntl) & i2c_MSTR_NO_STOP) != 0u )

#else
    /* Control Register Bit Locations */
    #if(i2c_MODE & i2c_MODE_MASTER)
    
        #define i2c_CTRL_STOP_SHIFT            i2c_bI2C_UDB_AsyncCtl_CtrlReg__6__POS
        #define i2c_CTRL_STOP_MASK             ( 0x01u << i2c_CTRL_STOP_SHIFT )
    
        #define i2c_CTRL_RESTART_SHIFT         i2c_bI2C_UDB_AsyncCtl_CtrlReg__5__POS
        #define i2c_CTRL_RESTART_MASK          ( 0x01u << i2c_CTRL_RESTART_SHIFT)

        #define i2c_CTRL_ENABLE_MASTER_SHIFT   i2c_bI2C_UDB_AsyncCtl_CtrlReg__1__POS
        #define i2c_CTRL_ENABLE_MASTER_MASK    ( 0x01u << i2c_CTRL_ENABLE_MASTER_SHIFT )
    
    #endif  /* End (i2c_MODE & i2c_MODE_MASTER) */
    
    #if(i2c_MODE & i2c_MODE_SLAVE)
               
        #define i2c_CTRL_ANY_ADDRESS_SHIFT     i2c_bI2C_UDB_AsyncCtl_CtrlReg__3__POS
        #define i2c_CTRL_ANY_ADDRESS_MASK      ( 0x01u << i2c_CTRL_ANY_ADDRESS_SHIFT )
     
        #define i2c_CTRL_ENABLE_SLAVE_SHIFT    i2c_bI2C_UDB_AsyncCtl_CtrlReg__0__POS
        #define i2c_CTRL_ENABLE_SLAVE_MASK     ( 0x01u << i2c_CTRL_ENABLE_SLAVE_SHIFT )
    
        #if(i2c_MODE == i2c_MODE_SLAVE)
            #define i2c_CTRL_RESTART_MASK          0x20u       /* NOT used in this mode */
        #endif  /* (i2c_MODE == i2c_MODE_SLAVE) */
        
    #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
    
    #define i2c_CTRL_TRANSMIT_SHIFT        i2c_bI2C_UDB_AsyncCtl_CtrlReg__2__POS
    #define i2c_CTRL_TRANSMIT_MASK         ( 0x01u << i2c_CTRL_TRANSMIT_SHIFT )
    
    #define i2c_CTRL_NACK_SHIFT            i2c_bI2C_UDB_AsyncCtl_CtrlReg__4__POS
    #define i2c_CTRL_NACK_MASK             ( 0x01u << i2c_CTRL_NACK_SHIFT )

    /* Status Register Bit Locations */
    #define i2c_STS_LOST_ARB_SHIFT         i2c_bI2C_UDB_StsReg__6__POS
    #define i2c_STS_LOST_ARB_MASK          ( 0x01u << i2c_STS_LOST_ARB_SHIFT )
    
    /* NOT available in Master mode */
    #if(i2c_MODE != i2c_MODE_MASTER)
        #define i2c_STS_STOP_SHIFT             i2c_bI2C_UDB_StsReg__5__POS
        #define i2c_STS_STOP_MASK              ( 0x01u << i2c_STS_STOP_SHIFT )
    #endif  /* End (i2c_MODE != i2c_MASTER)*/

    #define i2c_STS_BUSY_SHIFT             i2c_bI2C_UDB_StsReg__4__POS
    #define i2c_STS_BUSY_MASK              ( 0x01u << i2c_STS_BUSY_SHIFT )
    
    #define i2c_STS_ADDR_SHIFT             i2c_bI2C_UDB_StsReg__3__POS
    #define i2c_STS_ADDR_MASK              ( 0x01u << i2c_STS_ADDR_SHIFT )
    
    /* NOT available in Slave mode */
    #if(i2c_MODE != i2c_MODE_SLAVE)
        #define i2c_STS_MASTER_MODE_SHIFT      i2c_bI2C_UDB_StsReg__2__POS
        #define i2c_STS_MASTER_MODE_MASK      ( 0x01u << i2c_STS_MASTER_MODE_SHIFT )
    #endif  /* End (i2c_MODE != i2c_MODE_MASTER) */
    
    #define i2c_STS_LRB_SHIFT              i2c_bI2C_UDB_StsReg__1__POS
    #define i2c_STS_LRB_MASK               ( 0x01u << i2c_STS_LRB_SHIFT )

    #define i2c_STS_BYTE_COMPLETE_SHIFT    i2c_bI2C_UDB_StsReg__0__POS
    #define i2c_STS_BYTE_COMPLETE_MASK     ( 0x01u << i2c_STS_BYTE_COMPLETE_SHIFT )
    
    /* Master clock devider values */
    #define i2c_MCLK_PERIOD_VALUE      (0x0Fu)
    #define i2c_MCLK_COMPARE_VALUE     (0x08u)
    
    /* Enable counter and interrupts mask */
    #define i2c_COUNTER_ENABLE_MASK    (0x20u)     /* Enable counter7 */
    #define i2c_INT_ENABLE_MASK        (0x10u)     /* Enable interrupts */
    /* Conter period register value */ 
    #define i2c_PERIOD_VALUE           (0x07u)
     
    /* Masks to enalbe interrupts from Status register */
    #define i2c_STOP_IE_MASK               i2c_STS_STOP_MASK
    #define i2c_BYTE_COMPLETE_IE_MASK      i2c_STS_BYTE_COMPLETE_MASK
    
    /* Defines to make UDB match Fixed Function */
    #define i2c_CSR_LOST_ARB           i2c_STS_LOST_ARB_MASK          /* Set if Master Lost Arbitrage while send Start */
    #define i2c_CSR_STOP_STATUS        i2c_STS_STOP_MASK              /* Set if Stop has been detected */
    #define i2c_CSR_BUS_ERROR          (0x00u)                                     /* NOT used - Active high when bus error has occured */
    #define i2c_CSR_ADDRESS            i2c_STS_ADDR_MASK              /* Set in firmware 0 = status bit, 1 Address is slave */
    #define i2c_CSR_TRANSMIT           i2c_CTRL_TRANSMIT_MASK         /* Set in firmware 1 = transmit, 0 = receive. */
    #define i2c_CSR_LRB                i2c_STS_LRB_MASK               /* Last received bit */
    #define i2c_CSR_LRB_NAK            i2c_STS_LRB_MASK               /* Last received bit was an NAK */
    #define i2c_CSR_LRB_ACK            (0x00u)                                     /* Last received bit was an ACK */
    #define i2c_CSR_BYTE_COMPLETE      i2c_STS_BYTE_COMPLETE_MASK     /* Informs that last byte has been sent */
                
    /* MCSR Registers definitions */
    #if (i2c_MODE != i2c_MODE_SLAVE)
        #define i2c_MCSR_REG           i2c_CSR_REG
        #define i2c_MCSR_BUS_BUSY      i2c_STS_BUSY_MASK
        #define i2c_MCSR_START_GEN     (0x00u)             /* NOT used - always is 0 */
        #define i2c_MCSR_RESTART_GEN   i2c_CTRL_RESTART_MASK  /* Generates RESTART condition */
        #define i2c_MCSR_MSTR_MODE     i2c_STS_MASTER_MODE_MASK    /* Define if Master drives the bus */
    #endif  /* End (i2c_MODE & i2c_MODE_MASTER) */
      
    /* FF compatible defines */
    #define i2c_DISABLE_INT_ON_STOP    { i2c_INT_MASK_REG &= ~i2c_STOP_IE_MASK; }
    #define i2c_ENABLE_INT_ON_STOP     { i2c_INT_MASK_REG |= i2c_STOP_IE_MASK; }
    
    
    #define i2c_TRANSMIT_DATA      { i2c_CFG_REG |= i2c_CTRL_TRANSMIT_MASK; \
                                                  i2c_CFG_REG &= ~(i2c_CTRL_NACK_MASK | \
                                                                                i2c_CTRL_RESTART_MASK); \
                                                  i2c_GO_REG = 0x00u; \
                                                }
    
    #define i2c_ACK_AND_TRANSMIT   { i2c_CFG_REG |= i2c_CTRL_TRANSMIT_MASK; \
                                                  i2c_CFG_REG &= ~i2c_CTRL_NACK_MASK; \
                                                  i2c_GO_REG = 0x00u; \
                                                }
    
    #define i2c_NAK_AND_TRANSMIT   { i2c_CFG_REG |= i2c_CTRL_NACK_MASK; \
                                                  i2c_CFG_REG |= i2c_CTRL_TRANSMIT_MASK; \
                                                  i2c_GO_REG = 0x00u; \
                                                }
                                                          
    /* Special case: udb needs to ack, ff needs to nak. */
    #define i2c_ACKNAK_AND_TRANSMIT    { i2c_CFG_REG |= i2c_CTRL_TRANSMIT_MASK; \
                                                      i2c_CFG_REG &= ~i2c_CTRL_NACK_MASK; \
                                                      i2c_GO_REG = 0x00; \
                                                    }
    
    #define i2c_ACK_AND_RECEIVE    { i2c_CFG_REG &= ~(i2c_CTRL_TRANSMIT_MASK | \
                                                                                i2c_CTRL_NACK_MASK); \
                                                  i2c_GO_REG = 0x00u; \
                                                }
    
    #define i2c_NAK_AND_RECEIVE    { i2c_CFG_REG |= i2c_CTRL_NACK_MASK; \
                                                  i2c_CFG_REG &= ~i2c_CTRL_TRANSMIT_MASK; \
                                                  i2c_GO_REG = 0x00u; \
                                                }
    
    #define i2c_GENERATE_START     { i2c_CFG_REG &= ~(i2c_CTRL_TRANSMIT_MASK | \
                                                                                i2c_CTRL_NACK_MASK | \
                                                                                i2c_CTRL_STOP_MASK | \
                                                                                i2c_CTRL_RESTART_MASK); \
                                                  i2c_GO_REG = 0x00u; \
                                                }
                                                                  
    #define i2c_GENERATE_RESTART   { i2c_CFG_REG |= i2c_CTRL_RESTART_MASK; \
                                                  i2c_CFG_REG |= i2c_CTRL_NACK_MASK; \
                                                  i2c_CFG_REG &= ~(i2c_CTRL_TRANSMIT_MASK | \
                                                                                i2c_CTRL_STOP_MASK); \
                                                  i2c_GO_REG = 0x00u; \
                                                }
    
    #define i2c_GENERATE_STOP      { i2c_CFG_REG |= i2c_CTRL_STOP_MASK; \
                                                  i2c_CFG_REG |= i2c_CTRL_NACK_MASK; \
                                                  i2c_GO_REG = 0x00u; \
                                                }
        
    #define i2c_READY_TO_READ      { i2c_CFG_REG &= ~(i2c_CTRL_TRANSMIT_MASK | \
                                                                                i2c_CTRL_NACK_MASK | \
                                                                                i2c_CTRL_RESTART_MASK); \
                                                  i2c_GO_REG = 0x00u; \
                                                }
    /* Conditions definitions */
    #define i2c_CHECK_ADDR_ACK(csr)  ( ((csr) & i2c_CSR_LRB) == i2c_CSR_LRB_ACK )
    #define i2c_CHECK_ADDR_NAK(csr)  ( ((csr) & i2c_CSR_LRB) == i2c_CSR_LRB_NAK )
    #define i2c_CHECK_DATA_ACK(csr)  ( ((csr) & i2c_CSR_LRB) == i2c_CSR_LRB_ACK )
    #define i2c_CHECK_MASTER_MODE(mcsr)    ( ((mcsr) & i2c_MCSR_MSTR_MODE) != 0u )
    #define i2c_CHECK_BUS_FREE(mcsr)       ( ((mcsr) & i2c_MCSR_BUS_BUSY) == 0u )
    #define i2c_CHECK_LOST_ARB(csr)        ( ((csr) & i2c_CSR_LOST_ARB) != 0u)  
    #define i2c_CHECK_BYTE_COMPLETE(csr)   ( ((csr) & i2c_CSR_BYTE_COMPLETE) != 0u )
    #define i2c_WAIT_BYTE_COMPLETE(csr)    ( ((csr) & i2c_CSR_BYTE_COMPLETE) == 0u )
    #define i2c_CHECK_NO_STOP(mstrCntl)    ( ((mstrCntl) & i2c_MSTR_NO_STOP) != 0u )
    
#endif  /* End (i2c_IMPLEMENTATION == i2c_FF)*/
    
/* Create constansts to enable slave */
#if (i2c_MODE & i2c_MODE_SLAVE)
    #if(i2c_IMPLEMENTATION == i2c_FF)
        #define i2c_ENABLE_SLAVE  i2c_CFG_EN_SLAVE
        
    #else
        #define i2c_ENABLE_SLAVE  i2c_CTRL_ENABLE_SLAVE_MASK
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF)*/
    
#else
    #define  i2c_ENABLE_SLAVE    (0u)
    
#endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */

/* Create constansts to enable master */
#if (i2c_MODE & i2c_MODE_MASTER)
    #if (i2c_IMPLEMENTATION == i2c_FF)
        #define i2c_ENABLE_MASTER  i2c_CFG_EN_MSTR
        
    #else
        #define i2c_ENABLE_MASTER  i2c_CTRL_ENABLE_MASTER_MASK
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */
    
#else
    #define i2c_ENABLE_MASTER    (0u)
    
#endif  /* End (i2c_MODE & i2c_MODE_MASTER) */
 
#endif  /* End CY_I2C_i2c_H */


/* [] END OF FILE */
