/*******************************************************************************
* File Name: i2c_INT.c  
* Version 2.10
*
* Description:
*  This file provides the source code of Interrupt Service Routine (ISR)
*  for I2C component.
*
*   Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "i2c.h"  


/**********************************
*      System variables
**********************************/

volatile uint8 i2c_State;    /* Current state of I2C state machine */

/* Master variables */
#if (i2c_MODE & i2c_MODE_MASTER)
    volatile uint8 i2c_mstrStatus;           /* Master Status byte */
    volatile uint8 i2c_mstrControl;          /* Master Control byte */
    
    /* Transmit buffer variables */
    volatile uint8 * i2c_mstrRdBufPtr;       /* Pointer to Master Tx/Rx buffer */
    volatile uint8   i2c_mstrRdBufSize;      /* Master buffer size */
    volatile uint8   i2c_mstrRdBufIndex;     /* Master buffer Index */
    
    /* Receive buffer variables */
    volatile uint8 * i2c_mstrWrBufPtr;       /* Pointer to Master Tx/Rx buffer */
    volatile uint8   i2c_mstrWrBufSize;      /* Master buffer size  */
    volatile uint8   i2c_mstrWrBufIndex;     /* Master buffer Index */

#endif  /* End (i2c_MODE & i2c_MODE_MASTER) */

/* Slave variables */
#if (i2c_MODE & i2c_MODE_SLAVE)
    volatile uint8 i2c_slStatus;             /* Slave Status byte */

    #if (i2c_ADDR_DECODE == i2c_SW_DECODE)
        volatile uint8 i2c_slAddress;        /* Software address variable */
    #endif  /* End (i2c_ADDR_DECODE == i2c_SW_DECODE) */
    
    /* Transmit buffer variables */
    volatile uint8 * i2c_slRdBufPtr;         /* Pointer to Transmit buffer */
    volatile uint8   i2c_slRdBufSize;        /* Slave Transmit buffer size */
    volatile uint8   i2c_slRdBufIndex;       /* Slave Transmit buffer Index */
    
    /* Receive buffer variables */
    volatile uint8 * i2c_slWrBufPtr;         /* Pointer to Receive buffer */
    volatile uint8   i2c_slWrBufSize;        /* Slave Receive buffer size */
    volatile uint8   i2c_slWrBufIndex;       /* Slave Receive buffer Index */

#endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */


/*******************************************************************************
* Function Name: i2c_ISR
********************************************************************************
*
* Summary:
*  Handle Interrupt Service Routine.  
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  No
*
*******************************************************************************/
CY_ISR(i2c_ISR)
{
    #if (i2c_MODE & i2c_MODE_SLAVE)
       static uint8  tmp8;    /* Making these static so not wasting time allocating */
    #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
    
    static uint8  tmpCsr;  /* on the stack each time and no one else can see them */
    
    /* Entry from interrupt */
    /* In hardware address compare mode, we can assume we only get interrupted when */
    /* a valid address is recognized. In software address compare mode, we have to  */
    /* check every address after a start condition.                                 */
    
    tmpCsr = i2c_CSR_REG;          /* Make temp copy so that we can check */
                                                /* for stop condition after we are done */
    
    #if(i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
       
       /* MultiMaster-Slave */
        #if(i2c_MODE == i2c_MODE_MULTI_MASTER_SLAVE)
            /* Check if START has been generated */
            if((i2c_MCSR_REG & i2c_MCSR_START_GEN) != 0u)
            {
                /* Clear if was not generated */
                i2c_MCSR_REG &= ~i2c_MCSR_START_GEN;
            }
        #endif  /* End (i2c_MODE == i2c_MODE_MULTI_MASTER_SLAVE) */
                
        /* Check for loss of arbitration  */
        if(i2c_CHECK_LOST_ARB(tmpCsr))
        {
            /* MultiMaster-Slave */
            #if(i2c_MODE == i2c_MODE_MULTI_MASTER_SLAVE)    
            /* Check on which state of transaction lost ARBITRAGE:
              Address    - if Address and pass control to Slave
              No Address - release the bus */
            if((tmpCsr & i2c_CSR_ADDRESS) == 0u)
            {
                /* Lost ARBITRAGE:
                   On Adress, R/W bit, Data - reset state machine to IDLE
                   On ACK/NACK bit - pass control to SM_MSTR_WAIT_STOP state 
                                     to release the bus */
                if(i2c_State != i2c_SM_MSTR_WAIT_STOP)
                {
                    /* Clear ByteComplete to clear Enable Slave event */
                    tmpCsr &= ~i2c_CSR_BYTE_COMPLETE;
                    
                    /* Clear CSR to release the bus */
                    i2c_READY_TO_READ;
                    i2c_State = i2c_SM_IDLE;  /* Reset State Machine to IDLE */
                }
            }
            else
            {
                /* Let Slave decide the next step, the Stop status will be cleared if set */
                i2c_State = i2c_SM_IDLE;  /* Reset State Machine to IDLE */
            }
            
            /* MultiMaster */
            #else
                /* Lost ARBITRAGE:
                   On Adress, R/W bit, Data - reset state machine to IDLE
                   On ACK/NACK bit - pass control to SM_MSTR_WAIT_STOP state 
                                     to release the bus */
                if(i2c_State != i2c_SM_MSTR_WAIT_STOP)
                {
                    /* Clear CSR to release the bus, if MultiMaster */
                    i2c_READY_TO_READ;
                    i2c_State = i2c_SM_IDLE;  /* Reset State Machine to IDLE */
                }
            #endif  /* (i2c_MODE == i2c_MULTI_MASTER_ENABLE) */
            
            /* Set status error transfer and arbitration lost */
            i2c_mstrStatus |= (i2c_MSTAT_ERR_ARB_LOST | i2c_MSTAT_ERR_XFER);
        }
    #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
     
    /* Check for Master operation mode */
    if(i2c_State & i2c_SM_MASTER)    /*******  Master *******/
    {
        #if (i2c_MODE & i2c_MODE_MASTER)
        /* Enter Master state machine */
        
            if(i2c_CHECK_BYTE_COMPLETE(tmpCsr))
            {
                #if(i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
                    tmpCsr &= ~i2c_CSR_STOP_STATUS;  /* Clear STOP bit */
                #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
                
                switch(i2c_State)
                {
                    case i2c_SM_MSTR_WR_ADDR:    /* After address is sent, WRITE some data */
                    case i2c_SM_MSTR_RD_ADDR:    /* After address is sent, READ some data */
                    
                        /* Check for Slave address ACK */
                        if(i2c_CHECK_ADDR_ACK(tmpCsr))  /* Check ACK */
                        {
                            /* Setup for transmit or receive of data */
                            if(i2c_State == i2c_SM_MSTR_WR_ADDR)   /* TRANSMIT data */
                            {
                                if(i2c_mstrWrBufSize > 0)    /* Check if at least one byte is transfered */
                                {
                                    /* Load first data byte */
                                    i2c_DATA_REG = i2c_mstrWrBufPtr[0u];
                                    i2c_TRANSMIT_DATA;            /* Transmit data */
                                    i2c_mstrWrBufIndex = 1u;      /* Set index to 2nd location */
                                    
                                    /* Set transmit state until done */
                                    i2c_State = i2c_SM_MSTR_WR_DATA; 
                                }
                                else   /* No data to tranfer */
                                {
                                    /* Handles 0 bytes transfer */
                                    #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
                                        i2c_GENERATE_STOP;     /* Generate STOP */
                                        
                                        /* Set WRITE complete */
                                        i2c_mstrStatus |= i2c_MSTAT_WR_CMPLT;
                                        
                                        /* Reset State Machine to IDLE */
                                        i2c_State  = i2c_SM_IDLE; 
                                        
                                    #else  /* The PSoC3 ES3 only handles this well */
                                        if(i2c_CHECK_NO_STOP(i2c_mstrControl))
                                        {
                                            /* Reset State Machine to HALT, expect RESTART */
                                            i2c_State  = i2c_SM_MSTR_HALT;
                                            
                                            /* Set WRITE complete and Master HALTED */
                                            i2c_mstrStatus |= (i2c_MSTAT_XFER_HALT | 
                                                                            i2c_MSTAT_WR_CMPLT); 
                                            
                                            CyIntClearPending(i2c_ISR_NUMBER);
                                            i2c_DisableInt();
                                        }
                                        else  /* Do normal STOP */
                                        {
                                            i2c_GENERATE_STOP;     /* Generate STOP */
                                             
                                            /* Set WRITE complete */
                                            i2c_mstrStatus |= i2c_MSTAT_WR_CMPLT;
                                        
                                             /* Reset State Machine to IDLE */
                                            i2c_State = i2c_SM_IDLE;
                                        }
                                        
                                    #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
                                }
                            }
                            else  /* Master Receive data */
                            {                                
                                i2c_READY_TO_READ;     /* Ready to READ data */
                                
                                /* Set state machine to READ data */
                                i2c_State  = i2c_SM_MSTR_RD_DATA;
                            }
                        }
                        /* Check for Slave address NAK */
                        else if(i2c_CHECK_ADDR_NAK(tmpCsr))  /* Check NACK */
                        {
                            if(i2c_CHECK_NO_STOP(i2c_mstrControl))
                            {
                                /* Reset State Machine to HALT, expect RESTART */
                                i2c_State  = i2c_SM_MSTR_HALT;
                                
                                /* Set Address NAK Error and Master HALTED */
                                i2c_mstrStatus |= i2c_MSTAT_ERR_ADDR_NAK |
                                                               i2c_MSTAT_XFER_HALT;
                                
                                CyIntClearPending(i2c_ISR_NUMBER);
                                i2c_DisableInt();
                            }
                            else  /* Do normal Stop */
                            {
                                i2c_GENERATE_STOP;     /* Generate STOP */
                                i2c_State = i2c_SM_IDLE;  /* Reset State Machine to idle */
                                
                                /* Set Address NAK Error */
                                i2c_mstrStatus |= i2c_MSTAT_ERR_ADDR_NAK; 
                            }
                        }
                        /* Bogus */
                        else
                        {
                            CyIntClearPending(i2c_ISR_NUMBER);
                            i2c_DisableInt();
                        }
                        break;
                        
                    case i2c_SM_MSTR_WR_DATA:    /* Write data to slave */
                        
                        if(i2c_CHECK_DATA_ACK(tmpCsr))       /* Check ACK */
                        {
                            if(i2c_mstrWrBufIndex  < i2c_mstrWrBufSize) /* Check if end buf */
                            {
                                 /* Load first data byte  */
                                i2c_DATA_REG = i2c_mstrWrBufPtr[i2c_mstrWrBufIndex];
                                i2c_TRANSMIT_DATA;     /* Transmit */
                                
                                i2c_mstrWrBufIndex++;  /* Advance to data location */
                            }
                            else   /* Last byte was transmitted, send STOP */
                            {
                                if(i2c_CHECK_NO_STOP(i2c_mstrControl))
                                {
                                    /* Reset State Machine to HALT, expect RESTART */
                                    i2c_State  = i2c_SM_MSTR_HALT;
                                    
                                    /* Set WRITE complete and Master HALTED */
                                    i2c_mstrStatus |= (i2c_MSTAT_WR_CMPLT |
                                                                    i2c_MSTAT_XFER_HALT);
                                    
                                    CyIntClearPending(i2c_ISR_NUMBER);
                                    i2c_DisableInt();
                                }
                                else  /* Do normal Stop */
                                {
                                    i2c_Workaround();      /* Workaround for CDT 78083 */
                                    i2c_GENERATE_STOP;     /* Generate STOP */
                                    i2c_State = i2c_SM_IDLE; /* Reset State Machine to idle */
                                    
                                    /* Set WRITE complete */
                                    i2c_mstrStatus |= i2c_MSTAT_WR_CMPLT;
                                }
                            }
                        }
                        else /* If last byte NAKed, stop transmit and send STOP */
                        {
                            if(i2c_CHECK_NO_STOP(i2c_mstrControl)) /* Check STOP generation */
                            {
                                /* Reset State Machine to HALT, expect RESTART */
                                i2c_State  = i2c_SM_MSTR_HALT;
                                
                                /* Set WRITE complete, SHORT transfer and Master HALTED */
                                i2c_mstrStatus |= (i2c_MSTAT_WR_CMPLT |
                                                                i2c_MSTAT_ERR_SHORT_XFER |
                                                                i2c_MSTAT_XFER_HALT);
                                
                                CyIntClearPending(i2c_ISR_NUMBER);  
                                i2c_DisableInt();
                            }
                            else  /* Do normal Stop */
                            {
                                i2c_GENERATE_STOP;         /* Generate STOP */
                                i2c_State = i2c_SM_IDLE;    /* Reset State Machine to IDLE */
                                
                                /* Set WRITE complete and SHORT transfer */
                                i2c_mstrStatus |= (i2c_MSTAT_WR_CMPLT | 
                                                                i2c_MSTAT_ERR_SHORT_XFER);
                            }
                        }
                        break;
                        
                    case i2c_SM_MSTR_RD_DATA:    /* Data received */
                        
                        i2c_mstrRdBufPtr[i2c_mstrRdBufIndex] = i2c_DATA_REG;
                        i2c_mstrRdBufIndex++;      /* Inc pointer */
                        if(i2c_mstrRdBufIndex < i2c_mstrRdBufSize)/* Check if end of buffer */
                        {
                            i2c_ACK_AND_RECEIVE;       /* ACK and receive */
                        }
                        else   /* End of data, generate a STOP */
                        {
                            if(i2c_CHECK_NO_STOP(i2c_mstrControl)) /* Check STOP generation */
                            {
                                /* Reset State Machine to HALT, expect RESTART */
                                i2c_State = i2c_SM_MSTR_HALT;    
                                
                                /* Set READ complete and Master HALTED */
                                i2c_mstrStatus |= (i2c_MSTAT_RD_CMPLT |
                                                                i2c_MSTAT_XFER_HALT );
                            }
                            else   /* Do normal Stop */
                            {
                                #if(i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
                                    i2c_NAK_AND_RECEIVE;           /* NACK and TRY to generate STOP */
                                    i2c_ENABLE_INT_ON_STOP;        /* Enable interrupt on STOP */
                                     /* Wait for STOP or LOST ARBITRAGE */
                                    i2c_State = i2c_SM_MSTR_WAIT_STOP;
                                    
                                #else
                                    i2c_NAK_AND_RECEIVE;                   /* NACK and and generate STOP */
                                    i2c_State = i2c_SM_IDLE;  /* Set state to IDLE */
                                    
                                    /* Set READ complete */
                                    i2c_mstrStatus |= i2c_MSTAT_RD_CMPLT;
                                    
                                #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
                            }
                        }
                        break;
                        
                    /* Only wait for STOP if MultiMaster is used */
                    #if(i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
                        case i2c_SM_MSTR_WAIT_STOP:
                            i2c_READY_TO_READ;         /* The ARBITRAGE has been lost, release the bus */
                            i2c_DISABLE_INT_ON_STOP;                        /* Disable interrupt on STOP */
                            i2c_State = i2c_SM_IDLE;           /* Set state to IDLE */
                            /* Set status to complete READ */
                            i2c_mstrStatus |= i2c_MSTAT_RD_CMPLT;
                            break;
                    #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
                                            
                    /* This case used for NO STOP condition, ready for a restart */
                    case i2c_SM_MSTR_HALT: /* Do one transfer and halt, used for polling or single stepping */
                        i2c_mstrStatus |= i2c_MSTAT_WR_CMPLT;
                        i2c_DisableInt();
                        break;
                        
                    default:
                        /* Invalid state, reset state machine  to a known state */
                        i2c_State = i2c_SM_IDLE;
                        
                        /* Set transfer complete with error */
                        i2c_mstrStatus |= i2c_MSTAT_ERR_XFER;
                        break;
                }
            }
            
            #if(i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE)
                /* Check if STOP was detected */
                if((tmpCsr & i2c_CSR_STOP_STATUS) != 0u)
                {
                    /* The READ transaction only gets here on STOP */
                    i2c_DISABLE_INT_ON_STOP;
                    i2c_State = i2c_SM_IDLE;  /* Set state to IDLE */
                    
                    /* Set status to complete READ */
                    i2c_mstrStatus |= i2c_MSTAT_RD_CMPLT;
                }
            #endif  /* End (i2c_MODE & i2c_MODE_MULTI_MASTER_ENABLE) */
            
        #endif  /* End (i2c_MODE & i2c_MODE_MASTER) */
    }
    else    /******** Slave Mode ********/
    {
        #if (i2c_MODE & i2c_MODE_SLAVE)
            /* Check to see if a Start/Address is detected */
            if((tmpCsr & i2c_CSR_ADDRESS) != 0u)
            {
                /* Clears CSR STOP_STATUS bit. This bit sets by ANY of Stop condition detection on the bus */
                tmpCsr &= ~i2c_CSR_STOP_STATUS;  /* Clear Stop bit */
                
                /* This is a Start or ReStart, So Reset the state machine, Check for a Read/Write condition */
                    
                /* Check for software address detection */
                #if (i2c_ADDR_DECODE == i2c_SW_DECODE)  
                    /******************* Software address detection ************************/
                    tmp8 = ((i2c_DATA_REG >> 1u) & i2c_SLAVE_ADDR_MASK);
                    if(tmp8 == i2c_slAddress)   /* Check for address match  */
                    {
                        /* Check for read or write command */
                        if((i2c_DATA_REG & i2c_READ_FLAG) != 0u) 
                        {
                            /*******************************************/
                            /*  Place code to prepare read buffer here */
                            /*******************************************/
                            /* `#START SW_PREPARE_READ_BUF` */

                            /* `#END`  */
                            
                            /* Prepare next opeation to read, get data and place in data register */
                            if(i2c_slRdBufIndex < i2c_slRdBufSize)
                            {
                                /* Load first data byte */
                                i2c_DATA_REG = i2c_slRdBufPtr[i2c_slRdBufIndex];
                                i2c_ACK_AND_TRANSMIT;  /* ACK and transmit */
                                i2c_slRdBufIndex++;    /* Advance to data location */
                                
                                /* Set Read activity */
                                i2c_slStatus |= i2c_SSTAT_RD_BUSY; 
                            }
                            else    /* Data overflow */
                            {
                                i2c_DATA_REG = 0xFFu;    /* Out of range, send 0xFF  */
                                i2c_ACK_AND_TRANSMIT;    /* ACK and transmit */
                                
                                /* Set Read activity */
                                i2c_slStatus  |= (i2c_SSTAT_RD_BUSY | 
                                                               i2c_SSTAT_RD_ERR_OVFL); 
                            }
                            
                            i2c_State = i2c_SM_SL_RD_DATA; /* Prepare for Read transaction */
                        }
                        else  /* Start of a Write transaction, ready to write of the first byte */
                        {
                            /* Prepare to write the first byte */
                            i2c_ACK_AND_RECEIVE;
                            i2c_State = i2c_SM_SL_WR_DATA; /* Prepare for Write transaction */
                            
                            /* Set Write activity */
                            i2c_slStatus |= i2c_SSTAT_WR_BUSY;
                            i2c_ENABLE_INT_ON_STOP;    /* Enable interrupt on Stop */
                        }
                    }
                    /**********************************************/
                    /* Place compare for additional address here  */
                    /**********************************************/
                    /* `#START ADDR_COMPARE`  */

                    /* `#END`  */
                    
                    else   /* No address match */
                    {
                        i2c_NAK_AND_RECEIVE;   /* NAK address */
                    }
                    
                #else  /* Hardware address detection */
                    /* Check for read or write command */
                    if((i2c_DATA_REG & i2c_READ_FLAG) != 0u)
                    {
                        /*******************************************/
                        /*  Place code to prepare read buffer here */
                        /*******************************************/
                        /* `#START SW_PREPARE_READ_BUF` */

                        /* `#END`  */
                         
                        /* Prepare next opeation to read, get data and place in data register */
                        if(i2c_slRdBufIndex < i2c_slRdBufSize)
                        {
                            /* Load first data byte */
                            i2c_DATA_REG = i2c_slRdBufPtr[i2c_slRdBufIndex];
                            i2c_ACK_AND_TRANSMIT;                            /* ACK and transmit */
                            i2c_slRdBufIndex++;                              /* Advance to data location */
                            
                            /* Set Read activity */
                            i2c_slStatus  |= i2c_SSTAT_RD_BUSY;
                        }
                        else    /* Data overflow */
                        {
                            i2c_DATA_REG = 0xFFu;    /* Out of range, send 0xFF  */
                            i2c_ACK_AND_TRANSMIT;    /* ACK and transmit */
                            
                            /* Set Read activity */
                            i2c_slStatus  |= (i2c_SSTAT_RD_BUSY |
                                                           i2c_SSTAT_RD_ERR_OVFL);
                        }
                        
                        i2c_State = i2c_SM_SL_RD_DATA;    /* Prepare for Read transaction */
                    }
                    else  /* Start of a Write transaction, ready to write of the first byte */
                    {
                        /* Prepare to write the first byte */
                        i2c_ACK_AND_RECEIVE;       /* ACK and ready to receive addr */
                        i2c_State = i2c_SM_SL_WR_DATA;    /* Prepare for write transaction */
                        
                        /* Set Write activity */
                        i2c_slStatus |= i2c_SSTAT_WR_BUSY;
                        i2c_ENABLE_INT_ON_STOP;    /* Enable interrupt on Stop */
                    }
                    
                #endif  /* End (i2c_ADDR_DECODE == i2c_SW_DECODE) */
            }
            else if(i2c_CHECK_BYTE_COMPLETE(tmpCsr))    /* Check for data transfer */
            {
                if (i2c_State == i2c_SM_SL_WR_DATA)    /* Data write from Master to Slave */
                {
                    if(i2c_slWrBufIndex < i2c_slWrBufSize)       /* Check for valid range */
                    {
                        tmp8 = i2c_DATA_REG;                        /* Get data, to ACK quickly */
                        i2c_ACK_AND_RECEIVE;                        /* ACK and ready to receive */
                        i2c_slWrBufPtr[i2c_slWrBufIndex] = tmp8; /* Write data to array */
                        i2c_slWrBufIndex++;                        /* Inc pointer */
                    }
                    else
                    {
                        i2c_NAK_AND_RECEIVE;       /* NAK cause beyond write area */
                        
                        /* Set overflow, write completes on Stop */
                        i2c_slStatus |= i2c_SSTAT_WR_ERR_OVFL;
                    }
                }
                else if (i2c_State == i2c_SM_SL_RD_DATA) /* Data Read from Slave to Master */
                {
                    if(i2c_CHECK_DATA_ACK(tmpCsr))
                    {
                        if(i2c_slRdBufIndex < i2c_slRdBufSize) 
                        {
                             /* Get data from array */
                            i2c_DATA_REG = i2c_slRdBufPtr[i2c_slRdBufIndex];
                            i2c_TRANSMIT_DATA;         /* Send Data */
                            i2c_slRdBufIndex++;        /* Inc pointer */
                        }
                        else   /* Over flow */
                        {
                            i2c_DATA_REG = 0xFFu;          /* Send 0xFF at the end of the buffer */
                            i2c_TRANSMIT_DATA;             /* Send Data */
                            
                            /* Set overflow */
                            i2c_slStatus |= i2c_SSTAT_RD_ERR_OVFL;
                        }
                    }
                    else  /* Last byte NAKed, done */
                    {
                        i2c_DATA_REG = 0xFFu;      /* End of read transaction */
                        
                        i2c_NAK_AND_TRANSMIT;      /* Clear transmit bit at the end of read transaction */
                         /* Clear Busy Flag */
                        i2c_slStatus &= ~i2c_SSTAT_RD_BUSY;
                        /* Set Read complete Flag */
                        i2c_slStatus |= i2c_SSTAT_RD_CMPT;
                        
                        i2c_State = i2c_SM_IDLE;      /* Return to IDLE state */
                    }
                }
                else  /* This is an invalid state and should not occur */
                {
                    /* Invalid state, Reset */
                    i2c_State = i2c_SM_IDLE;
                    i2c_NAK_AND_RECEIVE;
                }   /* End Transfer mode */
            }  
            else if ((tmpCsr & i2c_CSR_BUS_ERROR) != 0u)    /* Quick check for Error */
            {
                if((i2c_CSR_REG & i2c_CSR_BUS_ERROR) != 0u)
                {
                    /* May want to reset bus here CHECK */
                }
            }  /* End if without else */
            
            if((tmpCsr & i2c_CSR_STOP_STATUS) != 0u)    /* Check if STOP was detected */
            {
                /* The Write transaction only IE on Stop, so Read never gets here */
                /* The WR_BUSY flag will be cleared at the end of "Write-ReStart-Read-Stop" transaction */
                /* Clear Busy Flag */
                i2c_slStatus &= ~i2c_SSTAT_WR_BUSY;
                /* Set complete Flag */
                i2c_slStatus |= i2c_SSTAT_WR_CMPT;
                
                i2c_DISABLE_INT_ON_STOP;               /* Disable interrupt on STOP */
                i2c_State = i2c_SM_IDLE;  /* Return to IDLE */
            }
        
        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
    }
    
    #if (i2c_PSOC3_ES2 && (i2c_I2C_IRQ__ES2_PATCH))
        i2c_ISR_PATCH();
    #endif  /* End (i2c_PSOC3_ES2 && (i2c_I2C_IRQ__ES2_PATCH)) */
}


/* [] END OF FILE */
