/*******************************************************************************
* File Name: i2c_PM.c
* Version 2.10
*
* Description:
*  This file provides Low power mode APIs for I2C component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "i2c.h"

#if (i2c_ADDR_DECODE == i2c_HDWR_DECODE)
    #if (i2c_ENABLE_WAKEUP)
        #define  i2c_DEFAULT_XCFG  (i2c_XCFG_HDWR_ADDR_EN | i2c_XCFG_CLK_EN | \
                                                 i2c_XCFG_I2C_ON)
    #else
        #define  i2c_DEFAULT_XCFG  (i2c_XCFG_HDWR_ADDR_EN | i2c_XCFG_CLK_EN)
    #endif  /* End (i2c_ENABLE_WAKEUP) */
#else
    #define  i2c_DEFAULT_XCFG      i2c_XCFG_CLK_EN
#endif  /* End (i2c_ADDR_DECODE == i2c_HDWR_DECODE) */

#if (i2c_ENABLE_WAKEUP)
    #if (i2c_I2C_PAIR_SELECTED == i2c_I2C_PAIR0)
        #define i2c_DEFAULT_CFG     (i2c_ENABLE_SLAVE | i2c_ENABLE_MASTER | \
                                                  i2c_DEFAULT_CLK_RATE | i2c_CFG_PSELECT | \
                                                  i2c_CFG_SIO_SELECT)
    #else
        #define i2c_DEFAULT_CFG     (i2c_ENABLE_SLAVE | i2c_ENABLE_MASTER | \
                                                  i2c_DEFAULT_CLK_RATE | i2c_CFG_PSELECT)
    #endif  /*  End (i2c_I2C_PAIR_SELECTED == i2c_I2C_PAIR0)*/
#else
    #define i2c_DEFAULT_CFG     (i2c_ENABLE_SLAVE | i2c_ENABLE_MASTER | \
                                              i2c_DEFAULT_CLK_RATE)
#endif  /* End (i2c_ENABLE_WAKEUP) */

/* Define active state */
#if(i2c_IMPLEMENTATION == i2c_FF)
    #define i2c_I2C_ENABLE_REG     i2c_ACT_PWRMGR_REG
    #define i2c_IS_I2C_ENABLE(reg) ( ((reg) & i2c_ACT_PWR_EN) != 0u )
#else
    #define i2c_I2C_ENABLE_REG     i2c_CFG_REG
    #define i2c_IS_I2C_ENABLE(reg) ( (((reg) & i2c_ENABLE_MASTER) != 0u) || \
                                                  (((reg)  & i2c_ENABLE_SLAVE) != 0u))
#endif  /* End (i2c_IMPLEMENTATION == i2c_FF)*/

i2c_BACKUP_STRUCT i2c_backup =
{   
    0x00u, /* enableState; */
    
    #if ((i2c_IMPLEMENTATION == i2c_FF) && (i2c_ENABLE_WAKEUP == 0u))
        i2c_DEFAULT_XCFG, /* xcfg */
        i2c_DEFAULT_CFG,  /* cfg */
        
        #if (i2c_MODE & i2c_MODE_SLAVE)
            (i2c_DEFAULT_ADDR & i2c_SLAVE_ADDR_MASK), /* addr */
        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
        
        #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
           i2c_DEFAULT_DIVIDE_FACTOR,
        #else
            LO8(i2c_DEFAULT_DIVIDE_FACTOR), /*  clk_div1 */
            HI8(i2c_DEFAULT_DIVIDE_FACTOR), /*  clk_div2 */
        #endif  /* End  (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
        
    #else
        #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
            i2c_INT_ENABLE_MASK, /* int_mask */
            
            #if (i2c_MODE & i2c_MODE_SLAVE)
                (i2c_DEFAULT_ADDR & i2c_SLAVE_ADDR_MASK), /* addr */
            #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
        #else
            /* ES3 Saves: 
                - Status Int mask: int_mask;
                - D0 register: addr;
                - Auxiliary Control: aux_ctl;
                - Period Register;
            */
        #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)*/
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF)*/
};


/*******************************************************************************
* Function Name: i2c_SaveConfig
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: disables I2C Master(if was enabled before go
*  to sleep), enables I2C backup regulator. Waits while on-going transaction be 
*  will completed and I2C will be ready go to sleep. All incoming transaction 
*  will be NACKed till power down will be asserted. The address match event 
*  wakes up the chip. 
*  Wakeup on address match disabled: saves I2C configuration and non-retention 
*  register values.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  i2c_backup - used to save component configuration and none-retention
*  registers before enter sleep mode.
*
* Reentrant:
*  No
*
*******************************************************************************/
void i2c_SaveConfig(void)
{     
    #if (i2c_IMPLEMENTATION == i2c_FF)
       
        #if (i2c_ENABLE_WAKEUP)
            uint8 enableInterrupts;
            
            /* Need to disable Master */
            #if (i2c_MODE & i2c_MODE_MASTER)
                i2c_CFG_REG &= ~i2c_CFG_EN_MSTR;
            #endif  /* End (i2c_MODE & i2c_MODE_MASTER)*/
            
            /* Enable the I2C regulator backup */
            enableInterrupts = CyEnterCriticalSection();
            i2c_PWRSYS_CR1_REG |= i2c_PWRSYS_CR1_I2C_REG_BACKUP;
            CyExitCriticalSection(enableInterrupts);
            
            /* 1) Set force NACK to ignore I2C transactions 
               2) Wait while I2C will be ready go to Sleep 
               3) These bits are cleared on wake up */
            i2c_XCFG_REG |= i2c_XCFG_FORCE_NACK;
            while ((i2c_XCFG_REG  & i2c_XCFG_RDY_TO_SLEEP) == 0u);
            
        #else
            i2c_backup.xcfg = i2c_XCFG_REG ;
            i2c_backup.cfg  = i2c_CFG_REG;
            
            #if (i2c_MODE & i2c_MODE_SLAVE)
                i2c_backup.addr = i2c_ADDR_REG;
            #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
            
            #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
                i2c_backup.clk_div  = i2c_CLKDIV_REG;
            
            #else
                i2c_backup.clk_div1  = i2c_CLKDIV1_REG;
                i2c_backup.clk_div2  = i2c_CLKDIV2_REG;
                
            #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
            
        #endif  /* End ((i2c_ENABLE_WAKEUP) && (i2c_I2C_PAIR_SELECTED)) */
        
    #else
        #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
        
            i2c_backup.int_mask = i2c_INT_MASK_REG;
            
            #if (i2c_MODE & i2c_MODE_SLAVE)
                i2c_backup.addr = i2c_ADDR_REG;
            
            #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
            
        #else
        
            /* Saves: 
                - Status Int mask: int_mask;
                - D0 register: addr;
                - Auxiliary Control: aux_ctl;
                - Period Register;
                - D0, D1 register: mclk_gen;  
            */
        #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */
    
    /* Disable interrupts */
    i2c_DisableInt();
}


/*******************************************************************************
* Function Name: i2c_Sleep
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: All incoming transaction will be NACKed till 
*  power down will be asserted. The address match event wakes up the chip.
*  Wakeup on address match disabled: Disables active mode power template bits or 
*  clock gating as appropriate. Saves I2C configuration and non-retention 
*  register values. 
*  Disables I2C interrupt.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
void i2c_Sleep(void)
{
    #if (i2c_ENABLE_WAKEUP)
        /* The I2C block should be always enabled if used as 
           wakeup source */
        
    #else
        /* Store PRS enable state */
        if(i2c_IS_I2C_ENABLE(i2c_I2C_ENABLE_REG))
        {
            i2c_backup.enableState = 1u;
            i2c_Stop();
        }
        else
        {
            i2c_backup.enableState = 0u;
        }
    #endif  /* End (i2c_ENABLE_WAKEUP) */
    
    i2c_SaveConfig();
}


/*******************************************************************************
* Function Name: i2c_RestoreConfig
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: enables I2C Master (if was enabled before go
*  to sleep), disables I2C backup regulator.
*  Wakeup on address match disabled: Restores I2C configuration and 
*  non-retention register values.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  i2c_backup - used to save component configuration and 
*  none-retention registers before enter sleep mode.
*
*******************************************************************************/
void i2c_RestoreConfig(void) 
{
    #if (i2c_IMPLEMENTATION == i2c_FF)
    
        #if (i2c_ENABLE_WAKEUP)
            uint8 enableInterrupts;
            
            /* Disable the I2C regulator backup */
            enableInterrupts = CyEnterCriticalSection();
            i2c_PWRSYS_CR1_REG &= ~i2c_PWRSYS_CR1_I2C_REG_BACKUP;
            CyExitCriticalSection(enableInterrupts);
            
            /* Need to re-enable Master */
            #if (i2c_MODE & i2c_MODE_MASTER)
                i2c_CFG_REG |= i2c_CFG_EN_MSTR;
            #endif  /* End (i2c_MODE & i2c_MODE_MASTER)*/
            
        #else
            i2c_XCFG_REG = i2c_backup.xcfg;
            i2c_CFG_REG = i2c_backup.cfg;
            
            #if (i2c_MODE & i2c_MODE_SLAVE)
                i2c_ADDR_REG = i2c_backup.addr;
            #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
                
            #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
                i2c_CLKDIV_REG =i2c_backup.clk_div;
            
            #else   
                i2c_CLKDIV1_REG = i2c_backup.clk_div1;
                i2c_CLKDIV2_REG = i2c_backup.clk_div2;
                 
            #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
        
        #endif  /* End (i2c_ENABLE_WAKEUP == 0u) */
        
    #else
        
        #if (i2c_PSOC3_ES2 || i2c_PSOC5_ES1)
            i2c_INT_MASK_REG = i2c_backup.int_mask;
            
            #if (i2c_MODE & i2c_MODE_MASTER)
            
                /* Restore Master Clock generator */
                i2c_MCLK_PRD_REG = i2c_MCLK_PERIOD_VALUE;
                i2c_MCLK_CMP_REG = i2c_MCLK_COMPARE_VALUE;
            
            #endif /* End (i2c_MODE & i2c_MODE_MASTER) */
            
            #if (i2c_MODE & i2c_MODE_SLAVE)
                i2c_ADDR_REG = i2c_backup.addr;
                
                /* Restore Slave bit counter */
                i2c_PERIOD_REG = i2c_PERIOD_VALUE;
                
            #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
            
        #else
            /* Saves: 
                - Status Int mask: int_mask;
                - D0 register: addr;
                - Auxiliary Control: aux_ctl;
                - Period Register: always 7;
            */
        #endif  /* End (i2c_PSOC3_ES2 || i2c_PSOC5_ES1) */
        
        /* Restore Control register */
        #if (i2c_MODE & i2c_MODE_SLAVE)
            #if (i2c_ADDR_DECODE == i2c_HDWR_DECODE)
                /* Turn off any address match */
                i2c_CFG_REG &= ~(i2c_CTRL_ANY_ADDRESS_MASK);
                
            #else
                /* Turn on any address match */
                i2c_CFG_REG |= i2c_CTRL_ANY_ADDRESS_MASK;
                
            #endif  /* End (i2c_ADDR_DECODE == i2c_HDWR_DECODE) */
            
        #endif  /* End (i2c_MODE & i2c_MODE_SLAVE) */
        
    #endif  /* End (i2c_IMPLEMENTATION == i2c_FF) */
    
    /* Enable interrupts */
    i2c_EnableInt();
}


/*******************************************************************************
* Function Name: i2c_Wakeup
********************************************************************************
*
* Summary:
*  Wakeup on address match enabled: enables I2C Master (if was enabled before go
*  to sleep) and disables I2C backup regulator.
*  Wakeup on address match disabled: Restores I2C configuration and non-retention 
*  register values. Restores Active mode power template bits or clock gating as 
*  appropriate.
*  The I2C interrupt remains disabled after function call.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Reentrant:
*  No
*
*******************************************************************************/
void i2c_Wakeup(void) 
{
    i2c_RestoreConfig();
    
    /* Restore I2C Enable state */
    if (i2c_backup.enableState != 0u)
    {
        i2c_Enable();
    }
}


/* [] END OF FILE */
