/*******************************************************************************
* File Name: i2s.c  
* Version 2.10
*
* Description:
*  This file contains the setup, control and status commands for the I2S
*  component.  
*
* Note: 
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "i2s.h"  

uint8 i2s_initVar = 0u;


/*******************************************************************************
* Function Name: i2s_Enable
********************************************************************************
*
* Summary:
*  Enables I2S interface.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void i2s_Enable(void) 
{  
    uint8 enableInterrupts;
    
    i2s_CONTROL_REG |= i2s_EN;    
    /* Enter critical section */
    enableInterrupts = CyEnterCriticalSection();
    i2s_AUX_CONTROL_REG |= i2s_CNTR7_EN;
    /* Exit critical section */
    CyExitCriticalSection(enableInterrupts);    
}


/*******************************************************************************
* Function Name: i2s_Init
********************************************************************************
*
* Summary:
*  Initializes Tx and/or Rx interrupt sources with initial values.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void i2s_Init(void) 
{
    /* Writes Tx and/or Rx interrupt source values provided for customizer */
    #if ((i2s_DIRECTION == i2s__TX) || \
        (i2s_DIRECTION == i2s__RX_AND_TX))
        i2s_SetTxInterruptMode(i2s_DEFAULT_INT_SOURCE);
    #endif /* i2s_DIRECTION == i2s__TX */

    #if ((i2s_DIRECTION == i2s__RX) || \
        (i2s_DIRECTION == i2s__RX_AND_TX))
        i2s_SetRxInterruptMode(i2s_DEFAULT_INT_SOURCE >> 3);
    #endif /* i2s_DIRECTION == i2s__RX */
}


/*******************************************************************************
* Function Name: i2s_Start
********************************************************************************
*
* Summary:
*  Starts the I2S interface.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  i2s_initVar - used to check initial configuration, modified on 
*  first function call.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void i2s_Start(void)
{
    /* Writes Tx and/or Rx interrupt source values provided for customizer */
    if (i2s_initVar == 0u)
    {
        i2s_Init();
        i2s_initVar = 1u;
    }
      
    i2s_Enable();
}


/*******************************************************************************
* Function Name: i2s_Stop
********************************************************************************
*
* Summary:
*  Disables the I2S interface.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void i2s_Stop(void) 
{
    uint8 enableInterrupts;
    
    i2s_CONTROL_REG &= ~i2s_EN;    
    /* Enter critical section */
    enableInterrupts = CyEnterCriticalSection();
    i2s_AUX_CONTROL_REG &= ~i2s_CNTR7_EN;
    /* Exit critical section */
    CyExitCriticalSection(enableInterrupts); 
    
    #if ((i2s_DIRECTION == i2s__TX) || \
        (i2s_DIRECTION == i2s__RX_AND_TX))
        i2s_DisableTx();
        i2s_ClearTxFIFO();
    #endif /* i2s_DIRECTION == i2s__TX */

    #if ((i2s_DIRECTION == i2s__RX) || \
        (i2s_DIRECTION == i2s__RX_AND_TX))
        i2s_DisableRx();
        i2s_ClearRxFIFO();
    #endif /* i2s_DIRECTION == i2s__RX */
}

/* Implementation of TX Direction */ 
#if ((i2s_DIRECTION == i2s__TX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX))

    /*******************************************************************************
    * Function Name: i2s_EnableTx
    ********************************************************************************
    *
    * Summary:
    *  Enables the Tx direction of the I2S interface.  At the next word
    *  select falling edge transmission will begin.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void i2s_EnableTx(void) 
    {
        uint8 enableInterrupts;
        
        i2s_CONTROL_REG |= i2s_TX_EN;     
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();    
        i2s_TX_STATUS_AUX_CONTROL_REG |= i2s_TX_INT_EN;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_DisableTx
    ********************************************************************************
    *
    * Summary:
    *  Disables the Tx direction of the I2S interface.  At the next word 
    *  select falling edge transmission of data will stop and a constant 0 value 
    *  will be transmitted.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void i2s_DisableTx(void) 
    {
        uint8 enableInterrupts;
        
        i2s_CONTROL_REG &= ~i2s_TX_EN;        
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        i2s_TX_STATUS_AUX_CONTROL_REG &= ~i2s_TX_INT_EN;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Sets the interrupt source for the I2S Tx direction interrupt.
    *
    * Parameters:
    *  Byte containing the constant for the selected interrupt sources.
    *   i2s_TX_FIFO_UNDEFLOW         
    *   i2s_TX_FIFO_0_NOT_FULL        
    *   i2s_TX_FIFO_1_NOT_FULL       
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void i2s_SetTxInterruptMode(uint8 interruptSource) \
                                                                
    {
        i2s_TX_STATUS_MASK_REG = (interruptSource & i2s_TX_ST_MASK); 
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Returns state in the I2S Tx status register.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  State of the I2S Tx status register
    *   i2s_RET_TX_FIFO_UNDEFLOW (Clear on Read)    
    *   i2s_RET_TX_FIFO_0_NOT_FULL    
    *   i2s_RET_TX_FIFO_1_NOT_FULL   
    *
    * Side Effects: 
    *  Clears the bits of I2S Tx status register that are Clear on Read.
    *
    *******************************************************************************/
    uint8 i2s_ReadTxStatus(void) 
    {
        return (i2s_TX_STATUS_REG & i2s_TX_ST_MASK);
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_WriteByte
    ********************************************************************************
    *
    * Summary:
    *  Writes a single byte into the Tx FIFO.
    *
    * Parameters:
    *  wrData: Byte containing the data to transmit.
    *  wordSelect: Indicates to write to the Left (0) or Right (1) channel.  In the 
    *              interleaved mode this parameter is ignored
    *
    * Return:
    *  None.
    * 
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void i2s_WriteByte(uint8 wrData, uint8 wordSelect)
    {        
        #if(i2s_TX_DATA_INTERLEAVING == i2s__INTERLEAVED)
            
            wordSelect = wordSelect; /* Supress compiler warning */
            
            i2s_TX_FIFO_0_REG = wrData;
            
        #else
            
            wordSelect &= i2s_WORD_SEL_MASK;
            
            if(wordSelect == i2s_TX_LEFT_CHANNEL)
            {
                i2s_TX_FIFO_0_REG = wrData; 
            }
            else
            {
                i2s_TX_FIFO_1_REG = wrData;    
            }
            
        #endif /* i2s_RX_DATA_INTERLEAVING == i2s__INTERLEAVED */
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_ClearTxFIFO
    ********************************************************************************
    *
    * Summary:
    *  Clears out the Tx FIFO.  Any data present in the FIFO will not be
    *  sent. This call should be made only when the Tx direction is disabled.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void i2s_ClearTxFIFO(void) 
    {
        uint8 enableInterrupts;
        
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();        
        i2s_TX_AUX_CONTROL_REG |=  i2s_TX_FIFO_0_CLR;
        i2s_TX_AUX_CONTROL_REG &= ~i2s_TX_FIFO_0_CLR;
        
        #if(i2s_TX_DATA_INTERLEAVING == i2s__SEPARATE)
            i2s_TX_AUX_CONTROL_REG |=  i2s_TX_FIFO_1_CLR;
            i2s_TX_AUX_CONTROL_REG &= ~i2s_TX_FIFO_1_CLR;    
        #endif /* i2s_RX_DATA_INTERLEAVING == i2s__SEPARATE */ 
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);
    }

#endif /* i2s_DIRECTION == i2s__TX */ 

/* Implementation of RX Direction */
#if ((i2s_DIRECTION == i2s__RX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX)) 

    /*******************************************************************************
    * Function Name: i2s_EnableRx
    ********************************************************************************
    *
    * Summary:
    *  Enables the Rx direction of the I2S interface.  At the next word
    *  select falling edge reception of data will begin.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void i2s_EnableRx(void) 
    {
        uint8 enableInterrupts;
        
        i2s_CONTROL_REG |= i2s_RX_EN;
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection(); 
        i2s_RX_STATUS_AUX_CONTROL_REG |= i2s_RX_INT_EN;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_DisableRx
    ********************************************************************************
    *
    * Summary:
    *  Disables the Rx direction of the I2S interface.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void i2s_DisableRx(void) 
    {
        uint8 enableInterrupts;
        
        i2s_CONTROL_REG &= ~i2s_RX_EN;
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();         
        i2s_RX_STATUS_AUX_CONTROL_REG &= ~i2s_RX_INT_EN;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Sets the interrupt source for the I2S Rx direction interrupt.
    *
    * Parameters:
    *  Byte containing the constant for the selected interrupt sources.
    *   i2s_RX_FIFO_OVERFLOW          
    *   i2s_RX_FIFO_0_NOT_EMPTY       
    *   i2s_RX_FIFO_1_NOT_EMPTY       
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void i2s_SetRxInterruptMode(uint8 interruptSource) \
                                                                
    {
        i2s_RX_STATUS_MASK_REG = (interruptSource & i2s_RX_ST_MASK); 
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Returns state in the I2S Rx status register.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  State of the I2S Rx status register
    *   i2s_RET_RX_FIFO_OVERFLOW (Clear on Read)     
    *   i2s_RET_RX_FIFO_0_NOT_EMPTY   
    *   i2s_RET_RX_FIFO_1_NOT_EMPTY   
    *
    * Side Effects: 
    *  Clears the bits of I2S Rx status register that are Clear on Read.
    *
    *******************************************************************************/
    uint8 i2s_ReadRxStatus(void) 
    {
        return (i2s_RX_STATUS_REG & i2s_RX_ST_MASK);
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_ReadByte
    ********************************************************************************
    *
    * Summary:
    *  Returns a single byte from the Rx FIFO.
    *
    * Parameters:
    *  wordSelect: Indicates to read from the Left (0) or Right (1) channel. In the 
    *              interleaved mode this parameter is ignored.
    *
    * Return:
    *  Byte containing the data received.
    * 
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 i2s_ReadByte(uint8 wordSelect)
    {               
        #if(i2s_RX_DATA_INTERLEAVING == i2s__INTERLEAVED)
            
            wordSelect = wordSelect; /* Supress compiler warning */
            
            return(i2s_RX_FIFO_0_REG);
            
        #else
           
            uint8 result;
            
            wordSelect &= i2s_WORD_SEL_MASK;
            
            if(wordSelect == i2s_RX_LEFT_CHANNEL)
            {
                result = i2s_RX_FIFO_0_REG; 
            }
            else
            {
                result = i2s_RX_FIFO_1_REG;    
            }
            
            return (result);
            
        #endif /* i2s_RX_DATA_INTERLEAVING == i2s__INTERLEAVED */
    }
        
        
    /*******************************************************************************
    * Function Name: i2s_ClearRxFIFO
    ********************************************************************************
    *
    * Summary:
    *  Clears out the Rx FIFO.  Any data present in the FIFO will be lost.
    *  This call should be made only when the Rx direction is disabled.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    * 
    *******************************************************************************/
    void i2s_ClearRxFIFO(void) 
    {
        uint8 enableInterrupts;
    
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        i2s_RX_AUX_CONTROL_REG |=  i2s_RX_FIFO_0_CLR;
        i2s_RX_AUX_CONTROL_REG &= ~i2s_RX_FIFO_0_CLR;
        
        #if(i2s_RX_DATA_INTERLEAVING == i2s__SEPARATE)
            i2s_RX_AUX_CONTROL_REG |=  i2s_RX_FIFO_1_CLR;
            i2s_RX_AUX_CONTROL_REG &= ~i2s_RX_FIFO_1_CLR;    
        #endif /* i2s_RX_DATA_INTERLEAVING == i2s__SEPARATE */
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);
    }

#endif /* i2s_DIRECTION == i2s__RX */


/* [] END OF FILE */
