/*******************************************************************************
* File Name: i2s.h
* Version 2.10
*
* Description:
*  This file provides constants and parameter values and API definition for the 
*  I2S Component
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#if !defined(CY_I2S_i2s_H)
#define CY_I2S_i2s_H

#include "cytypes.h"
#include "cyfitter.h"
#include "CyLib.h"


/*************************************** 
*   Conditional Compilation Parameters
***************************************/

#define i2s__RX 1
#define i2s__TX 2
#define i2s__RX_AND_TX 3

#define i2s_DIRECTION 3

#if ((i2s_DIRECTION == i2s__TX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX))
    #define i2s_TX_DATA_INTERLEAVING   1
#endif /* i2s_DIRECTION == i2s__TX */

#if ((i2s_DIRECTION == i2s__RX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX)) 
    #define i2s_RX_DATA_INTERLEAVING   1
#endif /* i2s_DIRECTION == i2s__RX */

/* PSoC3 ES2 or early */
#define i2s_PSOC3_ES2   ((CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_3A) && \
                                      (CYDEV_CHIP_REVISION_USED <= CYDEV_CHIP_REVISION_3A_ES2))
                     
/* PSoC5 ES1 */
#define i2s_PSOC5_ES1   ((CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_5A) && \
                                      (CYDEV_CHIP_REVISION_USED <= CYDEV_CHIP_REVISION_5A_ES1)) 


/***************************************
*     Data Struct Definitions
***************************************/

/* Sleep Mode API Support */
typedef struct _i2s_backupStruct
{
    uint8 enableState;
    
    uint8 CtrlReg;
    
    #if (i2s_PSOC3_ES2 || i2s_PSOC5_ES1)
        uint8 Cnt7Period;          
        #if ((i2s_DIRECTION == i2s__TX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            uint8 TxIntMask;
        #endif /* i2s_DIRECTION == i2s__TX */ 
        
        #if ((i2s_DIRECTION == i2s__RX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            uint8 RxIntMask;
        #endif /* i2s_DIRECTION == i2s__RX */   
    #endif /* (i2s_PSOC3_ES2 || i2s_PSOC5_ES1) */    
} i2s_BACKUP_STRUCT;


/***************************************
*        Function Prototypes            
***************************************/

void  i2s_Start(void);
void  i2s_Stop(void)                   ;
void  i2s_Sleep(void);                  
void  i2s_Wakeup(void)                 ;
void  i2s_Enable(void)                 ;
void  i2s_Init(void)                   ;
void  i2s_SaveConfig(void);
void  i2s_RestoreConfig(void)          ;

#if ((i2s_DIRECTION == i2s__TX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX)) 
    void   i2s_EnableTx(void)          ;
    void   i2s_DisableTx(void)         ;
    void   i2s_WriteByte(uint8 wrData, uint8 wordSelect);
    void   i2s_ClearTxFIFO(void)       ;
    void   i2s_SetTxInterruptMode(uint8 interruptSource) \
                                                    ;
    uint8  i2s_ReadTxStatus(void);     
#endif    /* i2s_DIRECTION == i2s__TX */ 

#if ((i2s_DIRECTION == i2s__RX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX)) 
    void   i2s_EnableRx(void)          ;
    void   i2s_DisableRx(void)         ;
    void   i2s_SetRxInterruptMode(uint8 interruptSource) \
                                                    ;
    uint8  i2s_ReadRxStatus(void)      ;
    uint8  i2s_ReadByte(uint8 wordSelect);
    void   i2s_ClearRxFIFO(void)       ;
#endif    /* i2s_DIRECTION == i2s__RX */


/***************************************
*           API Constants               
***************************************/

#if ((i2s_DIRECTION == i2s__TX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX))
    #define i2s_TX_LEFT_CHANNEL           (0x00u)
    #define i2s_TX_RIGHT_CHANNEL          (0x01u)
    
    #define i2s_TX_FIFO_UNDEFLOW          (0x01u)
    #define i2s_TX_FIFO_0_NOT_FULL        (0x02u)
    #define i2s_TX_FIFO_1_NOT_FULL        (0x04u)
    
    #define i2s_RET_TX_FIFO_UNDEFLOW      (0x01u)
    #define i2s_RET_TX_FIFO_0_NOT_FULL    (0x02u)
    #define i2s_RET_TX_FIFO_1_NOT_FULL    (0x04u)
#endif /* i2s_DIRECTION == i2s__TX */

#if ((i2s_DIRECTION == i2s__RX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX)) 
    #define i2s_RX_FIFO_OVERFLOW          (0x01u)
    #define i2s_RX_FIFO_0_NOT_EMPTY       (0x02u)
    #define i2s_RX_FIFO_1_NOT_EMPTY       (0x04u)

    #define i2s_RET_RX_FIFO_OVERFLOW      (0x01u)
    #define i2s_RET_RX_FIFO_0_NOT_EMPTY   (0x02u)
    #define i2s_RET_RX_FIFO_1_NOT_EMPTY   (0x04u)
    #define i2s_RX_LEFT_CHANNEL           (0x00u) 
#endif /* i2s_DIRECTION == i2s__RX */

#define i2s_WORD_SEL_MASK                 (0x01u)
#define i2s_DISABLED                      (0x00u)


/***************************************
*    Enumerated Types and Parameters    
***************************************/

#define i2s__SEPARATE 0
#define i2s__INTERLEAVED 1
 


/***************************************
*    Initial Parameter Constants        
***************************************/

/* Default interrupt source */
#define i2s_DEFAULT_INT_SOURCE (0u)

/* Word Select period. Set as (period * 2) - 1 because of 2X input clock */
#define i2s_DEFAULT_WS_PERIOD  ((32u << 1u) - 1u) 


/***************************************
*             Registers                 
***************************************/

#define i2s_CONTROL_REG   (* (reg8 *) i2s_AsyncCtl_ControlReg__CONTROL_REG)
#define i2s_CONTROL_PTR   (  (reg8 *) i2s_AsyncCtl_ControlReg__CONTROL_REG)

#define i2s_AUX_CONTROL_REG                (* (reg8 *) i2s_BitCounter__CONTROL_AUX_CTL_REG)
#define i2s_AUX_CONTROL_PTR                (  (reg8 *) i2s_BitCounter__CONTROL_AUX_CTL_REG)
#define i2s_CNT7_PERIOD_REG                (* (reg8 *) i2s_BitCounter__PERIOD_REG)
#define i2s_CNT7_PERIOD_PTR                (  (reg8 *) i2s_BitCounter__PERIOD_REG)

#if ((i2s_DIRECTION == i2s__TX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX))
    #define i2s_TX_REG                     (* (reg8 *) i2s_Tx_dpTx_u0__A0_REG)
    #define i2s_TX_PTR                     (  (reg8 *) i2s_Tx_dpTx_u0__A0_REG)

    #define i2s_TX_FIFO_0_REG              (* (reg8 *) i2s_Tx_dpTx_u0__F0_REG)
    #define i2s_TX_FIFO_0_PTR              (  (reg8 *) i2s_Tx_dpTx_u0__F0_REG)
    #define i2s_TX_FIFO_1_REG              (* (reg8 *) i2s_Tx_dpTx_u0__F1_REG)
    #define i2s_TX_FIFO_1_PTR              (  (reg8 *) i2s_Tx_dpTx_u0__F1_REG)

    #define i2s_TX_AUX_CONTROL_REG         (* (reg8 *) i2s_Tx_dpTx_u0__DP_AUX_CTL_REG)
    #define i2s_TX_AUX_CONTROL_PTR         (  (reg8 *) i2s_Tx_dpTx_u0__DP_AUX_CTL_REG)
    
    #define i2s_TX_STATUS_REG              (* (reg8 *) i2s_Tx_TxStsReg__STATUS_REG)
    #define i2s_TX_STATUS_PTR              (  (reg8 *) i2s_Tx_TxStsReg__STATUS_REG)
    #define i2s_TX_STATUS_MASK_REG         (* (reg8 *) i2s_Tx_TxStsReg__MASK_REG)
    #define i2s_TX_STATUS_MASK_PTR         (  (reg8 *) i2s_Tx_TxStsReg__MASK_REG)
    #define i2s_TX_STATUS_AUX_CONTROL_REG  (* (reg8 *) i2s_Tx_TxStsReg__STATUS_AUX_CTL_REG)
    #define i2s_TX_STATUS_AUX_CONTROL_PTR  (  (reg8 *) i2s_Tx_TxStsReg__STATUS_AUX_CTL_REG)

#endif /* i2s_DIRECTION == i2s__TX */

#if ((i2s_DIRECTION == i2s__RX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX)) 
    #define i2s_RX_REG                     (* (reg8 *) i2s_Rx_dpRx_u0__A0_REG)
    #define i2s_RX_PTR                     (  (reg8 *) i2s_Rx_dpRx_u0__A0_REG)
    #define i2s_RX_FIFO_0_REG              (* (reg8 *) i2s_Rx_dpRx_u0__F0_REG)
    #define i2s_RX_FIFO_0_PTR              (  (reg8 *) i2s_Rx_dpRx_u0__F0_REG)
    #define i2s_RX_FIFO_1_REG              (* (reg8 *) i2s_Rx_dpRx_u0__F1_REG)
    #define i2s_RX_FIFO_1_PTR              (  (reg8 *) i2s_Rx_dpRx_u0__F1_REG)
    
    #define i2s_RX_AUX_CONTROL_REG         (* (reg8 *) i2s_Rx_dpRx_u0__DP_AUX_CTL_REG)
    #define i2s_RX_AUX_CONTROL_PTR         (  (reg8 *) i2s_Rx_dpRx_u0__DP_AUX_CTL_REG)

    #define i2s_RX_STATUS_REG              (* (reg8 *) i2s_Rx_RxStsReg__STATUS_REG)
    #define i2s_RX_STATUS_PTR              (  (reg8 *) i2s_Rx_RxStsReg__STATUS_REG)
    #define i2s_RX_STATUS_MASK_REG         (* (reg8 *) i2s_Rx_RxStsReg__MASK_REG)
    #define i2s_RX_STATUS_MASK_PTR         (  (reg8 *) i2s_Rx_RxStsReg__MASK_REG)
    
    #define i2s_RX_STATUS_AUX_CONTROL_REG  (* (reg8 *) i2s_Rx_RxStsReg__STATUS_AUX_CTL_REG)
    #define i2s_RX_STATUS_AUX_CONTROL_PTR  (  (reg8 *) i2s_Rx_RxStsReg__STATUS_AUX_CTL_REG)
    
#endif /* i2s_DIRECTION == i2s__RX */


/***************************************
*       Register Constants              
***************************************/

#define i2s_TX_EN                      (0x01u)
#define i2s_RX_EN                      (0x02u)
#define i2s_EN                         (0x04u)

#if ((i2s_DIRECTION == i2s__TX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX))
    #define i2s_TX_ST_MASK             (0x07u)
    
    #define i2s_TX_FIFO_0_CLR          (0x01u)
    #define i2s_TX_FIFO_1_CLR          (0x02u) 
    
    #define i2s_TX_INT_EN              (0x10u)
#endif /* i2s_DIRECTION == i2s__TX */

#if ((i2s_DIRECTION == i2s__RX) || \
    (i2s_DIRECTION == i2s__RX_AND_TX))
    #define i2s_RX_ST_MASK             (0x07u)
    
    #define i2s_RX_FIFO_0_CLR          (0x01u)
    #define i2s_RX_FIFO_1_CLR          (0x02u)
    
    #define i2s_RX_INT_EN              (0x10u)
#endif /* i2s_DIRECTION == i2s__RX */

#define i2s_CNTR7_EN                   (0x20u)

#endif /* CY_I2S_i2s_H */


/* [] END OF FILE */
