/*******************************************************************************
* File Name: i2s_PM.c
* Version 2.10
*
* Description:
*  This file contains the setup, control and status commands to support 
*  component operations in low power mode.  
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "i2s.h"

static i2s_BACKUP_STRUCT  i2s_backup = 
{
    /* Generation of WS and SCK is disabled */
    i2s_DISABLED,
    
    /* RX and/or TX directions are disabled */
    i2s_DISABLED,
    
    #if (i2s_PSOC3_ES2 || i2s_PSOC5_ES1)
        /* WS Period */
        i2s_DEFAULT_WS_PERIOD,
        /* TX Interrupt Source */
        #if ((i2s_DIRECTION == i2s__TX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            i2s_DEFAULT_INT_SOURCE & i2s_TX_ST_MASK,
        #endif /* i2s_DIRECTION == i2s__TX */ 
        /* RX Interrupt Source */
        #if ((i2s_DIRECTION == i2s__RX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            (i2s_DEFAULT_INT_SOURCE >> 3) & i2s_RX_ST_MASK,
        #endif /* i2s_DIRECTION == i2s__RX */   
    #endif /* (i2s_PSOC3_ES2 || i2s_PSOC5_ES1) */ 
};


/*******************************************************************************
* Function Name: i2s_SaveConfig
********************************************************************************
*
* Summary:
*  Saves I2S configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  i2s_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void i2s_SaveConfig(void)
{   
    /* Saves CTL_REG bits to define what direction was enabled */    
    i2s_backup.CtrlReg = i2s_CONTROL_REG;
    
    /* Saves CNT7 and STATUSI MSK/PER for PSoC3 ES2 and PSoC5 ES1 */
    #if (i2s_PSOC3_ES2 || i2s_PSOC5_ES1)                       
        i2s_backup.Cnt7Period = i2s_CNT7_PERIOD_REG;
        
        #if ((i2s_DIRECTION == i2s__TX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            i2s_backup.TxIntMask = i2s_TX_STATUS_MASK_REG;
        #endif /* i2s_DIRECTION == i2s__TX */
        
        #if ((i2s_DIRECTION == i2s__RX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            i2s_backup.RxIntMask = i2s_RX_STATUS_MASK_REG;
        #endif /* i2s_DIRECTION == i2s__RX */       
    #endif /* i2s_PSOC3_ES2 || i2s_PSOC5_ES1 */
}


/*******************************************************************************
* Function Name: i2s_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores I2S configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  i2s_backup - used when non-retention registers are restored.
*
*******************************************************************************/
void i2s_RestoreConfig(void) 
{    
    /* Restores CNT7 and STATUSI MSK/PER registers for PSoC3 ES2 and PSoC5 ES1 */
    #if (i2s_PSOC3_ES2 || i2s_PSOC5_ES1)                         
        i2s_CNT7_PERIOD_REG = i2s_backup.Cnt7Period;
        
        #if ((i2s_DIRECTION == i2s__TX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            i2s_TX_STATUS_MASK_REG = i2s_backup.TxIntMask;
        #endif /* i2s_DIRECTION == i2s__TX */
        
        #if ((i2s_DIRECTION == i2s__RX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            i2s_RX_STATUS_MASK_REG = i2s_backup.RxIntMask;
        #endif /* i2s_DIRECTION == i2s__RX */
        
    #endif /* i2s_PSOC3_ES2 || i2s_PSOC5_ES1 */
}


/*******************************************************************************
* Function Name: i2s_Sleep
********************************************************************************
*
* Summary:
*  Prepares I2S goes to sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  i2s_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
* 
*******************************************************************************/
void i2s_Sleep(void)
{
    if((i2s_CONTROL_REG & i2s_EN) == i2s_EN)
    {
        i2s_backup.enableState = 1u;
    }
    else /* The component is disabled */
    {
        i2s_backup.enableState = 0u;
    }
    
    /* Save registers configuration */
    i2s_SaveConfig();
    
    /* Stop component */
    i2s_Stop();
}


/*******************************************************************************
* Function Name: i2s_Wakeup
********************************************************************************
*
* Summary:
*  Prepares I2S to wake up.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  i2s_backup - used when non-retention registers are restored.
*
* Reentrant:
*  Yes.
* 
*******************************************************************************/
void i2s_Wakeup(void) 
{
    /* Restore registers values */
    i2s_RestoreConfig();
    
    if(0u != i2s_backup.enableState)
    {
        /* Enable component's operation */
        i2s_Enable();
        
        /* Enable Tx/Rx direction if they were enabled before sleep */
        #if ((i2s_DIRECTION == i2s__RX) || \
        (i2s_DIRECTION == i2s__RX_AND_TX))
            if((i2s_backup.CtrlReg & i2s_RX_EN) == i2s_RX_EN)
            {
                i2s_EnableRx();
            }
        #endif /* i2s_DIRECTION == i2s__RX */
    
        #if ((i2s_DIRECTION == i2s__TX) || \
            (i2s_DIRECTION == i2s__RX_AND_TX))
            if((i2s_backup.CtrlReg & i2s_TX_EN) == i2s_TX_EN)
            {
                i2s_EnableTx();
            }
        #endif /* i2s_DIRECTION == i2s__TX */

    } /* Do nothing if component's block was disabled before */
}


/* [] END OF FILE */

