/*******************************************************************************
* File Name: millisecondTimer.c  
* Version 2.0
*
* Description:
*  The Timer User Module consists of a 8, 16, 24 or 32-bit timer with
*  a selectable period between 2 and 2^Width - 1.  The timer may free run
*  or be used as a capture timer as well.  The capture can be initiated
*  by a positive or negative edge signal as well as via software.
*  A trigger input can be programmed to enable the timer on rising edge
*  falling edge, either edge or continous run.
*  Interrupts may be generated due to a terminal count condition
*  or a capture event.
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "millisecondTimer.h"

uint8 millisecondTimer_initVar = 0u;


/*******************************************************************************
* Function Name: millisecondTimer_Init
********************************************************************************
* 
* Summary:
*  Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void millisecondTimer_Init(void) 
{
    #if(!millisecondTimer_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 millisecondTimer_interruptState;
    #endif

    #if (millisecondTimer_UsingFixedFunction)
        /* Clear all bits but the enable bit (if it's already set) for Timer operation */
        millisecondTimer_CONTROL &= millisecondTimer_CTRL_ENABLE;
        
        /* Clear the mode bits for continuous run mode */
        #if (millisecondTimer_PSOC3_ES2 || millisecondTimer_PSOC5_ES1)
            millisecondTimer_CONTROL2 &= ~millisecondTimer_CTRL_MODE_MASK;
        #endif
        #if (millisecondTimer_PSOC3_ES3 || millisecondTimer_PSOC5_ES2)
            millisecondTimer_CONTROL3 &= ~millisecondTimer_CTRL_MODE_MASK;                
        #endif

        /* Check if One Shot mode is enabled i.e. RunMode !=0*/
        #if (millisecondTimer_RunModeUsed != 0x0u)
            /* Set 3rd bit of Control register to enable one shot mode */
            millisecondTimer_CONTROL |= 0x04u;
        #endif
        
        #if (millisecondTimer_RunModeUsed == 2)
            #if (millisecondTimer_PSOC3_ES2 || millisecondTimer_PSOC5_ES1)
                /* Set last 2 bits of control2 register if one shot(halt on 
                interrupt) is enabled*/
                millisecondTimer_CONTROL2 |= 0x03u;
            #endif
            #if (millisecondTimer_PSOC3_ES3 || millisecondTimer_PSOC5_ES2)
                /* Set last 2 bits of control3 register if one shot(halt on 
                interrupt) is enabled*/
                millisecondTimer_CONTROL3 |= 0x03u;                
            #endif
        #endif
        
        #if (millisecondTimer_UsingHWEnable != 0)
            #if (millisecondTimer_PSOC3_ES2 || millisecondTimer_PSOC5_ES1)
                /* Set the default Run Mode of the Timer to Continuous */
                millisecondTimer_CONTROL2 |= millisecondTimer_CTRL_MODE_PULSEWIDTH;
            #endif
            #if (millisecondTimer_PSOC3_ES3 || millisecondTimer_PSOC5_ES2)
                /* Clear and Set ROD and COD bits of CFG2 register */
                millisecondTimer_CONTROL3 &= ~millisecondTimer_CTRL_RCOD_MASK;
                millisecondTimer_CONTROL3 |= millisecondTimer_CTRL_RCOD;  
                
                /* Clear and Enable the HW enable bit in CFG2 register */
                millisecondTimer_CONTROL3 &= ~millisecondTimer_CTRL_ENBL_MASK;
                millisecondTimer_CONTROL3 |= millisecondTimer_CTRL_ENBL;     
                
                /* Set the default Run Mode of the Timer to Continuous */
                millisecondTimer_CONTROL3 |= millisecondTimer_CTRL_MODE_CONTINUOUS;
            #endif
        #endif
        
        /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
        millisecondTimer_RT1 &= ~millisecondTimer_RT1_MASK;
        millisecondTimer_RT1 |= millisecondTimer_SYNC;     
                
        /*Enable DSI Sync all all inputs of the Timer*/
        millisecondTimer_RT1 &= ~(millisecondTimer_SYNCDSI_MASK);
        millisecondTimer_RT1 |= millisecondTimer_SYNCDSI_EN;
                
        /* Set the IRQ to use the status register interrupts */
        millisecondTimer_CONTROL2 |= millisecondTimer_CTRL2_IRQ_SEL;
    #endif 
        
    /* Set Initial values from Configuration */
    millisecondTimer_WritePeriod(millisecondTimer_INIT_PERIOD);
    millisecondTimer_WriteCounter(millisecondTimer_INIT_PERIOD);
    
    #if (millisecondTimer_UsingHWCaptureCounter)/* Capture counter is enabled */
        millisecondTimer_CAPTURE_COUNT_CTRL |= millisecondTimer_CNTR_ENABLE;
        millisecondTimer_SetCaptureCount(millisecondTimer_INIT_CAPTURE_COUNT);
    #endif
        
    #if (!millisecondTimer_UsingFixedFunction)
        #if (millisecondTimer_SoftwareCaptureMode)
            millisecondTimer_SetCaptureMode(millisecondTimer_INIT_CAPTURE_MODE);
        #endif
        
        #if (millisecondTimer_SoftwareTriggerMode)
            if (!(millisecondTimer_CONTROL & millisecondTimer__B_TIMER__TM_SOFTWARE))
            {
                millisecondTimer_SetTriggerMode(millisecondTimer_INIT_TRIGGER_MODE);
            }
        #endif
        
        /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
        /* Enter Critical Region*/
        millisecondTimer_interruptState = CyEnterCriticalSection();
        
        /* Use the interrupt output of the status register for IRQ output */
        millisecondTimer_STATUS_AUX_CTRL |= millisecondTimer_STATUS_ACTL_INT_EN_MASK;
        
        /* Exit Critical Region*/
        CyExitCriticalSection(millisecondTimer_interruptState);

        #if (millisecondTimer_EnableTriggerMode)
            millisecondTimer_EnableTrigger();
        #endif

        #if (millisecondTimer_InterruptOnCaptureCount)
             #if (!millisecondTimer_ControlRegRemoved)
                millisecondTimer_SetInterruptCount(millisecondTimer_INIT_INT_CAPTURE_COUNT);
            #endif
        #endif
        
        millisecondTimer_ClearFIFO();  
    #endif

    millisecondTimer_SetInterruptMode(millisecondTimer_INIT_INTERRUPT_MODE);
}


/*******************************************************************************
* Function Name: millisecondTimer_Enable
********************************************************************************
* 
* Summary:
*  Enable the Timer
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void millisecondTimer_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (millisecondTimer_UsingFixedFunction)
        millisecondTimer_GLOBAL_ENABLE |= millisecondTimer_BLOCK_EN_MASK;
        millisecondTimer_GLOBAL_STBY_ENABLE |= millisecondTimer_BLOCK_STBY_EN_MASK;
    #endif   
    
    /* Remove assignment if control register is removed */
    #if (!millisecondTimer_ControlRegRemoved || millisecondTimer_UsingFixedFunction)     
        millisecondTimer_CONTROL |= millisecondTimer_CTRL_ENABLE;
    #endif
}


/*******************************************************************************
* Function Name: millisecondTimer_Start
********************************************************************************
* 
* Summary:
*  The start function initializes the timer with the default values, the 
*  enables the timerto begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  millisecondTimer_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
* Reentrant
*  No
*
*******************************************************************************/
void millisecondTimer_Start(void) 
{
    if(millisecondTimer_initVar == 0u)
    {
        millisecondTimer_Init();
        
        millisecondTimer_initVar = 1u;   /* Clear this bit for Initialization */
    }
    
    /* Enable the Timer */		
    millisecondTimer_Enable();    
}


/*******************************************************************************
* Function Name: millisecondTimer_Stop
********************************************************************************
* 
* Summary:
*  The stop function halts the timer, but does not change any modes or disable
*  interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the timer.
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_Stop(void) 
{
    /* Disable Timer */
    #if(!millisecondTimer_ControlRegRemoved || millisecondTimer_UsingFixedFunction)    /* Remove assignment if control register is removed */  
        millisecondTimer_CONTROL &= ~millisecondTimer_CTRL_ENABLE;
    #endif  
    
    /* Globally disable the Fixed Function Block chosen */
    #if (millisecondTimer_UsingFixedFunction)
        millisecondTimer_GLOBAL_ENABLE &= ~millisecondTimer_BLOCK_EN_MASK;
        millisecondTimer_GLOBAL_STBY_ENABLE &= ~millisecondTimer_BLOCK_STBY_EN_MASK;
    #endif
}


/*******************************************************************************
* Function Name: millisecondTimer_SetInterruptMode
********************************************************************************
* 
* Summary:
*  This function selects which of the interrupt inputs may cause an interrupt.  
*  The twosources are caputure and terminal.  One, both or neither may 
*  be selected.
*
* Parameters:  
*  interruptMode:   This parameter is used to enable interrups on either/or 
*                   terminal count or capture.  
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_SetInterruptMode(uint8 interruptMode) 
{
    millisecondTimer_STATUS_MASK = interruptMode;
}

/*******************************************************************************
* Function Name: millisecondTimer_SoftwareCapture
********************************************************************************
* 
* Summary:
*  This function forces a capture independent of the capture signal.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Side Effects:
*  An existing hardware capture could be overwritten.
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_SoftwareCapture(void) 
{
    /* Generate a software capture by reading the counter register */
    CY_GET_REG8(millisecondTimer_COUNTER_LSB_PTR);
    /* Capture Data is now in the FIFO */
}


/*******************************************************************************
* Function Name: millisecondTimer_ReadStatusRegister
********************************************************************************
* 
* Summary:
*  Reads the status register and returns it's state. This function should use
*  defined types for the bit-field information as the bits in this register may
*  be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  The contents of the status register
*
* Side Effects:
*  Status register bits may be clear on read. 
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8   millisecondTimer_ReadStatusRegister(void) 
{
    return millisecondTimer_STATUS;
}


#if (!millisecondTimer_ControlRegRemoved)   /* Remove API if control register is removed */
/*******************************************************************************
* Function Name: millisecondTimer_ReadControlRegister
********************************************************************************
* 
* Summary:
*  Reads the control register and returns it's value. 
*
* Parameters:  
*  void
*
* Return: 
*  The contents of the control register
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint8 millisecondTimer_ReadControlRegister(void) 
{
    return millisecondTimer_CONTROL;
}


/*******************************************************************************
* Function Name: millisecondTimer_WriteControlRegister
********************************************************************************
* 
* Summary:
*  Sets the bit-field of the control register.  
*
* Parameters:  
*  control: The contents of the control register
*
* Return: 
*  
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_WriteControlRegister(uint8 control) 
{
    millisecondTimer_CONTROL = control;
}
#endif /* Remove API if control register is removed */


/*******************************************************************************
* Function Name: millisecondTimer_ReadPeriod
********************************************************************************
* 
* Summary:
*  This function returns the current value of the Period.
*
* Parameters:  
*  void
*
* Return: 
*  The present value of the counter.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 millisecondTimer_ReadPeriod(void) 
{
   return ( CY_GET_REG24(millisecondTimer_PERIOD_LSB_PTR) );  
}


/*******************************************************************************
* Function Name: millisecondTimer_WritePeriod
********************************************************************************
* 
* Summary:
*  This function is used to change the period of the counter.  The new period 
*  will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: This value may be between 1 and (2^Resolution)-1.  A value of 0 will result in
*          the counter remaining at zero.
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_WritePeriod(uint32 period) 
{
    #if(millisecondTimer_UsingFixedFunction)
        uint16 period_temp = (uint16)period;
        CY_SET_REG16(millisecondTimer_PERIOD_LSB_PTR, period_temp);
    #else
        CY_SET_REG24(millisecondTimer_PERIOD_LSB_PTR, period);
    #endif
}


/*******************************************************************************
* Function Name: millisecondTimer_ReadCapture
********************************************************************************
* 
* Summary:
*  This function returns the last value captured.
*
* Parameters:  
*  void 
*
* Return: 
*  Present Capture value.
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 millisecondTimer_ReadCapture(void) 
{
   return ( CY_GET_REG24(millisecondTimer_CAPTURE_LSB_PTR) );  
}


/*******************************************************************************
* Function Name: millisecondTimer_WriteCounter
********************************************************************************
* 
* Summary:
*  This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_WriteCounter(uint32 counter) 
{
    #if(millisecondTimer_UsingFixedFunction)
        counter = counter;
        /* This functionality is removed until a FixedFunction HW update to 
         * allow this register to be written 
         */
        /* uint16 counter_temp = (uint16)counter;
         * CY_SET_REG16(millisecondTimer_COUNTER_LSB_PTR, counter_temp);
         */
    #else
        CY_SET_REG24(millisecondTimer_COUNTER_LSB_PTR, counter);
    #endif
}


/*******************************************************************************
* Function Name: millisecondTimer_ReadCounter
********************************************************************************
* 
* Summary:
*  This function returns the current counter value.
*
* Parameters:  
*  void
*
* Return: 
*  Present compare value. 
*
* Reentrant
*  Yes
*
*******************************************************************************/
uint32 millisecondTimer_ReadCounter(void) 
{
    
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(millisecondTimer_COUNTER_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG24(millisecondTimer_CAPTURE_LSB_PTR));
}


#if(!millisecondTimer_UsingFixedFunction) /* UDB Specific Functions */

/*******************************************************************************
 * The functions below this point are only available using the UDB 
 * implementation.  If a feature is selected, then the API is enabled.
 ******************************************************************************/


#if (millisecondTimer_SoftwareCaptureMode)
/*******************************************************************************
* Function Name: millisecondTimer_SetCaptureMode
********************************************************************************
* 
* Summary:
*  This function sets the capture mode to either rising or falling edge.
*
* Parameters:  
*  captureMode:  This parameter sets the capture mode of the UDB capture feature.
*  The parameter values are defined using the #define millisecondTimer__B_TIMER__CM_NONE 0
#define millisecondTimer__B_TIMER__CM_RISINGEDGE 1
#define millisecondTimer__B_TIMER__CM_FALLINGEDGE 2
#define millisecondTimer__B_TIMER__CM_EITHEREDGE 3
#define millisecondTimer__B_TIMER__CM_SOFTWARE 4
. 
*  The following are the possible values of the parameter
*  millisecondTimer__B_TIMER__CM_NONE        - Set Capture mode to None
*  millisecondTimer__B_TIMER__CM_RISINGEDGE  - Capture on rising edge of Capture input
*  millisecondTimer__B_TIMER__CM_FALLINGEDGE - Capture on falling edge of Capture input
*  millisecondTimer__B_TIMER__CM_EITHEREDGE  - Capture on falling edge of Capture input
* 
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_SetCaptureMode(uint8 captureMode) 
{
    /* This must only set to two bits of the control register associated */
    captureMode = (captureMode << millisecondTimer_CTRL_CAP_MODE_SHIFT);
    captureMode &= (millisecondTimer_CTRL_CAP_MODE_MASK);
    
	/* Clear the Current Setting */
    millisecondTimer_CONTROL &= ~millisecondTimer_CTRL_CAP_MODE_MASK;
    
    /* Write The New Setting */   
    millisecondTimer_CONTROL |= captureMode;
}
#endif

#if (millisecondTimer_SoftwareTriggerMode)
/*******************************************************************************
* Function Name: millisecondTimer_SetTriggerMode
********************************************************************************
* 
* Summary:
*  This function sets the trigger input mode
*
* Parameters:  
*  triggerMode: Pass one of the pre-defined Trigger Modes (except Software)
    #define millisecondTimer__B_TIMER__TM_NONE 0x00
    #define millisecondTimer__B_TIMER__TM_RISINGEDGE 0x04
    #define millisecondTimer__B_TIMER__TM_FALLINGEDGE 0x08
    #define millisecondTimer__B_TIMER__TM_EITHEREDGE 0x0C
    #define millisecondTimer__B_TIMER__TM_SOFTWARE 0x10
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_SetTriggerMode(uint8 triggerMode) 
{
    /* This must only set to two bits of the control register associated */
    triggerMode &= millisecondTimer_CTRL_TRIG_MODE_MASK;
    
    /* Clear the Current Setting */
    millisecondTimer_CONTROL &= ~millisecondTimer_CTRL_TRIG_MODE_MASK;
    
    /* Write The New Setting */   
    millisecondTimer_CONTROL |= (triggerMode | millisecondTimer__B_TIMER__TM_SOFTWARE);     

}
#endif

#if (millisecondTimer_EnableTriggerMode)
/*******************************************************************************
* Function Name: millisecondTimer_EnableTrigger
********************************************************************************
* 
* Summary:
*  Sets the control bit enabling Hardware Trigger mode
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_EnableTrigger(void) 
{
    #if (!millisecondTimer_ControlRegRemoved)   /* Remove assignment if control register is removed */
        millisecondTimer_CONTROL |= millisecondTimer_CTRL_TRIG_EN;
    #endif
}

/*******************************************************************************
* Function Name: millisecondTimer_DisableTrigger
********************************************************************************
* 
* Summary:
*  Clears the control bit enabling Hardware Trigger mode
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_DisableTrigger(void) 
{
    #if (!millisecondTimer_ControlRegRemoved)   /* Remove assignment if control register is removed */
        millisecondTimer_CONTROL &= ~millisecondTimer_CTRL_TRIG_EN;
    #endif
}
#endif


#if(millisecondTimer_InterruptOnCaptureCount)
#if (!millisecondTimer_ControlRegRemoved)   /* Remove API if control register is removed */
/*******************************************************************************
* Function Name: millisecondTimer_SetInterruptCount
********************************************************************************
* 
* Summary:
*  This function sets the capture count before an interrupt is triggered.
*
* Parameters:  
*  interruptCount:  A value between 0 and 3 is valid.  If the value is 0, then 
*                   an interrupt will occur each time a capture occurs.  
*                   A value of 1 to 3 will cause the interrupt  
*                   to delay by the same number of captures.
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_SetInterruptCount(uint8 interruptCount) 
{
    /* This must only set to two bits of the control register associated */
    interruptCount &= millisecondTimer_CTRL_INTCNT_MASK;
    
    /* Clear the Current Setting */
    millisecondTimer_CONTROL &= ~millisecondTimer_CTRL_INTCNT_MASK;
    /* Write The New Setting */   
    millisecondTimer_CONTROL |= interruptCount;    
}
#endif /* Remove API if control register is removed */
#endif /* millisecondTimer_InterruptOnCaptureCount */


#if (millisecondTimer_UsingHWCaptureCounter)
/*******************************************************************************
* Function Name: millisecondTimer_SetCaptureCount
********************************************************************************
* 
* Summary:
*  This function sets the capture count
*
* Parameters:  
*  captureCount: A value between 2 and 127 inclusive is valid.  A value of 1 
*                to 127 will cause the interrupt to delay by the same number of 
*                captures.
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_SetCaptureCount(uint8 captureCount) 
{
    millisecondTimer_CAP_COUNT = captureCount;
}


/*******************************************************************************
* Function Name: millisecondTimer_ReadCaptureCount
********************************************************************************
* 
* Summary:
*  This function reads the capture count setting
*
* Parameters:  
*  void
*
* Return: 
*  Returns the Capture Count Setting
*
*******************************************************************************/
uint8 millisecondTimer_ReadCaptureCount(void) 
{
    return millisecondTimer_CAP_COUNT ;
}
#endif /* millisecondTimer_UsingHWCaptureCounter */


/*******************************************************************************
* Function Name: millisecondTimer_ClearFIFO
********************************************************************************
* 
* Summary:
*  This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant
*  Yes
*
*******************************************************************************/
void millisecondTimer_ClearFIFO(void) 
{
    while(millisecondTimer_ReadStatusRegister() & millisecondTimer_STATUS_FIFONEMP)
    {
        millisecondTimer_ReadCapture();
    }
}

#endif /* UDB Specific Functions */


/* [] END OF FILE */
