/*******************************************************************************
* File Name: millisecondTimer_PM.c  
* Version 2.0
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "millisecondTimer.h"
static millisecondTimer_backupStruct millisecondTimer_backup;

/*******************************************************************************
* Function Name: millisecondTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  millisecondTimer_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void millisecondTimer_SaveConfig(void)
{   
    #if (!millisecondTimer_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (millisecondTimer_PSOC3_ES2 || millisecondTimer_PSOC5_ES1)
            millisecondTimer_backup.TimerUdb = millisecondTimer_ReadCounter();
            millisecondTimer_backup.TimerPeriod = millisecondTimer_ReadPeriod();
            millisecondTimer_backup.InterruptMaskValue = millisecondTimer_STATUS_MASK;
            #if (millisecondTimer_UsingHWCaptureCounter)
                millisecondTimer_backup.TimerCaptureCounter = millisecondTimer_ReadCaptureCount();        
            #endif
        #endif
        
        #if (millisecondTimer_PSOC3_ES3 || millisecondTimer_PSOC5_ES2)
            millisecondTimer_backup.TimerUdb = millisecondTimer_ReadCounter();
            millisecondTimer_backup.InterruptMaskValue = millisecondTimer_STATUS_MASK;
            #if (millisecondTimer_UsingHWCaptureCounter)
                millisecondTimer_backup.TimerCaptureCounter = millisecondTimer_ReadCaptureCount();        
            #endif
        #endif
        
        #if(!millisecondTimer_ControlRegRemoved)
            millisecondTimer_backup.TimerControlRegister = millisecondTimer_ReadControlRegister();
        #endif
    #endif
}

/*******************************************************************************
* Function Name: millisecondTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  millisecondTimer_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void millisecondTimer_RestoreConfig(void) 
{
    #if (!millisecondTimer_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (millisecondTimer_PSOC3_ES2 || millisecondTimer_PSOC5_ES1)
            millisecondTimer_WriteCounter(millisecondTimer_backup.TimerUdb);
            millisecondTimer_WritePeriod(millisecondTimer_backup.TimerPeriod);
            millisecondTimer_STATUS_MASK =millisecondTimer_backup.InterruptMaskValue;
            #if (millisecondTimer_UsingHWCaptureCounter)
                millisecondTimer_SetCaptureCount(millisecondTimer_backup.TimerCaptureCounter);        
            #endif
        #endif
        
        #if (millisecondTimer_PSOC3_ES3 || millisecondTimer_PSOC5_ES2)
            millisecondTimer_WriteCounter(millisecondTimer_backup.TimerUdb);
            millisecondTimer_STATUS_MASK =millisecondTimer_backup.InterruptMaskValue;
            #if (millisecondTimer_UsingHWCaptureCounter)
                millisecondTimer_SetCaptureCount(millisecondTimer_backup.TimerCaptureCounter);           
            #endif
        #endif
        
        #if(!millisecondTimer_ControlRegRemoved)
            millisecondTimer_WriteControlRegister(millisecondTimer_backup.TimerControlRegister);
        #endif
    #endif
}


/*******************************************************************************
* Function Name: millisecondTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  millisecondTimer_backup.TimerEnableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*    No
*
*******************************************************************************/
void millisecondTimer_Sleep(void)
{
    #if(!millisecondTimer_ControlRegRemoved)
        /* Save Counter's enable state */
        if(millisecondTimer_CTRL_ENABLE == (millisecondTimer_CONTROL & millisecondTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            millisecondTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            millisecondTimer_backup.TimerEnableState = 0u;
        }
    #endif
    millisecondTimer_Stop();
    millisecondTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: millisecondTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  millisecondTimer_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void millisecondTimer_Wakeup(void) 
{
    millisecondTimer_RestoreConfig();
    #if(!millisecondTimer_ControlRegRemoved)
        if(millisecondTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                millisecondTimer_Enable();
        } /* Do nothing if Timer was disabled before */ 
    #endif
}


/* [] END OF FILE */
