/*******************************************************************************
* File Name: oled_uart_INT.c
* Version 2.0
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "oled_uart.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (oled_uart_RX_ENABLED || oled_uart_HD_ENABLED) && \
     (oled_uart_RXBUFFERSIZE > oled_uart_FIFO_LENGTH))

    extern volatile uint8 oled_uart_rxBuffer[];
    extern volatile uint8 oled_uart_rxBufferRead;
    extern volatile uint8 oled_uart_rxBufferWrite;
    extern volatile uint8 oled_uart_rxBufferLoopDetect;
    extern volatile uint8 oled_uart_rxBufferOverflow;


    /*******************************************************************************
    * Function Name: oled_uart_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  oled_uart_rxBuffer - RAM buffer pointer for save received data.
    *  oled_uart_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     increments after each byte saved to buffer.
    *  oled_uart_rxBufferRead - cyclic index for read from rxBuffer, 
    *     checked to detect overflow condition.
    *  oled_uart_rxBufferOverflow - software overflow flag. Set to one
    *     when oled_uart_rxBufferWrite index overtakes 
    *     oled_uart_rxBufferRead index.
    *  oled_uart_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when oled_uart_rxBufferWrite is equal to 
    *    oled_uart_rxBufferRead
    *
    *******************************************************************************/
    CY_ISR(oled_uart_RXISR)
    {
        uint8 readData;

        /* User code required at start of ISR */
        /* `#START oled_uart_RXISR_START` */

        /* `#END` */

        readData = oled_uart_RXSTATUS_REG;

        if(readData & (oled_uart_RX_STS_BREAK | oled_uart_RX_STS_PAR_ERROR | \
                       oled_uart_RX_STS_STOP_ERROR | oled_uart_RX_STS_OVERRUN))
        {
            /* ERROR handling. */
            /* `#START oled_uart_RXISR_ERROR` */

            /* `#END` */
        }

        while(readData & oled_uart_RX_STS_FIFO_NOTEMPTY)
        {
            /* Check pointer. */
            if(oled_uart_rxBufferWrite >= oled_uart_RXBUFFERSIZE)
            {
                oled_uart_rxBufferWrite = 0u;
                if(oled_uart_rxBufferWrite == oled_uart_rxBufferRead)
                {   /* Set Software Buffer status Overflow */
                    oled_uart_rxBufferOverflow = 1u;
                }
            }

            oled_uart_rxBuffer[oled_uart_rxBufferWrite] = oled_uart_RXDATA_REG;

            if(oled_uart_rxBufferLoopDetect)
            {   /* Set Software Buffer status Overflow */
                oled_uart_rxBufferOverflow = 1u;
            }
            /* Set next pointer. */
            oled_uart_rxBufferWrite++;

            if(oled_uart_rxBufferWrite == oled_uart_rxBufferRead)
            {
                oled_uart_rxBufferLoopDetect = 1u;
            }

            /* Check again if there is data. */
            readData = oled_uart_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START oled_uart_RXISR_END` */

        /* `#END` */

        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(oled_uart_PSOC3_ES2 && (oled_uart_RXInternalInterrupt__ES2_PATCH))
            oled_uart_ISR_PATCH();
        #endif /* End oled_uart_PSOC3_ES2*/
    }

#endif /* End oled_uart_RX_ENABLED && (oled_uart_RXBUFFERSIZE > oled_uart_FIFO_LENGTH) */


#if(oled_uart_TX_ENABLED && (oled_uart_TXBUFFERSIZE > oled_uart_FIFO_LENGTH))

    extern volatile uint8 oled_uart_txBuffer[];
    extern volatile uint8 oled_uart_txBufferRead;
    extern uint8 oled_uart_txBufferWrite;


    /*******************************************************************************
    * Function Name: oled_uart_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  oled_uart_txBuffer - RAM buffer pointer for transmit data from.
    *  oled_uart_txBufferRead - cyclic index for read and transmit data 
    *     from txBuffer, increments after each transmited byte.
    *  oled_uart_rxBufferWrite - cyclic index for write to txBuffer, 
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(oled_uart_TXISR)
    {

        /* User code required at start of ISR */
        /* `#START oled_uart_TXISR_START` */

        /* `#END` */

        while((oled_uart_txBufferRead != oled_uart_txBufferWrite) && \
             !(oled_uart_TXSTATUS_REG & oled_uart_TX_STS_FIFO_FULL))
        {
            /* Check pointer. */
            if(oled_uart_txBufferRead >= oled_uart_TXBUFFERSIZE)
            {
                oled_uart_txBufferRead = 0u;
            }

            oled_uart_TXDATA_REG = oled_uart_txBuffer[oled_uart_txBufferRead];

            /* Set next pointer. */
            oled_uart_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START oled_uart_TXISR_END` */

        /* `#END` */
        
        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(oled_uart_PSOC3_ES2 && (oled_uart_TXInternalInterrupt__ES2_PATCH))
            oled_uart_ISR_PATCH();
        #endif /* End oled_uart_PSOC3_ES2*/
    }

#endif /* End oled_uart_TX_ENABLED && (oled_uart_TXBUFFERSIZE > oled_uart_FIFO_LENGTH) */


/* [] END OF FILE */
