/*******************************************************************************
* File Name: rs232_INT.c
* Version 2.0
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "rs232.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (rs232_RX_ENABLED || rs232_HD_ENABLED) && \
     (rs232_RXBUFFERSIZE > rs232_FIFO_LENGTH))

    extern volatile uint8 rs232_rxBuffer[];
    extern volatile uint8 rs232_rxBufferRead;
    extern volatile uint8 rs232_rxBufferWrite;
    extern volatile uint8 rs232_rxBufferLoopDetect;
    extern volatile uint8 rs232_rxBufferOverflow;


    /*******************************************************************************
    * Function Name: rs232_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs232_rxBuffer - RAM buffer pointer for save received data.
    *  rs232_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     increments after each byte saved to buffer.
    *  rs232_rxBufferRead - cyclic index for read from rxBuffer, 
    *     checked to detect overflow condition.
    *  rs232_rxBufferOverflow - software overflow flag. Set to one
    *     when rs232_rxBufferWrite index overtakes 
    *     rs232_rxBufferRead index.
    *  rs232_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when rs232_rxBufferWrite is equal to 
    *    rs232_rxBufferRead
    *
    *******************************************************************************/
    CY_ISR(rs232_RXISR)
    {
        uint8 readData;

        /* User code required at start of ISR */
        /* `#START rs232_RXISR_START` */

        /* `#END` */

        readData = rs232_RXSTATUS_REG;

        if(readData & (rs232_RX_STS_BREAK | rs232_RX_STS_PAR_ERROR | \
                       rs232_RX_STS_STOP_ERROR | rs232_RX_STS_OVERRUN))
        {
            /* ERROR handling. */
            /* `#START rs232_RXISR_ERROR` */

            /* `#END` */
        }

        while(readData & rs232_RX_STS_FIFO_NOTEMPTY)
        {
            /* Check pointer. */
            if(rs232_rxBufferWrite >= rs232_RXBUFFERSIZE)
            {
                rs232_rxBufferWrite = 0u;
                if(rs232_rxBufferWrite == rs232_rxBufferRead)
                {   /* Set Software Buffer status Overflow */
                    rs232_rxBufferOverflow = 1u;
                }
            }

            rs232_rxBuffer[rs232_rxBufferWrite] = rs232_RXDATA_REG;

            if(rs232_rxBufferLoopDetect)
            {   /* Set Software Buffer status Overflow */
                rs232_rxBufferOverflow = 1u;
            }
            /* Set next pointer. */
            rs232_rxBufferWrite++;

            if(rs232_rxBufferWrite == rs232_rxBufferRead)
            {
                rs232_rxBufferLoopDetect = 1u;
            }

            /* Check again if there is data. */
            readData = rs232_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START rs232_RXISR_END` */

        /* `#END` */

        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(rs232_PSOC3_ES2 && (rs232_RXInternalInterrupt__ES2_PATCH))
            rs232_ISR_PATCH();
        #endif /* End rs232_PSOC3_ES2*/
    }

#endif /* End rs232_RX_ENABLED && (rs232_RXBUFFERSIZE > rs232_FIFO_LENGTH) */


#if(rs232_TX_ENABLED && (rs232_TXBUFFERSIZE > rs232_FIFO_LENGTH))

    extern volatile uint8 rs232_txBuffer[];
    extern volatile uint8 rs232_txBufferRead;
    extern uint8 rs232_txBufferWrite;


    /*******************************************************************************
    * Function Name: rs232_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs232_txBuffer - RAM buffer pointer for transmit data from.
    *  rs232_txBufferRead - cyclic index for read and transmit data 
    *     from txBuffer, increments after each transmited byte.
    *  rs232_rxBufferWrite - cyclic index for write to txBuffer, 
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(rs232_TXISR)
    {

        /* User code required at start of ISR */
        /* `#START rs232_TXISR_START` */

        /* `#END` */

        while((rs232_txBufferRead != rs232_txBufferWrite) && \
             !(rs232_TXSTATUS_REG & rs232_TX_STS_FIFO_FULL))
        {
            /* Check pointer. */
            if(rs232_txBufferRead >= rs232_TXBUFFERSIZE)
            {
                rs232_txBufferRead = 0u;
            }

            rs232_TXDATA_REG = rs232_txBuffer[rs232_txBufferRead];

            /* Set next pointer. */
            rs232_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START rs232_TXISR_END` */

        /* `#END` */
        
        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(rs232_PSOC3_ES2 && (rs232_TXInternalInterrupt__ES2_PATCH))
            rs232_ISR_PATCH();
        #endif /* End rs232_PSOC3_ES2*/
    }

#endif /* End rs232_TX_ENABLED && (rs232_TXBUFFERSIZE > rs232_FIFO_LENGTH) */


/* [] END OF FILE */
