/*******************************************************************************
* File Name: rs232.c
* Version 2.0
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "rs232.h"


/***************************************
* Local data allocation
***************************************/

static rs232_BACKUP_STRUCT  rs232_backup =
{
    /* enableState - disabled */
    0u,
};        



/*******************************************************************************
* Function Name: rs232_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs232_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs232_SaveConfig(void)
{
    /* PSoC3 ES2 or early, PSoC5 ES1*/
    #if (rs232_PSOC3_ES2 || rs232_PSOC5_ES1)

        #if(rs232_CONTROL_REG_REMOVED == 0u)
            rs232_backup.cr = rs232_CONTROL_REG;
        #endif /* End rs232_CONTROL_REG_REMOVED */

        #if( (rs232_RX_ENABLED) || (rs232_HD_ENABLED) )
            rs232_backup.rx_period = rs232_RXBITCTR_PERIOD_REG;
            rs232_backup.rx_mask = rs232_RXSTATUS_MASK_REG;
            #if (rs232_RXHW_ADDRESS_ENABLED)
                rs232_backup.rx_addr1 = rs232_RXADDRESS1_REG;
                rs232_backup.rx_addr2 = rs232_RXADDRESS2_REG;
            #endif /* End rs232_RXHW_ADDRESS_ENABLED */
        #endif /* End rs232_RX_ENABLED | rs232_HD_ENABLED*/

        #if(rs232_TX_ENABLED)
            #if(rs232_TXCLKGEN_DP)
                rs232_backup.tx_clk_ctr = rs232_TXBITCLKGEN_CTR_REG;
                rs232_backup.tx_clk_compl = rs232_TXBITCLKTX_COMPLETE_REG;
            #else
                rs232_backup.tx_period = rs232_TXBITCTR_PERIOD_REG;
            #endif /*End rs232_TXCLKGEN_DP */
            rs232_backup.tx_mask = rs232_TXSTATUS_MASK_REG;
        #endif /*End rs232_TX_ENABLED */

    /* PSoC3 ES3 or later, PSoC5 ES2 or later*/
    #elif (rs232_PSOC3_ES3 || rs232_PSOC5_ES2)

        #if(rs232_CONTROL_REG_REMOVED == 0u)
            rs232_backup.cr = rs232_CONTROL_REG;
        #endif /* End rs232_CONTROL_REG_REMOVED */

    #endif  /* End PSOC3_ES3 || PSOC5_ES2 */
}


/*******************************************************************************
* Function Name: rs232_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs232_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs232_RestoreConfig(void)
{
    /* PSoC3 ES2 or early, PSoC5 ES1*/
    #if (rs232_PSOC3_ES2 || rs232_PSOC5_ES1)

        #if(rs232_CONTROL_REG_REMOVED == 0u)
            rs232_CONTROL_REG = rs232_backup.cr;
        #endif /* End rs232_CONTROL_REG_REMOVED */

        #if( (rs232_RX_ENABLED) || (rs232_HD_ENABLED) )
            rs232_RXBITCTR_PERIOD_REG = rs232_backup.rx_period;
            rs232_RXSTATUS_MASK_REG = rs232_backup.rx_mask;
            #if (rs232_RXHW_ADDRESS_ENABLED)
                rs232_RXADDRESS1_REG = rs232_backup.rx_addr1;
                rs232_RXADDRESS2_REG = rs232_backup.rx_addr2;
            #endif /* End rs232_RXHW_ADDRESS_ENABLED */
        #endif  /* End (rs232_RX_ENABLED) || (rs232_HD_ENABLED) */

        #if(rs232_TX_ENABLED)
            #if(rs232_TXCLKGEN_DP)
                rs232_TXBITCLKGEN_CTR_REG = rs232_backup.tx_clk_ctr;
                rs232_TXBITCLKTX_COMPLETE_REG = rs232_backup.tx_clk_compl;
            #else
                rs232_TXBITCTR_PERIOD_REG = rs232_backup.tx_period;
            #endif /*End rs232_TXCLKGEN_DP */
            rs232_TXSTATUS_MASK_REG = rs232_backup.tx_mask;
        #endif /*End rs232_TX_ENABLED */

    /* PSoC3 ES3 or later, PSoC5 ES2 or later*/
    #elif (rs232_PSOC3_ES3 || rs232_PSOC5_ES2)

        #if(rs232_CONTROL_REG_REMOVED == 0u)
            rs232_CONTROL_REG = rs232_backup.cr;
        #endif /* End rs232_CONTROL_REG_REMOVED */

    #endif  /* End PSOC3_ES3 || PSOC5_ES2 */
}


/*******************************************************************************
* Function Name: rs232_Sleep
********************************************************************************
*
* Summary:
*  Stops and saves the user configuration. Should be called 
*  just prior to entering sleep.
*  
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs232_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs232_Sleep(void)
{

    #if(rs232_RX_ENABLED || rs232_HD_ENABLED)
        if((rs232_RXSTATUS_ACTL_REG  & rs232_INT_ENABLE) != 0u) 
        {
            rs232_backup.enableState = 1u;
        }
        else
        {
            rs232_backup.enableState = 0u;
        }
    #else
        if((rs232_TXSTATUS_ACTL_REG  & rs232_INT_ENABLE) !=0u)
        {
            rs232_backup.enableState = 1u;
        }
        else
        {
            rs232_backup.enableState = 0u;
        }
    #endif /* End rs232_RX_ENABLED || rs232_HD_ENABLED*/

    rs232_Stop();
    rs232_SaveConfig();
}


/*******************************************************************************
* Function Name: rs232_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called
*  just after awaking from sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs232_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs232_Wakeup(void)
{
    rs232_RestoreConfig();
    #if( (rs232_RX_ENABLED) || (rs232_HD_ENABLED) )
        rs232_ClearRxBuffer();
    #endif /* End (rs232_RX_ENABLED) || (rs232_HD_ENABLED) */
    #if(rs232_TX_ENABLED || rs232_HD_ENABLED)
        rs232_ClearTxBuffer();
    #endif /* End rs232_TX_ENABLED || rs232_HD_ENABLED */

    if(rs232_backup.enableState != 0u)
    {
        rs232_Enable();
    } 
}


/* [] END OF FILE */
