/*******************************************************************************
* File Name: rs485.c
* Version 2.0
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/


#include "rs485.h"
#include "CyLib.h"


/***************************************
* Global data allocation
***************************************/

#if( rs485_TX_ENABLED && (rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH))
    volatile uint8 rs485_txBuffer[rs485_TXBUFFERSIZE];
    volatile uint8 rs485_txBufferRead = 0u;
    uint8 rs485_txBufferWrite = 0u;
#endif /* End rs485_TX_ENABLED */
#if( ( rs485_RX_ENABLED || rs485_HD_ENABLED ) && \
     (rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH) )
    volatile uint8 rs485_rxBuffer[rs485_RXBUFFERSIZE];
    volatile uint8 rs485_rxBufferRead = 0u;
    volatile uint8 rs485_rxBufferWrite = 0u;
    volatile uint8 rs485_rxBufferLoopDetect = 0u;
    volatile uint8 rs485_rxBufferOverflow = 0u;
#endif /* End rs485_RX_ENABLED */


/***************************************
* Local data allocation
***************************************/

uint8 rs485_initVar = 0u;


/*******************************************************************************
* Function Name: rs485_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the UART component.
*  Enable the clock input to enable operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The rs485_intiVar variable is used to indicate initial 
*  configuration of this component. The variable is initialized to zero (0u) 
*  and set to one (1u) the first time UART_Start() is called. This allows for 
*  component initialization without re-initialization in all subsequent calls 
*  to the rs485_Start() routine. 
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs485_Start(void)
{
    /* If not Initialized then initialize all required hardware and software */
    if(rs485_initVar == 0u)
    {
        rs485_Init();
        rs485_initVar = 1u;
    }
    rs485_Enable();
}


/*******************************************************************************
* Function Name: rs485_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  rs485_Start().
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void rs485_Init(void) 
{
    #if(rs485_RX_ENABLED || rs485_HD_ENABLED)

        #if(rs485_RX_INTERRUPT_ENABLED && (rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH))
            /* Set the RX Interrupt. */
            CyIntSetVector(rs485_RX_VECT_NUM,   rs485_RXISR);
            CyIntSetPriority(rs485_RX_VECT_NUM, rs485_RX_PRIOR_NUM);
        #endif /* End rs485_RX_INTERRUPT_ENABLED */

        #if (rs485_RXHW_ADDRESS_ENABLED)
            rs485_SetRxAddressMode(rs485_RXAddressMode);
            rs485_SetRxAddress1(rs485_RXHWADDRESS1);
            rs485_SetRxAddress2(rs485_RXHWADDRESS2);
        #endif /* End rs485_RXHW_ADDRESS_ENABLED */

        /* Configure the Initial RX interrupt mask */
        rs485_RXSTATUS_MASK_REG  = rs485_INIT_RX_INTERRUPTS_MASK;
    #endif /* End rs485_RX_ENABLED || rs485_HD_ENABLED*/

    #if(rs485_TX_ENABLED)
        #if(rs485_TX_INTERRUPT_ENABLED && (rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH))
            /* Set the TX Interrupt. */
            CyIntSetVector(rs485_TX_VECT_NUM,   rs485_TXISR);
            CyIntSetPriority(rs485_TX_VECT_NUM, rs485_TX_PRIOR_NUM);
        #endif /* End rs485_TX_INTERRUPT_ENABLED */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if(rs485_TXCLKGEN_DP)
            rs485_TXBITCLKGEN_CTR_REG = rs485_BIT_CENTER;
            rs485_TXBITCLKTX_COMPLETE_REG = (rs485_NUMBER_OF_DATA_BITS + \
                                                    rs485_NUMBER_OF_START_BIT) * \
                                                    rs485_OVER_SAMPLE_COUNT;
        #else
            rs485_TXBITCTR_COUNTER_REG = (rs485_NUMBER_OF_DATA_BITS + \
                                                    rs485_NUMBER_OF_START_BIT) * \
                                                    rs485_OVER_SAMPLE_8;
        #endif /* End rs485_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if(rs485_TX_INTERRUPT_ENABLED && (rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH))
            rs485_TXSTATUS_MASK_REG = rs485_TX_STS_FIFO_EMPTY;
        #else
            rs485_TXSTATUS_MASK_REG = rs485_INIT_TX_INTERRUPTS_MASK;
        #endif /*End rs485_TX_INTERRUPT_ENABLED*/
        
    #endif /* End rs485_TX_ENABLED */

    #if(rs485_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        rs485_WriteControlRegister( \
            (rs485_ReadControlRegister() & ~rs485_CTRL_PARITY_TYPE_MASK) | \
            (rs485_PARITY_TYPE << rs485_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End rs485_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: rs485_Enable
********************************************************************************
*
* Summary:
*  Enables the UART block operation
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void rs485_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();
    
    #if(rs485_RX_ENABLED || rs485_HD_ENABLED)
        /*RX Counter (Count7) Enable */
        rs485_RXBITCTR_CONTROL_REG |= rs485_CNTR_ENABLE;
        /* Enable the RX Interrupt. */
        rs485_RXSTATUS_ACTL_REG  |= rs485_INT_ENABLE;
        #if(rs485_RX_INTERRUPT_ENABLED && (rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH))
            CyIntEnable(rs485_RX_VECT_NUM);
        #endif /* End rs485_RX_INTERRUPT_ENABLED */
    #endif /* End rs485_RX_ENABLED || rs485_HD_ENABLED*/

    #if(rs485_TX_ENABLED)
        /*TX Counter (DP/Count7) Enable */
        #if(!rs485_TXCLKGEN_DP)
            rs485_TXBITCTR_CONTROL_REG |= rs485_CNTR_ENABLE;
        #endif /* End rs485_TXCLKGEN_DP */
        /* Enable the TX Interrupt. */
        rs485_TXSTATUS_ACTL_REG |= rs485_INT_ENABLE;
        #if(rs485_TX_INTERRUPT_ENABLED && (rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH))
            CyIntEnable(rs485_TX_VECT_NUM);
        #endif /* End rs485_TX_INTERRUPT_ENABLED*/
     #endif /* End rs485_TX_ENABLED */

    #if(rs485_INTERNAL_CLOCK_USED)
        /* Set the bit to enable the clock. */
        rs485_INTCLOCK_CLKEN_REG |= rs485_INTCLOCK_CLKEN_MASK;
    #endif /* End rs485_INTERNAL_CLOCK_USED */
    
    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: rs485_Stop
********************************************************************************
*
* Summary:
*  Disable the UART component
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void rs485_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /*Write Bit Counter Disable */
    #if(rs485_RX_ENABLED || rs485_HD_ENABLED)
        rs485_RXBITCTR_CONTROL_REG &= ~rs485_CNTR_ENABLE;
    #endif /* End rs485_RX_ENABLED */

    #if(rs485_TX_ENABLED)
        #if(!rs485_TXCLKGEN_DP)
            rs485_TXBITCTR_CONTROL_REG &= ~rs485_CNTR_ENABLE;
        #endif /* End rs485_TXCLKGEN_DP */
    #endif /* rs485_TX_ENABLED */

    #if(rs485_INTERNAL_CLOCK_USED)
        /* Clear the bit to enable the clock. */
        rs485_INTCLOCK_CLKEN_REG &= ~rs485_INTCLOCK_CLKEN_MASK;
    #endif /* End rs485_INTERNAL_CLOCK_USED */
    
    /*Disable internal interrupt component*/
    #if(rs485_RX_ENABLED || rs485_HD_ENABLED)
        rs485_RXSTATUS_ACTL_REG  &= ~rs485_INT_ENABLE;
        #if(rs485_RX_INTERRUPT_ENABLED && (rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH))
            rs485_DisableRxInt();
        #endif /* End rs485_RX_INTERRUPT_ENABLED */
    #endif /* End rs485_RX_ENABLED */
    
    #if(rs485_TX_ENABLED)
        rs485_TXSTATUS_ACTL_REG &= ~rs485_INT_ENABLE;
        #if(rs485_TX_INTERRUPT_ENABLED && (rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH))
            rs485_DisableTxInt();
        #endif /* End rs485_TX_INTERRUPT_ENABLED */
    #endif /* End rs485_TX_ENABLED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: rs485_ReadControlRegister
********************************************************************************
*
* Summary:
*  Read the current state of the control register
*
* Parameters:
*  None.
*
* Return:
*  Current state of the control register.
*
*******************************************************************************/
uint8 rs485_ReadControlRegister(void) 
{
    #if( rs485_CONTROL_REG_REMOVED )
        return(0u);
    #else
        return(rs485_CONTROL_REG);
    #endif /* End rs485_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: rs485_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  rs485_WriteControlRegister(uint8 control) 
{
    #if( rs485_CONTROL_REG_REMOVED )
        control = control;      /* Reassigning to release compiler warning */ 
    #else
       rs485_CONTROL_REG = control;
    #endif /* End rs485_CONTROL_REG_REMOVED */
}


#if(rs485_RX_ENABLED || rs485_HD_ENABLED)

    #if(rs485_RX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: rs485_EnableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void rs485_EnableRxInt(void) 
        {
            CyIntEnable(rs485_RX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: rs485_DisableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void rs485_DisableRxInt(void) 
        {
            CyIntDisable(rs485_RX_VECT_NUM);
        }

    #endif /* rs485_RX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: rs485_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  IntSrc:  An or'd combination of the desired status bit masks (defined in
    *           the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void rs485_SetRxInterruptMode(uint8 intSrc) 
    {
        rs485_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: rs485_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns data in RX Data register without checking status register to 
    *  determine if data is valid
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  rs485_rxBuffer - RAM buffer pointer for save received data.
    *  rs485_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     checked to identify new data. 
    *  rs485_rxBufferRead - cyclic index for read from rxBuffer, 
    *     incremented after each byte has been read from buffer.
    *  rs485_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR. 
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 rs485_ReadRxData(void)
    {
        uint8 rxData;

        #if(rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_DisableRxInt();
            #endif /* End rs485_RX_INTERRUPT_ENABLED */

            if( (rs485_rxBufferRead != rs485_rxBufferWrite) || \
                (rs485_rxBufferLoopDetect > 0u) )
            {
                if(rs485_rxBufferLoopDetect > 0u ) 
                {
                    rs485_rxBufferLoopDetect = 0u;
                }

                if(rs485_rxBufferRead >= rs485_RXBUFFERSIZE)
                {
                    rs485_rxBufferRead = 0u;
                }

                rxData = rs485_rxBuffer[rs485_rxBufferRead];

                rs485_rxBufferRead++;
            }
            else
            {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
                rxData = rs485_RXDATA_REG;
            }

            /* Enable Rx interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_EnableRxInt();
            #endif /* End rs485_RX_INTERRUPT_ENABLED */

        #else /* rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH */

            /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
            rxData = rs485_RXDATA_REG;

        #endif /* rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: rs485_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the current state of the status register
    *  And detect software buffer overflow.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Global Variables:
    *  rs485_rxBufferOverflow - used to indicate overload condition. 
    *   It set to one in RX interrupt when there isn?t free space in 
    *   rs485_rxBufferRead to write new data. This condition returned 
    *   and cleared to zero by this API as an 
    *   rs485_RX_STS_SOFT_BUFF_OVER bit along with RX Status register 
    *   bits.
    *
    *******************************************************************************/
    uint8 rs485_ReadRxStatus(void) 
    {
        uint8 status;

        status = rs485_RXSTATUS_REG;
        status &= rs485_RX_HW_MASK;

        #if(rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH)
            if( rs485_rxBufferOverflow )
            {
                status |= rs485_RX_STS_SOFT_BUFF_OVER;
                rs485_rxBufferOverflow = 0u;
            }
        #endif /* rs485_RXBUFFERSIZE */

        return(status);
    }


    /*******************************************************************************
    * Function Name: rs485_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, if data is not available or an error 
    *  condition exists, zero is returned; otherwise, character is read and 
    *  returned.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  rs485_rxBuffer - RAM buffer pointer for save received data.
    *  rs485_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     checked to identify new data. 
    *  rs485_rxBufferRead - cyclic index for read from rxBuffer, 
    *     incremented after each byte has been read from buffer.
    *  rs485_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR. 
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 rs485_GetChar(void)
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

        #if(rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_DisableRxInt();
            #endif /* rs485_RX_INTERRUPT_ENABLED */

            if( (rs485_rxBufferRead != rs485_rxBufferWrite) || \
                (rs485_rxBufferLoopDetect > 0u) )
            {
                if(rs485_rxBufferLoopDetect > 0u ) 
                {
                    rs485_rxBufferLoopDetect = 0u;
                }

                if(rs485_rxBufferRead >= rs485_RXBUFFERSIZE)
                {
                    rs485_rxBufferRead = 0u;
                }

                rxData = rs485_rxBuffer[rs485_rxBufferRead];

                rs485_rxBufferRead++;
            }
            else
            {   rxStatus =rs485_RXSTATUS_REG;
                if(rxStatus & rs485_RX_STS_FIFO_NOTEMPTY)
                {   /* Read received data from FIFO*/
                    rxData = rs485_RXDATA_REG;
                    /*Check status on error*/
                    if(rxStatus & (rs485_RX_STS_BREAK | rs485_RX_STS_PAR_ERROR | \
                                   rs485_RX_STS_STOP_ERROR | rs485_RX_STS_OVERRUN))
                    {
                        rxData = 0u;
                    }    
                }
            }

            /* Enable Rx interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_EnableRxInt();
            #endif /* rs485_RX_INTERRUPT_ENABLED */

        #else /* rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH */

            rxStatus =rs485_RXSTATUS_REG;
            if(rxStatus & rs485_RX_STS_FIFO_NOTEMPTY)
            {   /* Read received data from FIFO*/
                rxData = rs485_RXDATA_REG;
                /*Check status on error*/
                if(rxStatus & (rs485_RX_STS_BREAK | rs485_RX_STS_PAR_ERROR | \
                               rs485_RX_STS_STOP_ERROR | rs485_RX_STS_OVERRUN))
                {
                    rxData = 0u;
                }
            }
        #endif /* rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: rs485_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Grab the next available byte of data from the recieve FIFO
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains Status Register and LSB contains UART RX data
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 rs485_GetByte(void)
    {
        return ( ((uint16)rs485_ReadRxStatus() << 8u) | rs485_ReadRxData() );
    }


    /*******************************************************************************
    * Function Name: rs485_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of bytes left in the RX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Integer count of the number of bytes left 
    *  in the RX buffer
    *
    * Global Variables:
    *  rs485_rxBufferWrite - used to calculate left bytes. 
    *  rs485_rxBufferRead - used to calculate left bytes.
    *  rs485_rxBufferLoopDetect - checked to decide left bytes amount. 
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 rs485_GetRxBufferSize(void)
    {
        uint8 size;

        #if(rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_DisableRxInt();
            #endif /* rs485_RX_INTERRUPT_ENABLED */

            if(rs485_rxBufferRead == rs485_rxBufferWrite)
            {
                if(rs485_rxBufferLoopDetect > 0u)
                {
                    size = rs485_RXBUFFERSIZE;
                }
                else
                {
                    size = 0u;
                }
            }
            else if(rs485_rxBufferRead < rs485_rxBufferWrite)
            {
                size = (rs485_rxBufferWrite - rs485_rxBufferRead);
            }
            else
            {
                size = (rs485_RXBUFFERSIZE - rs485_rxBufferRead) + rs485_rxBufferWrite;
            }

            /* Enable Rx interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_EnableRxInt();
            #endif /* End rs485_RX_INTERRUPT_ENABLED */

        #else /* rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH */

            /* We can only know if there is data in the fifo. */
            size = (rs485_RXSTATUS_REG & rs485_RX_STS_FIFO_NOTEMPTY) ? 1u : 0u;

        #endif /* End rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: rs485_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clear the RX RAM buffer by setting the read and write pointers both to zero.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_rxBufferWrite - cleared to zero. 
    *  rs485_rxBufferRead - cleared to zero.
    *  rs485_rxBufferLoopDetect - cleared to zero. 
    *  rs485_rxBufferOverflow - cleared to zero. 
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to 
    *  read and writing will resume at address 0 overwriting any data that may 
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *******************************************************************************/
    void rs485_ClearRxBuffer(void)
    {
        #if(rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH)
            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_DisableRxInt();
            #endif /* End rs485_RX_INTERRUPT_ENABLED */

            rs485_rxBufferRead = 0u;
            rs485_rxBufferWrite = 0u;
            rs485_rxBufferLoopDetect = 0u;
            rs485_rxBufferOverflow = 0u;

            /* Enable Rx interrupt. */
            #if(rs485_RX_INTERRUPT_ENABLED)
                rs485_EnableRxInt();
            #endif /* End rs485_RX_INTERRUPT_ENABLED */
        #endif /* End rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: rs485_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the receive addressing mode
    *
    * Parameters:
    *  addressMode: Use one of the Enumerated Types listed below
    *              #define rs485__B_UART__AM_SW_BYTE_BYTE 1
#define rs485__B_UART__AM_SW_DETECT_TO_BUFFER 2
#define rs485__B_UART__AM_HW_BYTE_BY_BYTE 3
#define rs485__B_UART__AM_HW_DETECT_TO_BUFFER 4
#define rs485__B_UART__AM_NONE 0

    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void rs485_SetRxAddressMode(uint8 addressMode)  
    {
        #if(rs485_RXHW_ADDRESS_ENABLED)
            #if(rs485_CONTROL_REG_REMOVED)
                addressMode = addressMode;
            #else /* rs485_CONTROL_REG_REMOVED */
                uint8 tmpCtrl = 0u;
                tmpCtrl = rs485_CONTROL_REG & ~rs485_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= ((addressMode << rs485_CTRL_RXADDR_MODE0_SHIFT) & \
                           rs485_CTRL_RXADDR_MODE_MASK);
                rs485_CONTROL_REG = tmpCtrl;
            #endif /* End rs485_CONTROL_REG_REMOVED */
        #else /* rs485_RXHW_ADDRESS_ENABLED */
            addressMode = addressMode;
        #endif /* End rs485_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: rs485_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Set the first hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void rs485_SetRxAddress1(uint8 address) 

    {
        rs485_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: rs485_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Set the second hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void rs485_SetRxAddress2(uint8 address) 
    {
        rs485_RXADDRESS2_REG = address;
    }

#endif  /* rs485_RX_ENABLED || rs485_HD_ENABLED*/


#if( (rs485_TX_ENABLED) || (rs485_HD_ENABLED) )

    #if(rs485_TX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: rs485_EnableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void rs485_EnableTxInt(void) 
        {
            CyIntEnable(rs485_TX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: rs485_DisableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void rs485_DisableTxInt(void) 
        {
            CyIntDisable(rs485_TX_VECT_NUM);
        }

    #endif /* rs485_TX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: rs485_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  intSrc: An or'd combination of the desired status bit masks (defined in
    *          the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void rs485_SetTxInterruptMode(uint8 intSrc) 
    {
        rs485_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: rs485_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Write a byte of data to the Transmit FIFO or TX buffer to be sent when the 
    *  bus is available. WriteTxData sends a byte without checking for buffer room 
    *  or status. It is up to the user to separately check status.    
    *
    * Parameters:
    *  TXDataByte: byte of data to place in the transmit FIFO
    *
    * Return:
    * void
    *
    * Global Variables:
    *  rs485_txBuffer - RAM buffer pointer for save data for transmission
    *  rs485_txBufferWrite - cyclic index for write to txBuffer, 
    *    incremented after each byte saved to buffer.
    *  rs485_txBufferRead - cyclic index for read from txBuffer, 
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  rs485_initVar - checked to identify that the component has been  
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void rs485_WriteTxData(uint8 txDataByte)
    {
        /* If not Initialized then skip this function*/
        if(rs485_initVar != 0u)
        {
            #if(rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH)

                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(rs485_TX_INTERRUPT_ENABLED)
                    rs485_DisableTxInt();
                #endif /* End rs485_TX_INTERRUPT_ENABLED */

                if( (rs485_txBufferRead == rs485_txBufferWrite) && \
                   !(rs485_TXSTATUS_REG & rs485_TX_STS_FIFO_FULL) )
                {
                    /* Add directly to the FIFO. */
                    rs485_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(rs485_txBufferWrite >= rs485_TXBUFFERSIZE)
                    {
                        rs485_txBufferWrite = 0;
                    }

                    rs485_txBuffer[rs485_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    rs485_txBufferWrite++;

                }

                /* Enable Tx interrupt. */
                #if(rs485_TX_INTERRUPT_ENABLED)
                    rs485_EnableTxInt();
                #endif /* End rs485_TX_INTERRUPT_ENABLED */

            #else /* rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH */

                /* Add directly to the FIFO. */
                rs485_TXDATA_REG = txDataByte;

            #endif /* End rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH */
        }
    }


    /*******************************************************************************
    * Function Name: rs485_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the status register for the component
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the status register which is clear on read. It is up to 
    *  the user to handle all bits in this return value accordingly, even if the bit 
    *  was not enabled as an interrupt source the event happened and must be handled
    *  accordingly.    
    *
    *******************************************************************************/
    uint8 rs485_ReadTxStatus(void) 
    {
        return(rs485_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: rs485_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Wait to send byte until TX register or buffer has room.
    *
    * Parameters:
    *  txDataByte: The 8-bit data value to send across the UART.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_txBuffer - RAM buffer pointer for save data for transmission
    *  rs485_txBufferWrite - cyclic index for write to txBuffer, 
    *     checked to identify free space in txBuffer and incremented after each byte 
    *     saved to buffer.
    *  rs485_txBufferRead - cyclic index for read from txBuffer, 
    *     checked to identify free space in txBuffer.
    *  rs485_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void rs485_PutChar(uint8 txDataByte)
    {
            #if(rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH)

                /* Block if buffer is full, so we dont overwrite. */
                while( rs485_txBufferWrite == (rs485_txBufferRead - 1u) || \
                    (uint8)(rs485_txBufferWrite - rs485_txBufferRead) ==
                    (uint8)(rs485_TXBUFFERSIZE - 1u) )
                {
                    /* Software buffer is full. */
                }
                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(rs485_TX_INTERRUPT_ENABLED)
                    rs485_DisableTxInt();
                #endif /* End rs485_TX_INTERRUPT_ENABLED */

                if( (rs485_txBufferRead == rs485_txBufferWrite) && \
                   !(rs485_TXSTATUS_REG & rs485_TX_STS_FIFO_FULL) )
                {
                    /* Add directly to the FIFO. */
                    rs485_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(rs485_txBufferWrite >= rs485_TXBUFFERSIZE)
                    {
                        rs485_txBufferWrite = 0;
                    }

                    rs485_txBuffer[rs485_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    rs485_txBufferWrite++;

                }

                /* Enable Rx interrupt. */
                #if(rs485_TX_INTERRUPT_ENABLED)
                    rs485_EnableTxInt();
                #endif /* End rs485_TX_INTERRUPT_ENABLED */

            #else /* rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH */

                /* Block if there isnt room. */
                while(rs485_TXSTATUS_REG & rs485_TX_STS_FIFO_FULL);

                /* Add directly to the FIFO. */
                rs485_TXDATA_REG = txDataByte;

            #endif /* End rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: rs485_PutString
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: uint8 pointer to character string of Data to Send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  This function will block if there is not enough memory to place the whole 
    *  string, it will block until the entire string has been written to the 
    *  transmit buffer.
    *
    *******************************************************************************/
    void rs485_PutString(uint8* string)
    {
        /* If not Initialized then skip this function*/
        if(rs485_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent*/
            while(*string != 0u)
            {
                rs485_PutChar(*string++);
            }
        }
    }


    /*******************************************************************************
    * Function Name: rs485_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of Bytes to be transmitted.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void rs485_PutArray(uint8* string, uint8 byteCount)
    {
        /* If not Initialized then skip this function*/
        if(rs485_initVar != 0u)
        {
            while(byteCount > 0u)
            {
                rs485_PutChar(*string++);
                byteCount--;
            }
        }
    }


    /*******************************************************************************
    * Function Name: rs485_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Write a character and then carriage return and line feed.
    *
    * Parameters:
    *  txDataByte: uint8 Character to send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_initVar - checked to identify that the component has been  
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void rs485_PutCRLF(uint8 txDataByte)
    {
        /* If not Initialized then skip this function*/
        if(rs485_initVar != 0u)
        {
            rs485_PutChar(txDataByte);
            rs485_PutChar(0x0Du);
            rs485_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: rs485_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of space left in the TX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Integer count of the number of bytes left in the TX buffer
    *
    * Global Variables:
    *  rs485_txBufferWrite - used to calculate left space. 
    *  rs485_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 rs485_GetTxBufferSize(void)
    {
        uint8 size;

        #if(rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(rs485_TX_INTERRUPT_ENABLED)
                rs485_DisableTxInt();
            #endif /* End rs485_TX_INTERRUPT_ENABLED */

            if(rs485_txBufferRead == rs485_txBufferWrite)
            {
                size = 0u;
            }
            else if(rs485_txBufferRead < rs485_txBufferWrite)
            {
                size = (rs485_txBufferWrite - rs485_txBufferRead);
            }
            else
            {
                size = (rs485_TXBUFFERSIZE - rs485_txBufferRead) + rs485_txBufferWrite;
            }

            /* Enable Tx interrupt. */
            #if(rs485_TX_INTERRUPT_ENABLED)
                rs485_EnableTxInt();
            #endif /* End rs485_TX_INTERRUPT_ENABLED */

        #else /* rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH */

            size = rs485_TXSTATUS_REG;

            /* Is the fifo is full. */
            if(size & rs485_TX_STS_FIFO_FULL)
            {
                size = rs485_FIFO_LENGTH;
            }
            else if(size & rs485_TX_STS_FIFO_EMPTY)
            {
                size = 0u;
            }
            else
            {
                /* We only know there is data in the fifo. */
                size = 1u;
            }

        #endif /* End rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: rs485_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clear the TX RAM buffer by setting the read and write pointers both to zero.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_txBufferWrite - cleared to zero. 
    *  rs485_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to 
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *
    *******************************************************************************/
    void rs485_ClearTxBuffer(void)
    {
        #if(rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(rs485_TX_INTERRUPT_ENABLED)
                rs485_DisableTxInt();
            #endif /* End rs485_TX_INTERRUPT_ENABLED */

            rs485_txBufferRead = 0u;
            rs485_txBufferWrite = 0u;

            /* Enable Tx interrupt. */
            #if(rs485_TX_INTERRUPT_ENABLED)
                rs485_EnableTxInt();
            #endif /* End rs485_TX_INTERRUPT_ENABLED */

        #endif /* End rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: rs485_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Write a Break command to the UART
    *
    * Parameters:
    *  uint8 retMode:  Wait mode,
    *   0 - Initialize registers for Break, sends the Break signal and return 
    *       imediately.
    *   1 - Wait until Break sending is complete, reinitialize registers to normal
    *       transmission mode then return.
    *   2 - Reinitialize registers to normal transmission mode then return.
    *   3 - both steps: 0 and 1
    *       init registers for Break, send Break signal
    *       wait until Break sending is complete, reinit registers to normal
    *       transmission mode then return.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_initVar - checked to identify that the component has been  
    *     initialized.
    *  tx_period - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  Trere are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Funcition will block CPU untill transmition 
    *     complete.
    *  2) User may want to use bloking time if UART configured to the low speed 
    *     operation
    *     Emample for this case:
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to init and use the interrupt for complete 
    *     break operation.
    *     Example for this case:
    *     Init TX interrupt whith "TX - On TX Complete" parameter
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     When interrupt appear with UART_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *   Uses static variable to keep registers configuration.
    *
    *******************************************************************************/
    void rs485_SendBreak(uint8 retMode)
    {

        /* If not Initialized then skip this function*/
        if(rs485_initVar != 0u)
        {
            /*Set the Counter to 13-bits and transmit a 00 byte*/
            /*When that is done then reset the counter value back*/
            uint8 tmpStat;

            #if(rs485_HD_ENABLED) /* Half Duplex mode*/

                if( (retMode == rs485_SEND_BREAK) ||
                    (retMode == rs485_SEND_WAIT_REINIT ) )
                {
                    /* CTRL_HD_SEND_BREAK - sends break bits in HD mode*/
                    rs485_WriteControlRegister(rs485_ReadControlRegister() | \
                                                          rs485_CTRL_HD_SEND_BREAK);
                    /* Send zeros*/
                    rs485_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = rs485_TXSTATUS_REG;
                    }while(tmpStat & rs485_TX_STS_FIFO_EMPTY);
                }

                if( (retMode == rs485_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == rs485_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = rs485_TXSTATUS_REG;
                    }while(~tmpStat & rs485_TX_STS_COMPLETE);
                }

                if( (retMode == rs485_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == rs485_REINIT) || \
                    (retMode == rs485_SEND_WAIT_REINIT) )
                {
                    rs485_WriteControlRegister(rs485_ReadControlRegister() & \
                                                         ~rs485_CTRL_HD_SEND_BREAK);
                }

            #else /* rs485_HD_ENABLED Full Duplex mode */

                static uint8 tx_period; 
                
                if( (retMode == rs485_SEND_BREAK) || \
                    (retMode == rs485_SEND_WAIT_REINIT) )
                {
                    /* CTRL_HD_SEND_BREAK - skip to send parity bit @ Break signal in Full Duplex mode*/
                    if( (rs485_PARITY_TYPE != rs485__B_UART__NONE_REVB) || \
                         rs485_PARITY_TYPE_SW )
                    {
                        rs485_WriteControlRegister(rs485_ReadControlRegister() | \
                                                              rs485_CTRL_HD_SEND_BREAK);
                    }                                                          

                    #if(rs485_TXCLKGEN_DP)
                        tx_period = rs485_TXBITCLKTX_COMPLETE_REG;
                        rs485_TXBITCLKTX_COMPLETE_REG = rs485_TXBITCTR_BREAKBITS;
                    #else
                        tx_period = rs485_TXBITCTR_PERIOD_REG;
                        rs485_TXBITCTR_PERIOD_REG = rs485_TXBITCTR_BREAKBITS8X;
                    #endif /* End rs485_TXCLKGEN_DP */

                    /* Send zeros*/
                    rs485_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = rs485_TXSTATUS_REG;
                    }while(tmpStat & rs485_TX_STS_FIFO_EMPTY);
                }

                if( (retMode == rs485_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == rs485_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = rs485_TXSTATUS_REG;
                    }while(~tmpStat & rs485_TX_STS_COMPLETE);
                }

                if( (retMode == rs485_WAIT_FOR_COMLETE_REINIT) || \
                    (retMode == rs485_REINIT) || \
                    (retMode == rs485_SEND_WAIT_REINIT) )
                {

                    #if(rs485_TXCLKGEN_DP)
                        rs485_TXBITCLKTX_COMPLETE_REG = tx_period;
                    #else
                        rs485_TXBITCTR_PERIOD_REG = tx_period;
                    #endif /* End rs485_TXCLKGEN_DP */

                    if( (rs485_PARITY_TYPE != rs485__B_UART__NONE_REVB) || 
                         rs485_PARITY_TYPE_SW )
                    {
                        rs485_WriteControlRegister(rs485_ReadControlRegister() & \
                                                             ~rs485_CTRL_HD_SEND_BREAK);
                    }                                     
                }
            #endif    /* End rs485_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: rs485_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the transmit addressing mode
    *
    * Parameters:
    *  addressMode: 0 -> Space
    *               1 -> Mark
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void rs485_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable*/
        if(addressMode != 0)
        {
            rs485_WriteControlRegister(rs485_ReadControlRegister() | rs485_CTRL_MARK);
        }
        else
        {
            rs485_WriteControlRegister(rs485_ReadControlRegister() & ~rs485_CTRL_MARK);
        }
    }

#endif  /* Endrs485_TX_ENABLED */

#if(rs485_HD_ENABLED)


    /*******************************************************************************
    * Function Name: rs485_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Rx configuration if required and loads the
    *  Tx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Tx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART. 
    *
    *******************************************************************************/
    void rs485_LoadTxConfig(void) 
    {
        rs485_WriteControlRegister(rs485_ReadControlRegister() | rs485_CTRL_HD_SEND);
        rs485_RXBITCTR_PERIOD_REG = rs485_HD_TXBITCTR_INIT;
    }


    /*******************************************************************************
    * Function Name: rs485_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Tx configuration if required and loads the
    *  Rx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Rx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART
    *
    *******************************************************************************/
    void rs485_LoadRxConfig(void) 
    {
        rs485_WriteControlRegister(rs485_ReadControlRegister() & ~rs485_CTRL_HD_SEND);
        rs485_RXBITCTR_PERIOD_REG = rs485_HD_RXBITCTR_INIT;
    }

#endif  /* rs485_HD_ENABLED */


/* [] END OF FILE */
