/*******************************************************************************
* File Name: rs485_INT.c
* Version 2.0
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "rs485.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (rs485_RX_ENABLED || rs485_HD_ENABLED) && \
     (rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH))

    extern volatile uint8 rs485_rxBuffer[];
    extern volatile uint8 rs485_rxBufferRead;
    extern volatile uint8 rs485_rxBufferWrite;
    extern volatile uint8 rs485_rxBufferLoopDetect;
    extern volatile uint8 rs485_rxBufferOverflow;


    /*******************************************************************************
    * Function Name: rs485_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_rxBuffer - RAM buffer pointer for save received data.
    *  rs485_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     increments after each byte saved to buffer.
    *  rs485_rxBufferRead - cyclic index for read from rxBuffer, 
    *     checked to detect overflow condition.
    *  rs485_rxBufferOverflow - software overflow flag. Set to one
    *     when rs485_rxBufferWrite index overtakes 
    *     rs485_rxBufferRead index.
    *  rs485_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when rs485_rxBufferWrite is equal to 
    *    rs485_rxBufferRead
    *
    *******************************************************************************/
    CY_ISR(rs485_RXISR)
    {
        uint8 readData;

        /* User code required at start of ISR */
        /* `#START rs485_RXISR_START` */

        /* `#END` */

        readData = rs485_RXSTATUS_REG;

        if(readData & (rs485_RX_STS_BREAK | rs485_RX_STS_PAR_ERROR | \
                       rs485_RX_STS_STOP_ERROR | rs485_RX_STS_OVERRUN))
        {
            /* ERROR handling. */
            /* `#START rs485_RXISR_ERROR` */

            /* `#END` */
        }

        while(readData & rs485_RX_STS_FIFO_NOTEMPTY)
        {
            /* Check pointer. */
            if(rs485_rxBufferWrite >= rs485_RXBUFFERSIZE)
            {
                rs485_rxBufferWrite = 0u;
                if(rs485_rxBufferWrite == rs485_rxBufferRead)
                {   /* Set Software Buffer status Overflow */
                    rs485_rxBufferOverflow = 1u;
                }
            }

            rs485_rxBuffer[rs485_rxBufferWrite] = rs485_RXDATA_REG;

            if(rs485_rxBufferLoopDetect)
            {   /* Set Software Buffer status Overflow */
                rs485_rxBufferOverflow = 1u;
            }
            /* Set next pointer. */
            rs485_rxBufferWrite++;

            if(rs485_rxBufferWrite == rs485_rxBufferRead)
            {
                rs485_rxBufferLoopDetect = 1u;
            }

            /* Check again if there is data. */
            readData = rs485_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START rs485_RXISR_END` */

        /* `#END` */

        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(rs485_PSOC3_ES2 && (rs485_RXInternalInterrupt__ES2_PATCH))
            rs485_ISR_PATCH();
        #endif /* End rs485_PSOC3_ES2*/
    }

#endif /* End rs485_RX_ENABLED && (rs485_RXBUFFERSIZE > rs485_FIFO_LENGTH) */


#if(rs485_TX_ENABLED && (rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH))

    extern volatile uint8 rs485_txBuffer[];
    extern volatile uint8 rs485_txBufferRead;
    extern uint8 rs485_txBufferWrite;


    /*******************************************************************************
    * Function Name: rs485_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  rs485_txBuffer - RAM buffer pointer for transmit data from.
    *  rs485_txBufferRead - cyclic index for read and transmit data 
    *     from txBuffer, increments after each transmited byte.
    *  rs485_rxBufferWrite - cyclic index for write to txBuffer, 
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(rs485_TXISR)
    {

        /* User code required at start of ISR */
        /* `#START rs485_TXISR_START` */

        /* `#END` */

        while((rs485_txBufferRead != rs485_txBufferWrite) && \
             !(rs485_TXSTATUS_REG & rs485_TX_STS_FIFO_FULL))
        {
            /* Check pointer. */
            if(rs485_txBufferRead >= rs485_TXBUFFERSIZE)
            {
                rs485_txBufferRead = 0u;
            }

            rs485_TXDATA_REG = rs485_txBuffer[rs485_txBufferRead];

            /* Set next pointer. */
            rs485_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START rs485_TXISR_END` */

        /* `#END` */
        
        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(rs485_PSOC3_ES2 && (rs485_TXInternalInterrupt__ES2_PATCH))
            rs485_ISR_PATCH();
        #endif /* End rs485_PSOC3_ES2*/
    }

#endif /* End rs485_TX_ENABLED && (rs485_TXBUFFERSIZE > rs485_FIFO_LENGTH) */


/* [] END OF FILE */
