/*******************************************************************************
* File Name: rs485.c
* Version 2.0
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "rs485.h"


/***************************************
* Local data allocation
***************************************/

static rs485_BACKUP_STRUCT  rs485_backup =
{
    /* enableState - disabled */
    0u,
};        



/*******************************************************************************
* Function Name: rs485_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs485_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs485_SaveConfig(void)
{
    /* PSoC3 ES2 or early, PSoC5 ES1*/
    #if (rs485_PSOC3_ES2 || rs485_PSOC5_ES1)

        #if(rs485_CONTROL_REG_REMOVED == 0u)
            rs485_backup.cr = rs485_CONTROL_REG;
        #endif /* End rs485_CONTROL_REG_REMOVED */

        #if( (rs485_RX_ENABLED) || (rs485_HD_ENABLED) )
            rs485_backup.rx_period = rs485_RXBITCTR_PERIOD_REG;
            rs485_backup.rx_mask = rs485_RXSTATUS_MASK_REG;
            #if (rs485_RXHW_ADDRESS_ENABLED)
                rs485_backup.rx_addr1 = rs485_RXADDRESS1_REG;
                rs485_backup.rx_addr2 = rs485_RXADDRESS2_REG;
            #endif /* End rs485_RXHW_ADDRESS_ENABLED */
        #endif /* End rs485_RX_ENABLED | rs485_HD_ENABLED*/

        #if(rs485_TX_ENABLED)
            #if(rs485_TXCLKGEN_DP)
                rs485_backup.tx_clk_ctr = rs485_TXBITCLKGEN_CTR_REG;
                rs485_backup.tx_clk_compl = rs485_TXBITCLKTX_COMPLETE_REG;
            #else
                rs485_backup.tx_period = rs485_TXBITCTR_PERIOD_REG;
            #endif /*End rs485_TXCLKGEN_DP */
            rs485_backup.tx_mask = rs485_TXSTATUS_MASK_REG;
        #endif /*End rs485_TX_ENABLED */

    /* PSoC3 ES3 or later, PSoC5 ES2 or later*/
    #elif (rs485_PSOC3_ES3 || rs485_PSOC5_ES2)

        #if(rs485_CONTROL_REG_REMOVED == 0u)
            rs485_backup.cr = rs485_CONTROL_REG;
        #endif /* End rs485_CONTROL_REG_REMOVED */

    #endif  /* End PSOC3_ES3 || PSOC5_ES2 */
}


/*******************************************************************************
* Function Name: rs485_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs485_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs485_RestoreConfig(void)
{
    /* PSoC3 ES2 or early, PSoC5 ES1*/
    #if (rs485_PSOC3_ES2 || rs485_PSOC5_ES1)

        #if(rs485_CONTROL_REG_REMOVED == 0u)
            rs485_CONTROL_REG = rs485_backup.cr;
        #endif /* End rs485_CONTROL_REG_REMOVED */

        #if( (rs485_RX_ENABLED) || (rs485_HD_ENABLED) )
            rs485_RXBITCTR_PERIOD_REG = rs485_backup.rx_period;
            rs485_RXSTATUS_MASK_REG = rs485_backup.rx_mask;
            #if (rs485_RXHW_ADDRESS_ENABLED)
                rs485_RXADDRESS1_REG = rs485_backup.rx_addr1;
                rs485_RXADDRESS2_REG = rs485_backup.rx_addr2;
            #endif /* End rs485_RXHW_ADDRESS_ENABLED */
        #endif  /* End (rs485_RX_ENABLED) || (rs485_HD_ENABLED) */

        #if(rs485_TX_ENABLED)
            #if(rs485_TXCLKGEN_DP)
                rs485_TXBITCLKGEN_CTR_REG = rs485_backup.tx_clk_ctr;
                rs485_TXBITCLKTX_COMPLETE_REG = rs485_backup.tx_clk_compl;
            #else
                rs485_TXBITCTR_PERIOD_REG = rs485_backup.tx_period;
            #endif /*End rs485_TXCLKGEN_DP */
            rs485_TXSTATUS_MASK_REG = rs485_backup.tx_mask;
        #endif /*End rs485_TX_ENABLED */

    /* PSoC3 ES3 or later, PSoC5 ES2 or later*/
    #elif (rs485_PSOC3_ES3 || rs485_PSOC5_ES2)

        #if(rs485_CONTROL_REG_REMOVED == 0u)
            rs485_CONTROL_REG = rs485_backup.cr;
        #endif /* End rs485_CONTROL_REG_REMOVED */

    #endif  /* End PSOC3_ES3 || PSOC5_ES2 */
}


/*******************************************************************************
* Function Name: rs485_Sleep
********************************************************************************
*
* Summary:
*  Stops and saves the user configuration. Should be called 
*  just prior to entering sleep.
*  
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs485_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs485_Sleep(void)
{

    #if(rs485_RX_ENABLED || rs485_HD_ENABLED)
        if((rs485_RXSTATUS_ACTL_REG  & rs485_INT_ENABLE) != 0u) 
        {
            rs485_backup.enableState = 1u;
        }
        else
        {
            rs485_backup.enableState = 0u;
        }
    #else
        if((rs485_TXSTATUS_ACTL_REG  & rs485_INT_ENABLE) !=0u)
        {
            rs485_backup.enableState = 1u;
        }
        else
        {
            rs485_backup.enableState = 0u;
        }
    #endif /* End rs485_RX_ENABLED || rs485_HD_ENABLED*/

    rs485_Stop();
    rs485_SaveConfig();
}


/*******************************************************************************
* Function Name: rs485_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called
*  just after awaking from sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  rs485_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void rs485_Wakeup(void)
{
    rs485_RestoreConfig();
    #if( (rs485_RX_ENABLED) || (rs485_HD_ENABLED) )
        rs485_ClearRxBuffer();
    #endif /* End (rs485_RX_ENABLED) || (rs485_HD_ENABLED) */
    #if(rs485_TX_ENABLED || rs485_HD_ENABLED)
        rs485_ClearTxBuffer();
    #endif /* End rs485_TX_ENABLED || rs485_HD_ENABLED */

    if(rs485_backup.enableState != 0u)
    {
        rs485_Enable();
    } 
}


/* [] END OF FILE */
