/*******************************************************************************
* File Name: sdcard_spi.c
* Version 2.10
*
* Description:
*  This file provides all API functionality of the SPI Master component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "CyLib.h"
#include "sdcard_spi.h"

#if(sdcard_spi_InternalClockUsed)
    #include "sdcard_spi_IntClock.h"   
#endif /* sdcard_spi_InternalClockUsed */

#if (sdcard_spi_TXBUFFERSIZE > 4u)

    volatile uint8 sdcard_spi_TXBUFFER[sdcard_spi_TXBUFFERSIZE] = {0u};
    volatile uint8 sdcard_spi_txBufferRead = 0u;
    volatile uint8 sdcard_spi_txBufferWrite = 0u;
    volatile uint8 sdcard_spi_txBufferFull = 0u;
    
#endif /* sdcard_spi_TXBUFFERSIZE > 4u */

#if (sdcard_spi_RXBUFFERSIZE > 4u)

    volatile uint8 sdcard_spi_RXBUFFER[sdcard_spi_RXBUFFERSIZE] = {0u};
    volatile uint8 sdcard_spi_rxBufferRead = 0u;
    volatile uint8 sdcard_spi_rxBufferWrite = 0u;
    volatile uint8 sdcard_spi_rxBufferFull = 0u;
    
#endif /* sdcard_spi_RXBUFFERSIZE > 4u */

uint8 sdcard_spi_initVar = 0u;

extern volatile uint8 sdcard_spi_swStatusTx;
extern volatile uint8 sdcard_spi_swStatusRx;


/*******************************************************************************
* Function Name: sdcard_spi_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default SPIM configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  When this function is called it initializes all of the necessary parameters
*  for execution. i.e. setting the initial interrupt mask, configuring the 
*  interrupt service routine, configuring the bit-counter parameters and 
*  clearing the FIFO and Status Register.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_Init(void)
{    
    /* Initialize the Bit counter */
    sdcard_spi_COUNTER_PERIOD_REG = sdcard_spi_BITCTR_INIT;
    
    /* ISR initialization  */  
    #if(sdcard_spi_InternalTxInterruptEnabled)
    
        CyIntDisable(sdcard_spi_TX_ISR_NUMBER);

        /* Set the ISR to point to the sdcard_spi_isr Interrupt. */
        CyIntSetVector(sdcard_spi_TX_ISR_NUMBER, sdcard_spi_TX_ISR);

        /* Set the priority. */
        CyIntSetPriority(sdcard_spi_TX_ISR_NUMBER, sdcard_spi_TX_ISR_PRIORITY);
        
    #endif /* sdcard_spi_InternalTxInterruptEnabled */                                
    
    #if(sdcard_spi_InternalRxInterruptEnabled)
    
        CyIntDisable(sdcard_spi_RX_ISR_NUMBER);

        /* Set the ISR to point to the sdcard_spi_isr Interrupt. */
        CyIntSetVector(sdcard_spi_RX_ISR_NUMBER, sdcard_spi_RX_ISR);

        /* Set the priority. */
        CyIntSetPriority(sdcard_spi_RX_ISR_NUMBER, sdcard_spi_RX_ISR_PRIORITY);
        
    #endif /* sdcard_spi_InternalRxInterruptEnabled */
    
    /* Clear any stray data from the RX and TX FIFO */    
	sdcard_spi_ClearFIFO();
	
	#if(sdcard_spi_RXBUFFERSIZE > 4u)
    
        sdcard_spi_rxBufferRead = 0u;
        sdcard_spi_rxBufferWrite = 0u;

    #endif /* sdcard_spi_RXBUFFERSIZE > 4u */
	
    #if(sdcard_spi_TXBUFFERSIZE > 4u)
    
        sdcard_spi_txBufferRead = 0u;
        sdcard_spi_txBufferWrite = 0u;

    #endif /* sdcard_spi_TXBUFFERSIZE > 4u */
    
    (void) sdcard_spi_ReadTxStatus(); /* Clear any pending status bits */
    (void) sdcard_spi_ReadRxStatus(); /* Clear any pending status bits */
	
	/* Configure the Initial interrupt mask */
    #if (sdcard_spi_TXBUFFERSIZE > 4u)
        sdcard_spi_TX_STATUS_MASK_REG  = sdcard_spi_TX_INIT_INTERRUPTS_MASK & 
                                                ~sdcard_spi_STS_TX_FIFO_NOT_FULL;                    
	#else /* sdcard_spi_TXBUFFERSIZE < 4u */    
        sdcard_spi_TX_STATUS_MASK_REG  = sdcard_spi_TX_INIT_INTERRUPTS_MASK;       
	#endif /* sdcard_spi_TXBUFFERSIZE > 4u */
    
    sdcard_spi_RX_STATUS_MASK_REG  = sdcard_spi_RX_INIT_INTERRUPTS_MASK; 
}
  
    
/*******************************************************************************
* Function Name: sdcard_spi_Enable
********************************************************************************
*
* Summary:
*  Enable SPIM component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void sdcard_spi_Enable(void) 
{             
    uint8 enableInterrupts = 0u;    
    
    enableInterrupts = CyEnterCriticalSection();
    
    sdcard_spi_COUNTER_CONTROL_REG |= sdcard_spi_CNTR_ENABLE;
    sdcard_spi_TX_STATUS_ACTL_REG |= sdcard_spi_INT_ENABLE;
    sdcard_spi_RX_STATUS_ACTL_REG |= sdcard_spi_INT_ENABLE;
    
    CyExitCriticalSection(enableInterrupts);
    
    #if(sdcard_spi_InternalClockUsed)    
        sdcard_spi_IntClock_Enable();        
    #endif /* sdcard_spi_InternalClockUsed */
    
    #if(sdcard_spi_InternalTxInterruptEnabled)    
        CyIntEnable(sdcard_spi_TX_ISR_NUMBER);        
    #endif /* sdcard_spi_InternalTxInterruptEnabled */
    
    #if(sdcard_spi_InternalRxInterruptEnabled)    
        CyIntEnable(sdcard_spi_RX_ISR_NUMBER);        
    #endif /* sdcard_spi_InternalRxInterruptEnabled */
}


/*******************************************************************************
* Function Name: sdcard_spi_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  sdcard_spi_initVar - used to check initial configuration, modified on
*  first function call.
*
* Theory:
*  Enable the clock input to enable operation.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_Start(void)
{       
    if(sdcard_spi_initVar == 0u)
    {               
        sdcard_spi_Init();
        sdcard_spi_initVar = 1u; 
    }                       
        
    sdcard_spi_Enable();        
}


/*******************************************************************************
* Function Name: sdcard_spi_Stop
********************************************************************************
*
* Summary:
*  Disable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the clock input to enable operation.
*
*******************************************************************************/
void sdcard_spi_Stop(void) 
{
    uint8 enableInterrupts = 0u;    
    
    enableInterrupts = CyEnterCriticalSection();
    
    sdcard_spi_TX_STATUS_ACTL_REG &= ~sdcard_spi_INT_ENABLE;
    sdcard_spi_RX_STATUS_ACTL_REG &= ~sdcard_spi_INT_ENABLE;
    
    CyExitCriticalSection(enableInterrupts);
    
    #if(sdcard_spi_InternalClockUsed)    
        sdcard_spi_IntClock_Disable();        
    #endif /* sdcard_spi_InternalClockUsed */
    
    #if(sdcard_spi_InternalTxInterruptEnabled)    
        CyIntDisable(sdcard_spi_TX_ISR_NUMBER);        
    #endif /* sdcard_spi_InternalTxInterruptEnabled */
    
    #if(sdcard_spi_InternalRxInterruptEnabled)    
        CyIntDisable(sdcard_spi_RX_ISR_NUMBER);        
    #endif /* sdcard_spi_InternalRxInterruptEnabled */
}


/*******************************************************************************
* Function Name: sdcard_spi_EnableTxInt
********************************************************************************
*
* Summary:
*  Enable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void sdcard_spi_EnableTxInt(void) 
{       
    #if(sdcard_spi_InternalTxInterruptEnabled)    
        CyIntEnable(sdcard_spi_TX_ISR_NUMBER);
    #endif /* sdcard_spi_InternalTxInterruptEnabled */     
}


/*******************************************************************************
* Function Name: sdcard_spi_EnableRxInt
********************************************************************************
*
* Summary:
*  Enable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void sdcard_spi_EnableRxInt(void) 
{       
    #if(sdcard_spi_InternalRxInterruptEnabled)            
        CyIntEnable(sdcard_spi_RX_ISR_NUMBER);
    #endif /* sdcard_spi_InternalRxInterruptEnabled */     
}


/*******************************************************************************
* Function Name: sdcard_spi_DisableTxInt
********************************************************************************
*
* Summary:
*  Disable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void sdcard_spi_DisableTxInt(void) 
{
    #if(sdcard_spi_InternalTxInterruptEnabled)    
        CyIntDisable(sdcard_spi_TX_ISR_NUMBER);        
    #endif /* sdcard_spi_InternalTxInterruptEnabled */
}


/*******************************************************************************
* Function Name: sdcard_spi_DisableRxInt
********************************************************************************
*
* Summary:
*  Disable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void sdcard_spi_DisableRxInt(void) 
{
    #if(sdcard_spi_InternalRxInterruptEnabled)    
        CyIntDisable(sdcard_spi_RX_ISR_NUMBER);        
    #endif /* sdcard_spi_InternalRxInterruptEnabled */
}


/*******************************************************************************
* Function Name: sdcard_spi_SetTxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the 
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void sdcard_spi_SetTxInterruptMode(uint8 intSrc) 
{
    sdcard_spi_TX_STATUS_MASK_REG  = intSrc;    
}


/*******************************************************************************
* Function Name: sdcard_spi_SetRxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the 
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void sdcard_spi_SetRxInterruptMode(uint8 intSrc) 
{
    sdcard_spi_RX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: sdcard_spi_ReadTxStatus
********************************************************************************
*
* Summary:
*  Read the Tx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Tx status register.
*
* Global variables:
*  sdcard_spi_swStatusTx - used to store in software status register, 
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Tx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 sdcard_spi_ReadTxStatus(void)
{
    uint8 tmpStatus = 0u;
        
    #if (sdcard_spi_TXBUFFERSIZE > 4u)
    
        sdcard_spi_DisableTxInt();
        
        tmpStatus = sdcard_spi_GET_STATUS_TX(sdcard_spi_swStatusTx);                    
        
        sdcard_spi_swStatusTx = 0u;        
        
        /* Enable Interrupts */
        sdcard_spi_EnableTxInt();
        
    #else /* (sdcard_spi_TXBUFFERSIZE < 4u) */
    
        tmpStatus = sdcard_spi_TX_STATUS_REG;
        
    #endif /* (sdcard_spi_TXBUFFERSIZE > 4u) */
    
    return(tmpStatus);
}


/*******************************************************************************
* Function Name: sdcard_spi_ReadRxStatus
********************************************************************************
*
* Summary:
*  Read the Rx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Rx status register.
*
* Global variables:
*  sdcard_spi_swStatusRx - used to store in software Rx status register, 
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Rx status register for error 
*  detection and flow control.
*
* Side Effects:
*  Clear Rx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 sdcard_spi_ReadRxStatus(void)
{
    uint8 tmpStatus = 0u;
        
    #if (sdcard_spi_RXBUFFERSIZE > 4u)
    
        sdcard_spi_DisableRxInt();
        
        tmpStatus = sdcard_spi_GET_STATUS_RX(sdcard_spi_swStatusRx);
               
        sdcard_spi_swStatusRx = 0u;
        
        /* Enable Interrupts */
        sdcard_spi_EnableRxInt();
        
    #else /* (sdcard_spi_RXBUFFERSIZE < 4u) */
    
        tmpStatus = sdcard_spi_RX_STATUS_REG;
        
    #endif /* (sdcard_spi_RXBUFFERSIZE > 4u) */
    
    return(tmpStatus);
}


/*******************************************************************************
* Function Name: sdcard_spi_WriteTxData
********************************************************************************
*
* Summary:
*  Write a byte of data to be sent across the SPI.
*
* Parameters:
*  txDataByte: The data value to send across the SPI.
*
* Return:
*  None.
*
* Global variables:
*  sdcard_spi_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call if TX Software Buffer is used.
*  sdcard_spi_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*  sdcard_spi_TXBUFFER[sdcard_spi_TXBUFFERSIZE] - used to store
*  data to sending, modified every function call if TX Software Buffer is used.
*
* Theory:
*  Allows the user to transmit any byte of data in a single transfer.
*
* Side Effects:
*  If this function is called again before the previous byte is finished then
*  the next byte will be appended to the transfer with no time between
*  the byte transfers. Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_WriteTxData(uint8 txData)
{    
    #if(sdcard_spi_TXBUFFERSIZE > 4u)

        int16 tmpTxBufferRead = 0u;
                
        /* Block if buffer is full, so we don't overwrite. */
        do
        {
            tmpTxBufferRead = sdcard_spi_txBufferRead - 1u;
            if (tmpTxBufferRead < 0u)
            {
                tmpTxBufferRead = sdcard_spi_TXBUFFERSIZE - 1u;        
            }    
        } while(tmpTxBufferRead == sdcard_spi_txBufferWrite);               
                   
        /* Disable Interrupt to protect variables that could change on interrupt. */
        sdcard_spi_DisableTxInt();
        
        sdcard_spi_swStatusTx = sdcard_spi_GET_STATUS_TX(sdcard_spi_swStatusTx);
        
        if((sdcard_spi_txBufferRead == sdcard_spi_txBufferWrite) &&
            ((sdcard_spi_swStatusTx & sdcard_spi_STS_TX_FIFO_NOT_FULL) != 0u))
        {
            /* Add directly to the FIFO. */
            CY_SET_REG8(sdcard_spi_TXDATA_PTR, txData);
        }
        else
        {
            /* Add to the software buffer. */
            sdcard_spi_txBufferWrite++;
            if(sdcard_spi_txBufferWrite >= sdcard_spi_TXBUFFERSIZE)
            {
                sdcard_spi_txBufferWrite = 0u;
            }   
                      
            if(sdcard_spi_txBufferWrite == sdcard_spi_txBufferRead)
            {
                sdcard_spi_txBufferRead++;
                if(sdcard_spi_txBufferRead >= sdcard_spi_RXBUFFERSIZE)
                {
                    sdcard_spi_txBufferRead = 0u;
                }
                sdcard_spi_txBufferFull = 1u;
            }
            
            sdcard_spi_TXBUFFER[sdcard_spi_txBufferWrite] = txData;
            
            sdcard_spi_TX_STATUS_MASK_REG |= sdcard_spi_STS_TX_FIFO_NOT_FULL;            
        }                         
        
        /* Enable Interrupt. */
        sdcard_spi_EnableTxInt();                        

    #else /* sdcard_spi_TXBUFFERSIZE <= 4u */

        /* Block while FIFO is full */
        while((sdcard_spi_TX_STATUS_REG & sdcard_spi_STS_TX_FIFO_NOT_FULL) == 0u);
        
        /* Then write the byte */
        CY_SET_REG8(sdcard_spi_TXDATA_PTR, txData);

    #endif /* sdcard_spi_TXBUFFERSIZE > 4u */
}


/*******************************************************************************
* Function Name: sdcard_spi_ReadRxData
********************************************************************************
*
* Summary:
*  Read the next byte of data received across the SPI.
*
* Parameters:
*  None.
*
* Return:
*  The next byte of data read from the FIFO.
*
* Global variables:
*  sdcard_spi_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  sdcard_spi_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function
*  call if RX Software Buffer is used.
*  sdcard_spi_RXBUFFER[sdcard_spi_RXBUFFERSIZE] - used to store
*  received data.
*
* Theory:
*  Allows the user to read a byte of data received.
*
* Side Effects:
*  Will return invalid data if the FIFO is empty. The user should Call 
*  GetRxBufferSize() and if it returns a non-zero value then it is safe to call 
*  ReadByte() function.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 sdcard_spi_ReadRxData(void)
{
    uint8 rxData = 0u;

    #if(sdcard_spi_RXBUFFERSIZE > 4u)
    
        /* Disable Interrupt to protect variables that could change on interrupt. */
        sdcard_spi_DisableRxInt();
        
        if(sdcard_spi_rxBufferRead != sdcard_spi_rxBufferWrite)
        {      
            if(sdcard_spi_rxBufferFull == 0u)
            {
                sdcard_spi_rxBufferRead++;
                if(sdcard_spi_rxBufferRead >= sdcard_spi_RXBUFFERSIZE)
                {
                    sdcard_spi_rxBufferRead = 0u;
                }
            }
            else
            {
                sdcard_spi_rxBufferFull = 0u;
            }
        }    
        
        rxData = sdcard_spi_RXBUFFER[sdcard_spi_rxBufferRead];
                           
        /* Enable Interrupt. */
        sdcard_spi_EnableRxInt();
    
    #else /* sdcard_spi_RXBUFFERSIZE <= 4u */
    
        rxData = CY_GET_REG8(sdcard_spi_RXDATA_PTR);
    
    #endif /* sdcard_spi_RXBUFFERSIZE > 4u */

	return (rxData);
    
}


/*******************************************************************************
* Function Name: sdcard_spi_GetRxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the RX buffer.
*  If RX Software Buffer not used then function return 0 if FIFO empty or 1 if 
*  FIFO not empty. In another case function return size of RX Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the RX buffer.
*
* Global variables:
*  sdcard_spi_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  sdcard_spi_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8 sdcard_spi_GetRxBufferSize(void) 
{
    uint8 size = 0u;

    #if(sdcard_spi_RXBUFFERSIZE > 4u)
    
        /* Disable Interrupt to protect variables that could change on interrupt. */
        sdcard_spi_DisableRxInt();
    
        if(sdcard_spi_rxBufferRead == sdcard_spi_rxBufferWrite)
        {
            size = 0u; /* No data in RX buffer */
        }
        else if(sdcard_spi_rxBufferRead < sdcard_spi_rxBufferWrite)
        {
            size = (sdcard_spi_rxBufferWrite - sdcard_spi_rxBufferRead);
        }
        else
        {
            size = (sdcard_spi_RXBUFFERSIZE - sdcard_spi_rxBufferRead) + sdcard_spi_rxBufferWrite;
        }
    
        /* Enable interrupt. */
        sdcard_spi_EnableRxInt();
    
    #else /* sdcard_spi_RXBUFFERSIZE <= 4u */
    
        /* We can only know if there is data in the fifo. */
        size = ((sdcard_spi_RX_STATUS_REG & sdcard_spi_STS_RX_FIFO_NOT_EMPTY) == 
                 sdcard_spi_STS_RX_FIFO_NOT_EMPTY) ? 1u : 0u;
    
    #endif /* sdcard_spi_RXBUFFERSIZE < 4u */

    return (size);
}


/*******************************************************************************
* Function Name: sdcard_spi_GetTxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the TX buffer.
*  If TX Software Buffer not used then function return 0 - if FIFO empty, 1 - if 
*  FIFO not full, 4 - if FIFO full. In another case function return size of TX
*  Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the TX buffer.
*
* Global variables:
*  sdcard_spi_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  sdcard_spi_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8  sdcard_spi_GetTxBufferSize(void) 
{
    uint8 size = 0u;

    #if(sdcard_spi_TXBUFFERSIZE > 4u)
    
        /* Disable Interrupt to protect variables that could change on interrupt. */
        sdcard_spi_DisableTxInt();
    
        if(sdcard_spi_txBufferRead == sdcard_spi_txBufferWrite)
        {
            size = 0u;
        }
        else if(sdcard_spi_txBufferRead < sdcard_spi_txBufferWrite)
        {
            size = (sdcard_spi_txBufferWrite - sdcard_spi_txBufferRead);
        }
        else
        {
            size = (sdcard_spi_TXBUFFERSIZE - sdcard_spi_txBufferRead) + sdcard_spi_txBufferWrite;
        }
    
        /* Enable Interrupt. */
        sdcard_spi_EnableTxInt();
    
    #else /* sdcard_spi_TXBUFFERSIZE <= 4u */
    
        size = sdcard_spi_TX_STATUS_REG;
    
        /* Is the fifo is full. */
        if((size & sdcard_spi_STS_TX_FIFO_EMPTY) == sdcard_spi_STS_TX_FIFO_EMPTY)
        {
            size = 0u;
        }
        else if((size & sdcard_spi_STS_TX_FIFO_NOT_FULL) == sdcard_spi_STS_TX_FIFO_NOT_FULL)
        {
            size = 1u;
        }
        else
        {
            /* We only know there is data in the fifo. */
            size = 4u;
        }
    
    #endif /* sdcard_spi_TXBUFFERSIZE > 4u */

    return (size);
}


/*******************************************************************************
* Function Name: sdcard_spi_ClearRxBuffer
********************************************************************************
*
* Summary:
*  Clear the RX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  sdcard_spi_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer, modified every function 
*  call - resets to zero.
*  sdcard_spi_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to 
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any received data not read from the RAM buffer will be lost when overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_ClearRxBuffer(void)
{
	/* Clear Hardware RX FIFO */
    while((!(sdcard_spi_RX_STATUS_REG & sdcard_spi_STS_RX_FIFO_NOT_EMPTY)) == 0u)
    {
        CY_GET_REG8(sdcard_spi_RXDATA_PTR);
    }
	
    #if(sdcard_spi_RXBUFFERSIZE > 4u)
    
        /* Disable interrupt to protect variables that could change on interrupt. */        
        sdcard_spi_DisableRxInt();
    
        sdcard_spi_rxBufferRead = 0u;
        sdcard_spi_rxBufferWrite = 0u;
    
        /* Enable Rx interrupt. */
        sdcard_spi_EnableRxInt();
        
    #endif /* sdcard_spi_RXBUFFERSIZE > 4u */
}


/*******************************************************************************
* Function Name: sdcard_spi_ClearTxBuffer
********************************************************************************
*
* Summary:
*  Clear the TX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  sdcard_spi_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call - resets to zero.
*  sdcard_spi_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to 
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any data not yet transmitted from the RAM buffer will be lost when 
*  overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_ClearTxBuffer(void)
{
    uint8 enableInterrupts = 0u;
    
    /* Clear Hardware TX FIFO */       
    enableInterrupts = CyEnterCriticalSection();
    
    #if(sdcard_spi_DataWidth <= 8u)
    
        /* Clear TX FIFO */
        sdcard_spi_AUX_CONTROL_DP0_REG |= sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP0_REG &= ~sdcard_spi_FIFO_CLR;
    
    #else
    
        /* Clear TX FIFO */
        sdcard_spi_AUX_CONTROL_DP0_REG |= sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP0_REG &= ~sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP1_REG |= sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP1_REG &= ~sdcard_spi_FIFO_CLR;
        
    #endif /* sdcard_spi_DataWidth > 8u */
    
    CyExitCriticalSection(enableInterrupts);
	
    #if(sdcard_spi_TXBUFFERSIZE > 4u)
    
        /* Disable Interrupt to protect variables that could change on interrupt. */
        sdcard_spi_DisableTxInt();
    
        sdcard_spi_txBufferRead = 0u;
        sdcard_spi_txBufferWrite = 0u;
    
        /* If Buffer is empty then disable TX FIFO status interrupt */
        sdcard_spi_TX_STATUS_MASK_REG &= ~sdcard_spi_STS_TX_FIFO_NOT_FULL;

        /* Enable Interrupt. */
        sdcard_spi_EnableTxInt();
    
    #endif /* sdcard_spi_TXBUFFERSIZE > 4u */
}


#if (sdcard_spi_BidirectionalMode == 1u)

    /*******************************************************************************
    * Function Name: sdcard_spi_TxEnable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to transmit.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void sdcard_spi_TxEnable(void) 
    {
        sdcard_spi_CONTROL_REG |= sdcard_spi_CTRL_TX_SIGNAL_EN;
    }
    
    
    /*******************************************************************************
    * Function Name: sdcard_spi_TxDisable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to receive.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void sdcard_spi_TxDisable(void) 
    {
        sdcard_spi_CONTROL_REG &= ~sdcard_spi_CTRL_TX_SIGNAL_EN;
    }
    
#endif /* sdcard_spi_BidirectionalMode == 1u */


/*******************************************************************************
* Function Name: sdcard_spi_PutArray
********************************************************************************                       
*
* Summary:
*  Write available data from ROM/RAM to the TX buffer while space is available 
*  in the TX buffer. Keep trying until all data is passed to the TX buffer.
*
* Parameters:
*  *buffer: Pointer to the location in RAM containing the data to send
*  byteCount: The number of bytes to move to the transmit buffer.
*
* Return:
*  None.
*
* Side Effects:
*  Will stay in this routine until all data has been sent.  May get locked in
*  this loop if data is not being initiated by the master if there is not
*  enough room in the TX FIFO.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_PutArray(uint8 *buffer, uint8 byteCount)
{
    while(byteCount > 0u)
    {
        sdcard_spi_WriteTxData(*buffer++);
        byteCount--;
    }
}


/*******************************************************************************
* Function Name: sdcard_spi_ClearFIFO
********************************************************************************
*
* Summary:
*  Clear the RX and TX FIFO's of all data for a fresh start.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
void sdcard_spi_ClearFIFO(void) 
{
    uint8 enableInterrupts = 0u;
    
    while((!(sdcard_spi_RX_STATUS_REG & sdcard_spi_STS_RX_FIFO_NOT_EMPTY)) == 0u)
    {
        CY_GET_REG8(sdcard_spi_RXDATA_PTR);
    }
    
    enableInterrupts = CyEnterCriticalSection();
    
    #if(sdcard_spi_DataWidth <= 8u)
    
        /* Clear TX FIFO */
        sdcard_spi_AUX_CONTROL_DP0_REG |= sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP0_REG &= ~sdcard_spi_FIFO_CLR;
    
    #else
    
        /* Clear TX FIFO */
        sdcard_spi_AUX_CONTROL_DP0_REG |= sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP0_REG &= ~sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP1_REG |= sdcard_spi_FIFO_CLR;
        sdcard_spi_AUX_CONTROL_DP1_REG &= ~sdcard_spi_FIFO_CLR;
        
    #endif /* sdcard_spi_DataWidth > 8u */
    
    CyExitCriticalSection(enableInterrupts);
}


/* Following functions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects.
*/

/*******************************************************************************
* Function Name: sdcard_spi_EnableInt
********************************************************************************
*
* Summary:
*  Enable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void sdcard_spi_EnableInt(void) 
{       
    #if(sdcard_spi_InternalTxInterruptEnabled)    
        CyIntEnable(sdcard_spi_TX_ISR_NUMBER);
    #endif /* sdcard_spi_InternalTxInterruptEnabled */                                
    
    #if(sdcard_spi_InternalRxInterruptEnabled)           
        CyIntEnable(sdcard_spi_RX_ISR_NUMBER);
    #endif /* sdcard_spi_InternalRxInterruptEnabled */     
}


/*******************************************************************************
* Function Name: sdcard_spi_DisableInt
********************************************************************************
*
* Summary:
*  Disable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void sdcard_spi_DisableInt(void) 
{
    #if(sdcard_spi_InternalTxInterruptEnabled)    
        CyIntDisable(sdcard_spi_TX_ISR_NUMBER);
    #endif /* sdcard_spi_InternalTxInterruptEnabled */
    
    #if(sdcard_spi_InternalRxInterruptEnabled)           
        CyIntDisable(sdcard_spi_RX_ISR_NUMBER);
    #endif /* sdcard_spi_InternalRxInterruptEnabled */
}


/*******************************************************************************
* Function Name: sdcard_spi_SetInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the 
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void sdcard_spi_SetInterruptMode(uint8 intSrc) 
{
    sdcard_spi_TX_STATUS_MASK_REG  = intSrc & ~(1u << sdcard_spi_STS_SPI_IDLE_SHIFT);
    sdcard_spi_RX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: sdcard_spi_ReadStatus
********************************************************************************
*
* Summary:
*  Read the status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the status register.
*
* Global variables:
*  sdcard_spi_swStatus - used to store in software status register, 
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the status register for error detection
*  and flow control.
*
* Side Effects:
*  Clear status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 sdcard_spi_ReadStatus(void)
{
    uint8 tmpStatus;
        
    #if ((sdcard_spi_TXBUFFERSIZE > 4u) || (sdcard_spi_RXBUFFERSIZE > 4u))
    
        sdcard_spi_DisableInt();
        
        tmpStatus = (sdcard_spi_GET_STATUS_TX(sdcard_spi_swStatusTx) & 
                      ~(1u << sdcard_spi_STS_SPI_IDLE_SHIFT)) | 
                      sdcard_spi_GET_STATUS_RX(sdcard_spi_swStatusRx);
        
        sdcard_spi_swStatusTx = 0u;
        sdcard_spi_swStatusRx = 0u;
        
        /* Enable Interrupts */
        sdcard_spi_EnableInt();
        
    #else /* (sdcard_spi_TXBUFFERSIZE < 4u) && (sdcard_spi_RXBUFFERSIZE < 4u) */
    
        tmpStatus = (sdcard_spi_TX_STATUS_REG & ~(1u << sdcard_spi_STS_SPI_IDLE_SHIFT)) |
                     sdcard_spi_RX_STATUS_REG;
        
    #endif /* (sdcard_spi_TXBUFFERSIZE > 4u) || (sdcard_spi_RXBUFFERSIZE > 4u) */
    
    return(tmpStatus);
}


/* [] END OF FILE */
