/*******************************************************************************
* File Name: sdcard_spi_INT.c
* Version 2.10
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "sdcard_spi.h"

#if (sdcard_spi_RXBUFFERSIZE > 4u)

    extern volatile uint8 sdcard_spi_RXBUFFER[];
    extern volatile uint8 sdcard_spi_rxBufferRead;
    extern volatile uint8 sdcard_spi_rxBufferWrite;
    extern volatile uint8 sdcard_spi_rxBufferFull;
    
#endif /* sdcard_spi_RXBUFFERSIZE > 4u */

#if (sdcard_spi_TXBUFFERSIZE > 4u)

    extern volatile uint8 sdcard_spi_TXBUFFER[];
    extern volatile uint8 sdcard_spi_txBufferRead;
    extern volatile uint8 sdcard_spi_txBufferWrite;
    extern volatile uint8 sdcard_spi_txBufferFull;

#endif /* sdcard_spi_TXBUFFERSIZE > 4u */

volatile uint8 sdcard_spi_swStatusTx = 0u;
volatile uint8 sdcard_spi_swStatusRx = 0u;

/* User code required at start of ISR */
/* `#START sdcard_spi_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: sdcard_spi_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  sdcard_spi_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  sdcard_spi_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to 
*  sending and FIFO Not Full.
*  sdcard_spi_TXBUFFER[sdcard_spi_TXBUFFERSIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(sdcard_spi_TX_ISR)
{     
    /* User code required at start of ISR */
    /* `#START sdcard_spi_TX_ISR_START` */

    /* `#END` */
    
    #if((sdcard_spi_InternalTxInterruptEnabled) && (sdcard_spi_TXBUFFERSIZE > 4u))
                         
        /* See if TX data buffer is not empty and there is space in TX FIFO */
        while(sdcard_spi_txBufferRead != sdcard_spi_txBufferWrite)
        {
            sdcard_spi_swStatusTx = sdcard_spi_GET_STATUS_TX(sdcard_spi_swStatusTx);
            
            if ((sdcard_spi_swStatusTx & sdcard_spi_STS_TX_FIFO_NOT_FULL) != 0u)
            {            
                if(sdcard_spi_txBufferFull == 0u)
                {
                   sdcard_spi_txBufferRead++;

                    if(sdcard_spi_txBufferRead >= sdcard_spi_TXBUFFERSIZE)
                    {
                        sdcard_spi_txBufferRead = 0u;
                    }
                }
                else
                {
                    sdcard_spi_txBufferFull = 0u;
                }
            
                /* Move data from the Buffer to the FIFO */
                CY_SET_REG8(sdcard_spi_TXDATA_PTR,
                    sdcard_spi_TXBUFFER[sdcard_spi_txBufferRead]);
            }
            else
            {
                break;
            }            
        }
            
        /* Disable Interrupt on TX_fifo_not_empty if BUFFER is empty */
        if(sdcard_spi_txBufferRead == sdcard_spi_txBufferWrite)
        {
            sdcard_spi_TX_STATUS_MASK_REG  &= ~sdcard_spi_STS_TX_FIFO_NOT_FULL; 
        }                       
        
	#endif /* sdcard_spi_InternalTxInterruptEnabled && (sdcard_spi_TXBUFFERSIZE > 4u) */
    
    /* User code required at end of ISR (Optional) */
    /* `#START sdcard_spi_TX_ISR_END` */

    /* `#END` */
    
    #ifdef sdcard_spi_TxInternalInterrupt__ES2_PATCH
        #if(sdcard_spi_PSOC3_ES2 && (sdcard_spi_TxInternalInterrupt__ES2_PATCH))
            sdcard_spi_TX_ISR_PATCH();
        #endif /* End PSOC3_ES2 */
    #endif /* sdcard_spi_TxInternalInterrupt__ES2_PATCH */    
}


/*******************************************************************************
* Function Name: sdcard_spi_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  sdcard_spi_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  sdcard_spi_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  sdcard_spi_RXBUFFER[sdcard_spi_RXBUFFERSIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(sdcard_spi_RX_ISR)
{     
    #if((sdcard_spi_InternalRxInterruptEnabled) && (sdcard_spi_RXBUFFERSIZE > 4u))
        uint8 rxData = 0u; 
    #endif /* sdcard_spi_InternalRxInterruptEnabled */  
    
    /* User code required at start of ISR */
    /* `#START sdcard_spi_RX_ISR_START` */

    /* `#END` */
    
    #if((sdcard_spi_InternalRxInterruptEnabled) && (sdcard_spi_RXBUFFERSIZE > 4u))
         
        sdcard_spi_swStatusRx = sdcard_spi_GET_STATUS_RX(sdcard_spi_swStatusRx);          
        
        /* See if RX data FIFO has some data and if it can be moved to the RX Buffer */
        while((sdcard_spi_swStatusRx & sdcard_spi_STS_RX_FIFO_NOT_EMPTY) == 
                                                                                sdcard_spi_STS_RX_FIFO_NOT_EMPTY)
        {
            rxData = CY_GET_REG8(sdcard_spi_RXDATA_PTR);
            
            /* Set next pointer. */
            sdcard_spi_rxBufferWrite++;
            if(sdcard_spi_rxBufferWrite >= sdcard_spi_RXBUFFERSIZE)
            {
                sdcard_spi_rxBufferWrite = 0u;
            }
            
            if(sdcard_spi_rxBufferWrite == sdcard_spi_rxBufferRead)
            {
                sdcard_spi_rxBufferRead++;
                if(sdcard_spi_rxBufferRead >= sdcard_spi_RXBUFFERSIZE)
                {
                    sdcard_spi_rxBufferRead = 0u;
                }
                sdcard_spi_rxBufferFull = 1u;
            }
            
            /* Move data from the FIFO to the Buffer */
            sdcard_spi_RXBUFFER[sdcard_spi_rxBufferWrite] = rxData;
                
            sdcard_spi_swStatusRx = sdcard_spi_GET_STATUS_RX(sdcard_spi_swStatusRx);
        }                    
        
	#endif /* sdcard_spi_InternalRxInterruptEnabled  && (sdcard_spi_RXBUFFERSIZE > 4u) */        
    
    /* User code required at end of ISR (Optional) */
    /* `#START sdcard_spi_RX_ISR_END` */

    /* `#END` */
    
    #ifdef sdcard_spi_RxInternalInterrupt__ES2_PATCH
        #if(sdcard_spi_PSOC3_ES2 && (sdcard_spi_RxInternalInterrupt__ES2_PATCH))
            sdcard_spi_RX_ISR_PATCH();
        #endif /* End PSOC3_ES2 */  
    #endif /* sdcard_spi_RxInternalInterrupt__ES2_PATCH */       
}

/* [] END OF FILE */
