/*******************************************************************************
* File Name: sdcard_spi_PM.c
* Version 2.10
*
* Description:
*  This file contains the setup, control and status commands to support 
*  component operations in low power mode.  
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "sdcard_spi.h"

static sdcard_spi_BACKUP_STRUCT sdcard_spi_backup = {
                                        0u,
                                        sdcard_spi_BITCTR_INIT,
                                        #if(sdcard_spi_PSOC3_ES2 || sdcard_spi_PSOC5_ES1)
                                            sdcard_spi_TX_INIT_INTERRUPTS_MASK,
                                            sdcard_spi_RX_INIT_INTERRUPTS_MASK
                                        #endif /* sdcard_spi_PSOC3_ES2 || sdcard_spi_PSOC5_ES1 */
                                        };

#if (sdcard_spi_TXBUFFERSIZE > 4u)

    extern volatile uint8 sdcard_spi_txBufferRead;
    extern volatile uint8 sdcard_spi_txBufferWrite;
    
#endif /* sdcard_spi_TXBUFFERSIZE > 4u */

#if (sdcard_spi_RXBUFFERSIZE > 4u)

    extern volatile uint8 sdcard_spi_rxBufferRead;
    extern volatile uint8 sdcard_spi_rxBufferWrite;
    
#endif /* sdcard_spi_RXBUFFERSIZE > 4u */


/*******************************************************************************
* Function Name: sdcard_spi_SaveConfig
********************************************************************************
*
* Summary:
*  Saves SPIM configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
* 
* Global Variables:
*  sdcard_spi_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_SaveConfig(void)
{
    /* Store Status Mask registers */
    #if (sdcard_spi_PSOC3_ES2 || sdcard_spi_PSOC5_ES1)     
    
       sdcard_spi_backup.saveSrTxIntMask = sdcard_spi_TX_STATUS_MASK_REG;
       sdcard_spi_backup.saveSrRxIntMask = sdcard_spi_RX_STATUS_MASK_REG;
       sdcard_spi_backup.cntrPeriod = sdcard_spi_COUNTER_PERIOD_REG;
       
    #endif /* (sdcard_spi_PSOC3_ES2 || sdcard_spi_PSOC5_ES1) */
}


/*******************************************************************************
* Function Name: sdcard_spi_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores SPIM configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  sdcard_spi_backup - used when non-retention registers are restored.
*
* Side Effects:
*  If this API is called without first calling SaveConfig then in the following
*  registers will be default values from Customizer: 
*  sdcard_spi_STATUS_MASK_REG and sdcard_spi_COUNTER_PERIOD_REG.
*
*******************************************************************************/
void sdcard_spi_RestoreConfig(void) 
{
    /* Restore the data, saved by SaveConfig() function */
    #if (sdcard_spi_PSOC3_ES2 || sdcard_spi_PSOC5_ES1)
    
        sdcard_spi_TX_STATUS_MASK_REG = sdcard_spi_backup.saveSrTxIntMask;
        sdcard_spi_RX_STATUS_MASK_REG = sdcard_spi_backup.saveSrRxIntMask;
        sdcard_spi_COUNTER_PERIOD_REG = sdcard_spi_backup.cntrPeriod;
        
    #endif /* (sdcard_spi_PSOC3_ES2 || sdcard_spi_PSOC5_ES1) */
}


/*******************************************************************************
* Function Name: sdcard_spi_Sleep
********************************************************************************
*
* Summary:
*  Prepare SPIM Component goes to sleep.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  sdcard_spi_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_Sleep(void)
{
    /* Save components enable state */
    if ((sdcard_spi_TX_STATUS_ACTL_REG & sdcard_spi_INT_ENABLE) == sdcard_spi_INT_ENABLE)
    {
        sdcard_spi_backup.enableState = 1u;
    }
    else /* Components block is disabled */
    {
        sdcard_spi_backup.enableState = 0u;
    }

    sdcard_spi_Stop();

    sdcard_spi_SaveConfig();
}


/*******************************************************************************
* Function Name: sdcard_spi_Wakeup
********************************************************************************
*
* Summary:
*  Prepare SPIM Component to wake up.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  sdcard_spi_backup - used when non-retention registers are restored.
*  sdcard_spi_txBufferWrite - modified every function call - resets to 
*  zero.
*  sdcard_spi_txBufferRead - modified every function call - resets to 
*  zero.
*  sdcard_spi_rxBufferWrite - modified every function call - resets to
*  zero.
*  sdcard_spi_rxBufferRead - modified every function call - resets to
*  zero. 
*
* Reentrant:
*  No.
*
*******************************************************************************/
void sdcard_spi_Wakeup(void)
{        
    sdcard_spi_RestoreConfig();
         
    #if (sdcard_spi_TXBUFFERSIZE > 4u)
    
        sdcard_spi_txBufferRead = 0u;
        sdcard_spi_txBufferWrite = 0u;
        
    #endif /* sdcard_spi_TXBUFFERSIZE > 4u */
    
    #if (sdcard_spi_RXBUFFERSIZE > 4u)    
    
        sdcard_spi_rxBufferRead = 0u;
        sdcard_spi_rxBufferWrite = 0u;
        
    #endif /* sdcard_spi_RXBUFFERSIZE > 4u */ 
    
    sdcard_spi_ClearFIFO();
    
    /* Restore components block enable state */
    if (sdcard_spi_backup.enableState != 0u)
    {
         /* Components block was enabled */
         sdcard_spi_Enable();
    } /* Do nothing if components block was disabled */
}


/* [] END OF FILE */
