/*******************************************************************************
* File Name: spi.h
* Version 2.10
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_spi_H)
#define CY_SPIM_spi_H

#include "cyfitter.h"

/* PSoC3 ES2 or earlier*/
#define spi_PSOC3_ES2   ((CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_3A) && \
                     			      (CYDEV_CHIP_REVISION_USED <= CYDEV_CHIP_REVISION_3A_ES2))
                     
/* PSoC5 ES1 or earlier */
#define spi_PSOC5_ES1   ((CYDEV_CHIP_MEMBER_USED == CYDEV_CHIP_MEMBER_5A) && \
                     		    	  (CYDEV_CHIP_REVISION_USED <= CYDEV_CHIP_REVISION_5A_ES1))
	
#ifdef spi_TxInternalInterrupt__ES2_PATCH
	#if(spi_PSOC3_ES2 && spi_TxInternalInterrupt__ES2_PATCH)
		#include <intrins.h>
        #define spi_TX_ISR_PATCH() _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();
	#endif /* End PSOC3_ES2 */
#endif /* spi_TxInternalInterrupt__ES2_PATCH */

#ifdef spi_RxInternalInterrupt__ES2_PATCH  
	#if(spi_PSOC3_ES2 && spi_RxInternalInterrupt__ES2_PATCH)
		#include <intrins.h>
        #define spi_RX_ISR_PATCH() _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();
	#endif /* End PSOC3_ES2 */
#endif /* spi_RxInternalInterrupt__ES2_PATCH */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define spi_DataWidth                  (8u)
#define spi_InternalClockUsed          (1u)
#define spi_InternalTxInterruptEnabled (0u)
#define spi_InternalRxInterruptEnabled (0u)
#define spi_ModeUseZero                (1u)
#define spi_BidirectionalMode          (0u)
#define spi_Mode                       (1u)

/* Following definitions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects 
*/
#define spi_DATAWIDHT                (spi_DataWidth)
#define spi_InternalInterruptEnabled (0u)


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct _spi_backupStruct
{
    uint8 enableState;
    uint8 cntrPeriod;
    
    #if(spi_PSOC3_ES2 || spi_PSOC5_ES1) /* PSoC3 ES2 or earlier, PSoC5 ES1 */
    
        uint8 saveSrTxIntMask;
        uint8 saveSrRxIntMask;
        
    #endif /* End spi_PSOC3_ES2 || spi_PSOC5_ES1 */

} spi_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  spi_Init(void);
void  spi_Enable(void) ;
void  spi_Start(void);
void  spi_Stop(void) ;
void  spi_EnableTxInt(void) ;
void  spi_EnableRxInt(void) ;
void  spi_DisableTxInt(void) ;
void  spi_DisableRxInt(void) ;
void  spi_SetTxInterruptMode(uint8 intSource) ;
void  spi_SetRxInterruptMode(uint8 intSource) ;
uint8 spi_ReadTxStatus(void);
uint8 spi_ReadRxStatus(void);
void  spi_WriteTxData(uint8 txData);
uint8 spi_ReadRxData(void);
uint8 spi_GetRxBufferSize(void) ;
uint8 spi_GetTxBufferSize(void) ;
void  spi_ClearRxBuffer(void);
void  spi_ClearTxBuffer(void);
void  spi_ClearFIFO(void) ;
void  spi_PutArray(uint8 *buffer, uint8 byteCount);
void  spi_Sleep(void);
void  spi_Wakeup(void);
void  spi_SaveConfig(void);
void  spi_RestoreConfig(void) ;

#if (spi_BidirectionalMode)

    void  spi_TxEnable(void) ;
    void  spi_TxDisable(void) ;

#endif /* spi_BidirectionalMode == 1u*/

CY_ISR_PROTO(spi_TX_ISR);
CY_ISR_PROTO(spi_RX_ISR);

/* Macros for getting software status of SPIM Statusi Register */
#define spi_GET_STATUS_TX(swTxSts) (uint8)(spi_TX_STATUS_REG | \
                                                       (swTxSts & spi_TX_STS_CLR_ON_RD_BYTES_MASK))
#define spi_GET_STATUS_RX(swRxSts) (uint8)(spi_RX_STATUS_REG | \
                                                       (swRxSts & spi_RX_STS_CLR_ON_RD_BYTES_MASK))                                                  

/* Following definitions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects 
*/
#define spi_WriteByte (spi_WriteTxData)
#define spi_ReadByte  (spi_ReadRxData)
void  spi_SetInterruptMode(uint8 intSource) ;
uint8 spi_ReadStatus(void);
void  spi_EnableInt(void) ;
void  spi_DisableInt(void) ;


/***************************************
*           API Constants
***************************************/

#define spi_TX_ISR_NUMBER     (spi_TxInternalInterrupt__INTC_NUMBER)
#define spi_RX_ISR_NUMBER     (spi_RxInternalInterrupt__INTC_NUMBER)
#define spi_TX_ISR_PRIORITY   (spi_TxInternalInterrupt__INTC_PRIOR_NUM)
#define spi_RX_ISR_PRIORITY   (spi_RxInternalInterrupt__INTC_PRIOR_NUM)


/***************************************
*    Initial Parameter Constants
***************************************/
                                               
#define spi_INT_ON_SPI_DONE    (0u << spi_STS_SPI_DONE_SHIFT)
#define spi_INT_ON_TX_EMPTY    (0u << spi_STS_TX_FIFO_EMPTY_SHIFT)
#define spi_INT_ON_TX_NOT_FULL (0u << spi_STS_TX_FIFO_NOT_FULL_SHIFT)
#define spi_INT_ON_BYTE_COMP   (0u << spi_STS_BYTE_COMPLETE_SHIFT)
#define spi_INT_ON_SPI_IDLE    (0u << spi_STS_SPI_IDLE_SHIFT)

#define spi_TX_INIT_INTERRUPTS_MASK  (spi_INT_ON_SPI_DONE | \
                                            spi_INT_ON_TX_EMPTY | spi_INT_ON_TX_NOT_FULL | \
                                            spi_INT_ON_BYTE_COMP | spi_INT_ON_SPI_IDLE)
            
#define spi_INT_ON_RX_FULL      (0u << spi_STS_RX_FIFO_FULL_SHIFT)
#define spi_INT_ON_RX_NOT_EMPTY (0u << spi_STS_RX_FIFO_NOT_EMPTY_SHIFT)
#define spi_INT_ON_RX_OVER      (0u << spi_STS_RX_FIFO_OVERRUN_SHIFT)

#define spi_RX_INIT_INTERRUPTS_MASK  (spi_INT_ON_RX_FULL | \
                                            spi_INT_ON_RX_NOT_EMPTY | spi_INT_ON_RX_OVER)
                                               
#define spi_BITCTR_INIT           ( (spi_DataWidth << 1u) - 1u)

#define spi_TXBUFFERSIZE          (4u)
#define spi_RXBUFFERSIZE          (4u)

/* Following definitions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects 
*/
#define spi_INIT_INTERRUPTS_MASK  (spi_INT_ON_SPI_DONE | spi_INT_ON_TX_EMPTY | \
                                            spi_INT_ON_TX_NOT_FULL | spi_INT_ON_RX_FULL | \
                                            spi_INT_ON_RX_NOT_EMPTY | spi_INT_ON_RX_OVER | \
                                            spi_INT_ON_BYTE_COMP)


/***************************************
*             Registers
***************************************/

#define spi_TXDATA_REG         (* (reg8 *) \
        spi_BSPIM_sR8_Dp_u0__F0_REG)
#define spi_TXDATA_PTR         (  (reg8 *) \
        spi_BSPIM_sR8_Dp_u0__F0_REG)
#define spi_RXDATA_REG         (* (reg8 *) \
        spi_BSPIM_sR8_Dp_u0__F1_REG)
#define spi_RXDATA_PTR         (  (reg8 *) \
        spi_BSPIM_sR8_Dp_u0__F1_REG)

#define spi_AUX_CONTROL_DP0_REG (* (reg8 *) \
        spi_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)
#define spi_AUX_CONTROL_DP0_PTR (  (reg8 *) \
        spi_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)        

#if (spi_DataWidth > 8u)

    #define spi_AUX_CONTROL_DP1_REG  (* (reg8 *) \
            spi_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define spi_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
            spi_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
#endif /* spi_DataWidth > 8u */

#define spi_COUNTER_PERIOD_REG  (* (reg8 *)  spi_BSPIM_BitCounter__PERIOD_REG)
#define spi_COUNTER_PERIOD_PTR  (  (reg8 *)  spi_BSPIM_BitCounter__PERIOD_REG)
#define spi_COUNTER_CONTROL_REG (* (reg8 *)  spi_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define spi_COUNTER_CONTROL_PTR (  (reg8 *)  spi_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define spi_TX_STATUS_REG       (* (reg8 *)  spi_BSPIM_TxStsReg__STATUS_REG)
#define spi_TX_STATUS_PTR       (  (reg8 *)  spi_BSPIM_TxStsReg__STATUS_REG)
#define spi_RX_STATUS_REG       (* (reg8 *)  spi_BSPIM_RxStsReg__STATUS_REG)
#define spi_RX_STATUS_PTR       (  (reg8 *)  spi_BSPIM_RxStsReg__STATUS_REG)

#define spi_CONTROL_REG         (* (reg8 *)  \
        spi_BSPIM_BidirMode_AsyncCtl_CtrlReg__CONTROL_REG)
#define spi_CONTROL_PTR         (  (reg8 *)  \
        spi_BSPIM_BidirMode_AsyncCtl_CtrlReg__CONTROL_REG)
        
#define spi_TX_STATUS_MASK_REG  (* (reg8 *)  spi_BSPIM_TxStsReg__MASK_REG)
#define spi_TX_STATUS_MASK_PTR  (  (reg8 *)  spi_BSPIM_TxStsReg__MASK_REG)
#define spi_RX_STATUS_MASK_REG  (* (reg8 *)  spi_BSPIM_RxStsReg__MASK_REG)
#define spi_RX_STATUS_MASK_PTR  (  (reg8 *)  spi_BSPIM_RxStsReg__MASK_REG)

#define spi_TX_STATUS_ACTL_REG  (* (reg8 *)  spi_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define spi_TX_STATUS_ACTL_PTR  (  (reg8 *)  spi_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define spi_RX_STATUS_ACTL_REG  (* (reg8 *)  spi_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define spi_RX_STATUS_ACTL_PTR  (  (reg8 *)  spi_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

/* Obsolete register names. Not to be used in new designs */
#define spi_TXDATA                 (spi_TXDATA_REG)
#define spi_RXDATA                 (spi_RXDATA_REG)
#define spi_AUX_CONTROLDP0         (spi_AUX_CONTROL_DP0_REG)
#define spi_TXBUFFERREAD           (spi_txBufferRead)
#define spi_TXBUFFERWRITE          (spi_txBufferWrite)
#define spi_RXBUFFERREAD           (spi_rxBufferRead)
#define spi_RXBUFFERWRITE          (spi_rxBufferWrite)

#if(spi_DataWidth > 8u)    
    #define spi_AUX_CONTROLDP1     (spi_AUX_CONTROL_DP1_REG)    
#endif /* spi_DataWidth > 8u */
    
#define spi_COUNTER_PERIOD         (spi_COUNTER_PERIOD_REG)
#define spi_COUNTER_CONTROL        (spi_COUNTER_CONTROL_REG)
#define spi_STATUS                 (spi_TX_STATUS_REG)
#define spi_CONTROL                (spi_CONTROL_REG)
#define spi_STATUS_MASK            (spi_TX_STATUS_MASK_REG)
#define spi_STATUS_ACTL            (spi_TX_STATUS_ACTL_REG)


/***************************************
*       Register Constants 
***************************************/

/* Status Register Definitions */
#define spi_STS_SPI_DONE_SHIFT          (0x00u)
#define spi_STS_TX_FIFO_EMPTY_SHIFT     (0x01u)
#define spi_STS_TX_FIFO_NOT_FULL_SHIFT  (0x02u)
#define spi_STS_BYTE_COMPLETE_SHIFT     (0x03u)
#define spi_STS_SPI_IDLE_SHIFT          (0x04u)
#define spi_STS_RX_FIFO_FULL_SHIFT      (0x04u)
#define spi_STS_RX_FIFO_NOT_EMPTY_SHIFT (0x05u)
#define spi_STS_RX_FIFO_OVERRUN_SHIFT   (0x06u)

#define spi_STS_SPI_DONE                (0x01u << spi_STS_SPI_DONE_SHIFT)        
#define spi_STS_TX_FIFO_EMPTY           (0x01u << spi_STS_TX_FIFO_EMPTY_SHIFT)    
#define spi_STS_TX_FIFO_NOT_FULL        (0x01u << spi_STS_TX_FIFO_NOT_FULL_SHIFT)    
#define spi_STS_SPI_IDLE                (0x01u << spi_STS_SPI_IDLE_SHIFT)
#define spi_STS_RX_FIFO_FULL            (0x01u << spi_STS_RX_FIFO_FULL_SHIFT)    
#define spi_STS_RX_FIFO_NOT_EMPTY       (0x01u << spi_STS_RX_FIFO_NOT_EMPTY_SHIFT)    
#define spi_STS_RX_FIFO_OVERRUN         (0x01u << spi_STS_RX_FIFO_OVERRUN_SHIFT)  
#define spi_STS_BYTE_COMPLETE           (0x01u << spi_STS_BYTE_COMPLETE_SHIFT)

#define spi_TX_STS_CLR_ON_RD_BYTES_MASK (0x09u)
#define spi_RX_STS_CLR_ON_RD_BYTES_MASK (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define spi_INT_ENABLE                  (0x10u)
#define spi_FIFO_CLR                    (0x03u)
                                                                 
/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define spi_CNTR_ENABLE                 (0x20u)   
                                                                  
/* Bi-Directional mode control bit */
#define spi_CTRL_TX_SIGNAL_EN           (0x01u)

/* Datapath Auxillary Control Register definitions */
#define spi_AUX_CTRL_FIFO0_CLR          (0x01u)
#define spi_AUX_CTRL_FIFO1_CLR          (0x02u)
#define spi_AUX_CTRL_FIFO0_LVL          (0x04u)
#define spi_AUX_CTRL_FIFO1_LVL          (0x08u)
#define spi_STATUS_ACTL_INT_EN_MASK     (0x10u)

#endif  /* CY_SPIM_spi_H */


/* [] END OF FILE */
