/*******************************************************************************
* File Name: spi_INT.c
* Version 2.10
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "spi.h"

#if (spi_RXBUFFERSIZE > 4u)

    extern volatile uint8 spi_RXBUFFER[];
    extern volatile uint8 spi_rxBufferRead;
    extern volatile uint8 spi_rxBufferWrite;
    extern volatile uint8 spi_rxBufferFull;
    
#endif /* spi_RXBUFFERSIZE > 4u */

#if (spi_TXBUFFERSIZE > 4u)

    extern volatile uint8 spi_TXBUFFER[];
    extern volatile uint8 spi_txBufferRead;
    extern volatile uint8 spi_txBufferWrite;
    extern volatile uint8 spi_txBufferFull;

#endif /* spi_TXBUFFERSIZE > 4u */

volatile uint8 spi_swStatusTx = 0u;
volatile uint8 spi_swStatusRx = 0u;

/* User code required at start of ISR */
/* `#START spi_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: spi_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  spi_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  spi_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to 
*  sending and FIFO Not Full.
*  spi_TXBUFFER[spi_TXBUFFERSIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(spi_TX_ISR)
{     
    /* User code required at start of ISR */
    /* `#START spi_TX_ISR_START` */

    /* `#END` */
    
    #if((spi_InternalTxInterruptEnabled) && (spi_TXBUFFERSIZE > 4u))
                         
        /* See if TX data buffer is not empty and there is space in TX FIFO */
        while(spi_txBufferRead != spi_txBufferWrite)
        {
            spi_swStatusTx = spi_GET_STATUS_TX(spi_swStatusTx);
            
            if ((spi_swStatusTx & spi_STS_TX_FIFO_NOT_FULL) != 0u)
            {            
                if(spi_txBufferFull == 0u)
                {
                   spi_txBufferRead++;

                    if(spi_txBufferRead >= spi_TXBUFFERSIZE)
                    {
                        spi_txBufferRead = 0u;
                    }
                }
                else
                {
                    spi_txBufferFull = 0u;
                }
            
                /* Move data from the Buffer to the FIFO */
                CY_SET_REG8(spi_TXDATA_PTR,
                    spi_TXBUFFER[spi_txBufferRead]);
            }
            else
            {
                break;
            }            
        }
            
        /* Disable Interrupt on TX_fifo_not_empty if BUFFER is empty */
        if(spi_txBufferRead == spi_txBufferWrite)
        {
            spi_TX_STATUS_MASK_REG  &= ~spi_STS_TX_FIFO_NOT_FULL; 
        }                       
        
	#endif /* spi_InternalTxInterruptEnabled && (spi_TXBUFFERSIZE > 4u) */
    
    /* User code required at end of ISR (Optional) */
    /* `#START spi_TX_ISR_END` */

    /* `#END` */
    
    #ifdef spi_TxInternalInterrupt__ES2_PATCH
        #if(spi_PSOC3_ES2 && (spi_TxInternalInterrupt__ES2_PATCH))
            spi_TX_ISR_PATCH();
        #endif /* End PSOC3_ES2 */
    #endif /* spi_TxInternalInterrupt__ES2_PATCH */    
}


/*******************************************************************************
* Function Name: spi_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  spi_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  spi_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  spi_RXBUFFER[spi_RXBUFFERSIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(spi_RX_ISR)
{     
    #if((spi_InternalRxInterruptEnabled) && (spi_RXBUFFERSIZE > 4u))
        uint8 rxData = 0u; 
    #endif /* spi_InternalRxInterruptEnabled */  
    
    /* User code required at start of ISR */
    /* `#START spi_RX_ISR_START` */

    /* `#END` */
    
    #if((spi_InternalRxInterruptEnabled) && (spi_RXBUFFERSIZE > 4u))
         
        spi_swStatusRx = spi_GET_STATUS_RX(spi_swStatusRx);          
        
        /* See if RX data FIFO has some data and if it can be moved to the RX Buffer */
        while((spi_swStatusRx & spi_STS_RX_FIFO_NOT_EMPTY) == 
                                                                                spi_STS_RX_FIFO_NOT_EMPTY)
        {
            rxData = CY_GET_REG8(spi_RXDATA_PTR);
            
            /* Set next pointer. */
            spi_rxBufferWrite++;
            if(spi_rxBufferWrite >= spi_RXBUFFERSIZE)
            {
                spi_rxBufferWrite = 0u;
            }
            
            if(spi_rxBufferWrite == spi_rxBufferRead)
            {
                spi_rxBufferRead++;
                if(spi_rxBufferRead >= spi_RXBUFFERSIZE)
                {
                    spi_rxBufferRead = 0u;
                }
                spi_rxBufferFull = 1u;
            }
            
            /* Move data from the FIFO to the Buffer */
            spi_RXBUFFER[spi_rxBufferWrite] = rxData;
                
            spi_swStatusRx = spi_GET_STATUS_RX(spi_swStatusRx);
        }                    
        
	#endif /* spi_InternalRxInterruptEnabled  && (spi_RXBUFFERSIZE > 4u) */        
    
    /* User code required at end of ISR (Optional) */
    /* `#START spi_RX_ISR_END` */

    /* `#END` */
    
    #ifdef spi_RxInternalInterrupt__ES2_PATCH
        #if(spi_PSOC3_ES2 && (spi_RxInternalInterrupt__ES2_PATCH))
            spi_RX_ISR_PATCH();
        #endif /* End PSOC3_ES2 */  
    #endif /* spi_RxInternalInterrupt__ES2_PATCH */       
}

/* [] END OF FILE */
