/*******************************************************************************
* File Name: spi_PM.c
* Version 2.10
*
* Description:
*  This file contains the setup, control and status commands to support 
*  component operations in low power mode.  
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "spi.h"

static spi_BACKUP_STRUCT spi_backup = {
                                        0u,
                                        spi_BITCTR_INIT,
                                        #if(spi_PSOC3_ES2 || spi_PSOC5_ES1)
                                            spi_TX_INIT_INTERRUPTS_MASK,
                                            spi_RX_INIT_INTERRUPTS_MASK
                                        #endif /* spi_PSOC3_ES2 || spi_PSOC5_ES1 */
                                        };

#if (spi_TXBUFFERSIZE > 4u)

    extern volatile uint8 spi_txBufferRead;
    extern volatile uint8 spi_txBufferWrite;
    
#endif /* spi_TXBUFFERSIZE > 4u */

#if (spi_RXBUFFERSIZE > 4u)

    extern volatile uint8 spi_rxBufferRead;
    extern volatile uint8 spi_rxBufferWrite;
    
#endif /* spi_RXBUFFERSIZE > 4u */


/*******************************************************************************
* Function Name: spi_SaveConfig
********************************************************************************
*
* Summary:
*  Saves SPIM configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
* 
* Global Variables:
*  spi_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void spi_SaveConfig(void)
{
    /* Store Status Mask registers */
    #if (spi_PSOC3_ES2 || spi_PSOC5_ES1)     
    
       spi_backup.saveSrTxIntMask = spi_TX_STATUS_MASK_REG;
       spi_backup.saveSrRxIntMask = spi_RX_STATUS_MASK_REG;
       spi_backup.cntrPeriod = spi_COUNTER_PERIOD_REG;
       
    #endif /* (spi_PSOC3_ES2 || spi_PSOC5_ES1) */
}


/*******************************************************************************
* Function Name: spi_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores SPIM configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  spi_backup - used when non-retention registers are restored.
*
* Side Effects:
*  If this API is called without first calling SaveConfig then in the following
*  registers will be default values from Customizer: 
*  spi_STATUS_MASK_REG and spi_COUNTER_PERIOD_REG.
*
*******************************************************************************/
void spi_RestoreConfig(void) 
{
    /* Restore the data, saved by SaveConfig() function */
    #if (spi_PSOC3_ES2 || spi_PSOC5_ES1)
    
        spi_TX_STATUS_MASK_REG = spi_backup.saveSrTxIntMask;
        spi_RX_STATUS_MASK_REG = spi_backup.saveSrRxIntMask;
        spi_COUNTER_PERIOD_REG = spi_backup.cntrPeriod;
        
    #endif /* (spi_PSOC3_ES2 || spi_PSOC5_ES1) */
}


/*******************************************************************************
* Function Name: spi_Sleep
********************************************************************************
*
* Summary:
*  Prepare SPIM Component goes to sleep.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  spi_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void spi_Sleep(void)
{
    /* Save components enable state */
    if ((spi_TX_STATUS_ACTL_REG & spi_INT_ENABLE) == spi_INT_ENABLE)
    {
        spi_backup.enableState = 1u;
    }
    else /* Components block is disabled */
    {
        spi_backup.enableState = 0u;
    }

    spi_Stop();

    spi_SaveConfig();
}


/*******************************************************************************
* Function Name: spi_Wakeup
********************************************************************************
*
* Summary:
*  Prepare SPIM Component to wake up.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  spi_backup - used when non-retention registers are restored.
*  spi_txBufferWrite - modified every function call - resets to 
*  zero.
*  spi_txBufferRead - modified every function call - resets to 
*  zero.
*  spi_rxBufferWrite - modified every function call - resets to
*  zero.
*  spi_rxBufferRead - modified every function call - resets to
*  zero. 
*
* Reentrant:
*  No.
*
*******************************************************************************/
void spi_Wakeup(void)
{        
    spi_RestoreConfig();
         
    #if (spi_TXBUFFERSIZE > 4u)
    
        spi_txBufferRead = 0u;
        spi_txBufferWrite = 0u;
        
    #endif /* spi_TXBUFFERSIZE > 4u */
    
    #if (spi_RXBUFFERSIZE > 4u)    
    
        spi_rxBufferRead = 0u;
        spi_rxBufferWrite = 0u;
        
    #endif /* spi_RXBUFFERSIZE > 4u */ 
    
    spi_ClearFIFO();
    
    /* Restore components block enable state */
    if (spi_backup.enableState != 0u)
    {
         /* Components block was enabled */
         spi_Enable();
    } /* Do nothing if components block was disabled */
}


/* [] END OF FILE */
