#include <math.h>
#include <device.h>
#include <stdio.h>
#include <string.h>
#include "hardware.h"
#include "display.h"
#include "adc.h"
#include "sdcard.h"
#include "log.h"

#define MAX_LOGGING_SIZE 1024

uint8_t g_logging = 0;
float g_loggingData[CHANNEL_COUNT][MAX_LOGGING_SIZE];
uint8_t g_startTriggered = 0;
uint8_t g_stopTriggered = 0;
uint32_t g_halfSeconds = 0;
uint32_t g_seconds = 0;
uint32_t g_milliseconds = 0;
uint32_t g_milliseconds2 = 0;
LogEntry g_logEntry;
char g_textBuffer[512];

void updateTimestamp(void)
{
	if (g_milliseconds > 0) {
		g_milliseconds--;
	} else {
		g_milliseconds = 1000;
		g_seconds++;
	}

	if (g_milliseconds2 > 0) {
		g_milliseconds2--;
	} else {
		g_milliseconds2 = 500;
		g_halfSeconds++;
	}
}

void onMillisecond()
{
	// scan all ADC inputs
	scanInputs();

	// update CapSense scanning
	updateCapSense();
	
	// update timestamp
	updateTimestamp();
}

void testReadWrite(void)
{
	uint8_t buffer[512];
	int i;
	#if 0
	for (i = 0; i < 512; i++) {
		buffer[i] = 512-i;
	}
	if (sdCardWriteBlock(0, buffer)) {
		rs232_PutString((uint8*) "write block error\r\n");
		return;
	}
	#endif
	for (i = 0; i < 512; i++) {
		buffer[i] = 0;
	}
	if (sdCardReadBlock(0, buffer)) {
		rs232_PutString((uint8*) "read block error\r\n");
		return;
	}
	#if 0
	for (i = 0; i < 512; i++) {
		if ((i % 16) == 0) rs232_PutString((uint8*) "\r\n");
		sprintf(g_textBuffer, "%02x ", buffer[i]);
		rs232_PutString((uint8*) g_textBuffer);
	}
	rs232_PutString((uint8*) "\r\n");
	#endif
}

void drawLine(int y, const char* text)
{
	char buf[12];
	int i;
	for (i = 0; i < 11; i++) buf[i] = 32;
	buf[11] = 0;
	i = 0;
	while (*text) {
		buf[i] = *text;
		if (i == 10) break;
		i++;
		text++;
		if (*text == 0) break;
	}
	drawString(40, y, COLOR_WHITE, 1, buf);
}

void readInputs(void)
{
	g_logEntry.timestamp = g_halfSeconds * 500;
	uint8_t channel;
	for (channel = 0; channel < CHANNEL_COUNT; channel++) {
		g_logEntry.channel[channel] = getVoltage(channel);
	}
	g_logEntry.digitalIn = 0;
	g_logEntry.digitalIn |= gpio0_Read() ? 1 : 0;
	g_logEntry.digitalIn |= gpio1_Read() ? 2 : 0;
	g_logEntry.temperature = getTemperature();
}

void createTextOutputFromLogEntry(void)
{
	uint8_t channel;
	char buf[256];
	sprintf(g_textBuffer, "%i\t", (int) g_logEntry.timestamp);
	for (channel = 0; channel < CHANNEL_COUNT; channel++) {
		sprintf(buf, "%+.5f\t", g_logEntry.channel[channel]);
		strcat(g_textBuffer, buf);
	}
	sprintf(buf, "%i\t%i\t%.1f\r\n",
		(g_logEntry.digitalIn & 1) ? 1 : 0,
		(g_logEntry.digitalIn & 2) ? 1 : 0,
		g_logEntry.temperature);
	strcat(g_textBuffer, buf);
}

void displayLogEntry(void)
{
		uint8_t channel;
		for (channel = 0; channel < CHANNEL_COUNT; channel++) {
			float voltage = g_logEntry.channel[channel];
			float vabs = fabs(voltage);
			if (vabs < 0.1f) {
				sprintf(g_textBuffer, "%+.1f mV", voltage * 1000.0f);
			} else if (vabs < 1.0f) {
				sprintf(g_textBuffer, "%+.0f mV", voltage * 1000.0f);
			} else {
				sprintf(g_textBuffer, "%+.2f V", voltage);
			}
			drawLine(60 + 12 * channel, g_textBuffer);
		}
		
		// output digital inputs
		sprintf(g_textBuffer, "i0: %i i1: %i",
			(g_logEntry.digitalIn & 1) ? 1 : 0,
			(g_logEntry.digitalIn & 2) ? 1 : 0);
		drawLine(84, g_textBuffer);

		// temperature
		sprintf(g_textBuffer, "%.1f C", g_logEntry.temperature);
		drawLine(96, g_textBuffer);
}

void main()
{
	// last button state
	uint8_t lastStartButton = 0;
	uint8_t lastStopButton = 0;
	uint32_t lastHalfSeconds = 0;
	
	// init hardware
	initHardware();
	rs232_PutString((uint8*) "MultiLogger V 0.1\r\n");
	rs232_PutString((uint8*) "hardware initialized\r\n");
	
	// detect SD card
	if (sdCardInit()) {
		rs232_PutString((uint8*) "SD card not detected\r\n");
	} else {
		rs232_PutString((uint8*) "SD card detected\r\n");
		testReadWrite();
	}
	
	// init display
	initDisplay();
	rs232_PutString((uint8*) "display initialized\r\n");
	
	// draw static parts
	drawString(24, 0, COLOR_YELLOW, 2, "Multi");
	drawString(16, 26, COLOR_YELLOW, 2, "Logger");
	drawString(0, 60, COLOR_LIGHT_SEA_GREEN, 1, "Ch0:");
	drawString(0, 72, COLOR_LIGHT_SEA_GREEN, 1, "Ch1:");
	drawString(0, 84, COLOR_LIGHT_SEA_GREEN, 1, "PIO:");
	drawString(0, 96, COLOR_LIGHT_SEA_GREEN, 1, "Tem:");
	drawString(0, 108, COLOR_LIGHT_SEA_GREEN, 1, "Log:");

	// main loop
	while (1) {
		// two measurements per second
		while (g_halfSeconds == lastHalfSeconds) {}
		lastHalfSeconds = g_halfSeconds;
		g_conversionFinished = 0;
		
		// read all inputs in g_logEntry
		readInputs();
		
		// show on OLED display
		displayLogEntry();
		
		// create text line with all values and print on RS232
#ifdef CALIBRATION
		int i;
		uint8_t channel;
		for (channel = 0; channel < CHANNEL_COUNT; channel++) {
			uint8_t divider;
			for (divider = 0; divider < DIVIDER_COUNT; divider++) {
				float filtered = getFiltered(channel, divider);
				sprintf(g_textBuffer, "%i\t", (int) filtered);
				rs232_PutString((uint8*) g_textBuffer);
			}
		}
		rs232_PutString((uint8*) "\r\n");
#endif
		createTextOutputFromLogEntry();
		rs232_PutString((uint8*) g_textBuffer);

		// if logging, save next entry
		if (g_logging) {
			g_logEntry.magic = VALID_MAGIC;
			addEntry(&g_logEntry);
		}

		// logging status
		if (g_logging) {
			sprintf(g_textBuffer, "%u s", (unsigned int) g_seconds);
			drawLine(108, g_textBuffer);
		} else {
			drawLine(108, "stopped");
		}

		// test for start logging
		if (getStartButton() && !lastStartButton) {
			if (!g_logging) {
				g_logging = 1;
				resetLogPointer();
				g_milliseconds = 0;
				g_milliseconds2 = 0;
				g_seconds = 0;
				g_halfSeconds = 0;
				lastHalfSeconds = 0;
				rs232_PutString((uint8*) "\r\nlogging started\r\n");
			}
		}
		
		// dump logged data on stop button
		if (getStopButton() && !lastStopButton) {
			drawLine(108, "transfer...");
			rs232_PutString((uint8*) "\r\ntransferring logged data:\r\n");
			g_logging = 0;
			resetLogPointer();
			while (1) {
				int result = readNextEntry(&g_logEntry);
				if (result) break;
				createTextOutputFromLogEntry();
				rs232_PutString((uint8*) g_textBuffer);
			}
			rs232_PutString((uint8*) "dump end\r\n\r\n");
		}

		lastStartButton = getStartButton();
		lastStopButton = getStopButton();
	}
}
