// SD Card accessing functions

#include <stdio.h>

#include "hardware.h"
#include "sdcard.h"

// last received bytes from sdCardSendCommand
static uint8_t g_sdCardResponse[16];

// sends 'count' times 0xff
static void sdCardDelay(int count)
{
	int i;
	for (i = 0; i < count; i++) sdCardTransmit(0xff);
}

// read response
// returns 0, if ok, 1 on timeout
static int sdCardReadCommandResponse(int responseLength)
{
	// read first byte (MSB)
	int timeout = 1000;
	int tmp;
	while (timeout) {
		tmp = sdCardTransmit(0xff);
		if ((tmp & 0x80) == 0) break;
		timeout--;
	}
	
	// test for timeout
	if (timeout == 0) return 1;
	
	// read rest
	while (responseLength > 0) {
		responseLength--;
		g_sdCardResponse[responseLength] = tmp;
		tmp = sdCardTransmit(0xff);
	}
	
	// return ok
	return 0;
}

// send command header and CRC
static void sdCardSendCommandHeader(uint8_t command, uint32_t arg)
{
	// header/command
    sdCardTransmit((command & 0x3f) | 0x40);
	
	// argument
    sdCardTransmit((arg >> 24) & 0xff);
    sdCardTransmit((arg >> 16) & 0xff);
    sdCardTransmit((arg >> 8) & 0xff);
    sdCardTransmit(arg & 0xff);
	
	// CRC is needed for IDLE command, only
	if (command == 0) {
	    sdCardTransmit(0x95);
	} else {
	    sdCardTransmit(0xff);
	}
}

// send command
// returns 0, if ok, 1 on timeout
static int sdCardSendCommand(int command, uint32_t arg, int responseLength)
{
    sdCardSelect();
	sdCardSendCommandHeader(command, arg);
	int result = sdCardReadCommandResponse(responseLength);
    sdCardDeselect();
	return result;
}

int sdCardInit(void)
{
	// sync
	sdCardSelect();
	sdCardDelay(100);
	sdCardDeselect();
	sdCardDelay(10);
	
	// send CMD0: idle state
	sdCardSendCommand(0, 0, 1);
	
	// wait until idle
	int timeout = 1000;
	while (timeout) {
		sdCardSendCommand(55, 0, 1);
		sdCardSendCommand(41, 0, 1);
		if ((g_sdCardResponse[0] & 1) == 0) break;
		timeout--;
	}
	if (timeout == 0) return 0;
	
	// set block length
	int result = sdCardSendCommand(16, SD_BLOCK_LENGTH, 1);

	sdCardDelay(10);

	return result;
}

int sdCardReadBlock(uint32_t blockNumber, uint8_t* buffer)
{
    // send block-read command
    sdCardSendCommand(17, blockNumber << SD_BLOCK_LENGTH_SHIFT, 1);
	if (g_sdCardResponse[0] != 0) return g_sdCardResponse[0];

	// select card and wait for token
	sdCardSelect();
	int timeout = 1000;
	while (timeout) {
		if (sdCardTransmit(0xff) != 0xff) break;
		timeout--;
	}
	if (timeout == 0) {
	    sdCardDeselect();
		return 1;
	}

    // read the block
	int i;
    for (i = 0; i < SD_BLOCK_LENGTH; i++) buffer[i] = sdCardTransmit(0xff);
    
    // ignore the checksum
    sdCardTransmit(0xFF);          
    sdCardTransmit(0xFF);
    
    // delect card
    sdCardDeselect();
	
	sdCardDelay(10);

    return 0;
}

int sdCardWriteBlock(uint32_t blockNumber, uint8_t* buffer)
{
    // send block-write command
    sdCardSendCommand(24, blockNumber << SD_BLOCK_LENGTH_SHIFT, 1);

	// select card and one byte delay
    sdCardSelect();
	sdCardDelay(1);
	
    // send start block token
    sdCardTransmit(0xfe);

    // write block
	int i;
    for (i = 0; i < SD_BLOCK_LENGTH; i++) sdCardTransmit(buffer[i]);

	// ignore the checksum
    sdCardTransmit(0xFF);          
    sdCardTransmit(0xFF);
    
    // wait for response token
	int timeout = 1000;
	while (timeout) {
		if (sdCardTransmit(0xff) == 0xff) break;
		timeout--;
	}
	int result = 0;
	if (timeout == 0) result = 1;
    
    // one byte delay for the card to finish the write and deselect
	sdCardDelay(1);
    sdCardDeselect();
	
	// another two bytes for resynchonize?
	sdCardDelay(10);

	return result;
}
